TITLE    SUBROUTINE SOUND(IFREQ,IDUR)
;CAUSES THE PC'S SPEAKER TO PRODUCE A SOUND OF A SPECIFIED FREQUENCY
;AND TIME DURATION.  THE FREQUENCY UNITS ARE HERTZ AND THE DURATION
;UNITS ARE IN HUNDREDTHS OF A SECOND (CENTISECONDS).  THE LOWEST
;ACCEPTABLE FREQUENCY IS 21 HERTZ AND THE MAXIMUM AUDIBLE FREQUENCY
;IS ABOUT 20000 HERTZ.
;
;ADAPTED BY W.D. GRUNDY   JANUARY 1985
;
;REFERENCE: Scanlon, Leo J.,1983; IBM PC assembly language, a guide
;           for programmers, Robert J. Brady Co.,Bowie, Maryland,
;           page 270.
;
;
;FORTRAN USAGE
;     INTEGER*2 IFREQ,IDUR
;     ...........
;     ...........
;     CALL SOUND(IFREQ,IDUR)
;
FRAME    STRUC
SAVESI   DW       ?
SAVEDI   DW       ?
SAVEDS   DW       ?
SAVEBP   DW       ?                ;SAVE AREA FOR CALLER'S BP.
SAVERET  DD       ?                ;SAVE AREA FOR FAR RETURN.
B        DD       ?                ;WORD ADDRESS FOR DURATION.
A        DD       ?                ;WORD ADDRESS FOR FREQUENCY.
FRAME    ENDS
;
_DATA    SEGMENT  'DATA'
_DATA    ENDS
;
SND_TEXT SEGMENT  'CODE'
DGROUP   GROUP    _DATA
         ASSUME   CS:SND_TEXT,DS:DGROUP,SS:DGROUP
         PUBLIC   SOUND
SOUND    PROC     FAR
         PUSH     BP               ;SAVE CALLER'S BP
         PUSH     DS
         PUSH     DI
         PUSH     SI
         MOV      BP,SP
         LES      BX,[BP]+A        ;GET ADDRESS FOR FREQUENCY, AND
         MOV      DI,ES:[BX]       ;PUT FREQUENCY IN HERTZ INTO DI.
         CMP      DI,21            ;FREQUENCY TOO LOW?
         JL       RETURN           ;YES, QUIT IMMEDIATELY.
         LES      BX,[BP]+B        ;GET ADDRESS FOR DURATION, AND
         MOV      BX,ES:[BX]       ;PUT CENTISECONDS DURATION IN BX.
         MOV      AL,0B6H          ;WRITE TIMER MODE REGISTER
         OUT      43H,AL
         MOV      DX,14H           ;TIMER DIVISOR=
         MOV      AX,4F38H         ;1331000/FREQUENCY
         DIV      DI
         OUT      42H,AL           ;WRITE TIMER 2 COUNT LOW BYTE.
         MOV      AL,AH
         OUT      42H,AL           ;WRITE TIMER 2 COUNT HIGH BYTE.
         IN       AL,61H           ;GET CURRENT PORT B SETTING
         MOV      AH,AL            ;AND SAVE IT IN AH.
         OR       AL,3             ;TURN SPEAKER ON.
         OUT      61H,AL
WAIT:    MOV      CX,2801          ;WAIT 10 MILLISECONDS
SPKR_ON: LOOP     SPKR_ON          ;NOTE: ABOVE CONSTANT FOR 4.77 mHERTZ
         DEC      BX               ;SPEAKER ON COUNT EXPIRED?
         JG       WAIT             ;IF NOT, KEEP SPEAKER ON.
         MOV      AL,AH            ;OTHERWISE RECOVER VALUE OF PORT.
         OUT      61H,AL
RETURN:                            ;BRANCH TARGET FOR FREQ TOO LOW.
         POP      SI
         POP      DI
         POP      DS
         POP      BP               ;RESTORE CALLER'S BP.
         RET      08H              ;FAR RETURN.  RELEASE 8 BYTES.
SOUND    ENDP
SND_TEXT ENDS
         END
