$NOFLOATCALLS
$STORAGE:2
      PROGRAM SS2DBLOK
C -----------------------------------------------------------------------
C***  PROGRAM COMPUTES MEAN VALUE OF A REGIONALIZED VARIABLE IN A
C***  BLOCK OF GROUND.
C***
C***  REFERENCES
C***
C***  Knudsen, H.P., and Kim, Y.C., 1977, A short course on geostatist-
C***      ical ore reserve estimation: College of mines, University of
C***      Arizona, Tucson, Arizona, 202 pages and appendices.
C***
C***  Kim, Y.C., and Knudsen, H.P., 1977, Geostatistical ore reserve
C***      estimation for a roll-front type uranium deposit (practioner's
C***      guide), University of Arizona, Tucson, Arizona, 61 p.
C***      (issued by U.S. Dept. of Energy, Grand Junction Office, Grand
C***      Junction, Colorado, 81501)
C***
C***  Skrivan, J.A., and Karlinger, M.R., 1980, Semi-variogram estima-
C***      tion and universal kriging program, computer contribution,
C***      U.S. Geological Survey, Water Resources Division, Tacoma,
C***      Washington, 98 p.
C***
C***  Journel, A.G., and Huijbregts, Ch. J., 1978, Mining geo-
C***      statistics: Academic Press, New York, 600 p.
C***
C***  David, M., 1977, Geostatistical ore reserve estimation:
C***      Elsevier Scientific Publishing Company, Amsterdam, 364 p.
C***
C***  THIS IS A TWO DIMENSIONAL BLOCK-KRIGING PROGRAM.
C***  PROGRAM COMPUTES KRIGED MEAN VALUE AND THE KRIGING
C***  VARIANCE OF A STATPAC VARIABLE IN A SINGLE BLOCK OF
C***  ANY SPECIFIED SHAPE.
C***  THIS PROGRAM HAS A CHOICE OF UNIVERSAL OR ORDINARY KRIGING AND
C***  ALLOWS THE USER TO SPECIFY THE NEIGHBORHOOD SIZE.  FIVE
C***  VARIOGRAM MODELS, THE SPHERICAL, EXPONENTIAL, LINEAR,
C***  GAUSSIAN, AND CUBIC ARE INCLUDED.  AS MANY AS FIVE NESTED
C***  VARIOGRAMS MAY BE USED.  ANISOTROPIES, IF PRESENT,
C***  MUST BE THE GEOMETRIC TYPE.
C***  PROGRAM ADAPTED BY W.D. GRUNDY FROM PROGRAMS ESTVAR AND
C***  UKRIG WRITTEN BY H.P. KNUDSEN OF THE UNIVERSITY OF
C***  ARIZONA, AND THE KRIGING PACKAGE OF SKRIVAN AND KARLINGER.
C***  THE ALGORITHM FOR COMPUTING BLOCK VARIANCE IS A COMBINATION
C***  OF ALGORITHMS PUBLISHED BY JOURNEL AND HUIJBREGTS AND BY DAVID.
C***
C***  LINKS REQUIRED: AREA, ASCIIZ, BYTMOV, CBAR4, CHEKER, CHRKNT,
C***                  CKSAFE, CONCAT, COVAR4, CTROID, DELAY, DOLITL,
C***                  DOSREN, DRFTRM, DSORT, ERRMSG, FNDCUR, FSTDSP,
C***                  GETFLE, GRDPTS, GRIDER, HLSRCH, IRREG, KRGOT4,
C***                  KRIG4B, MAXCOR, MNVAR, MOVCUR, NAMFIL, NEIGH,
C***                  ODOMTR, PCHCEN, PSET4, PTLOC, RAND, SAVRST,
C***                  SCREAD, SOUND, TERM, TICKER, UNPK, VARIN4,
C***                  VRBLK4
C***  NOTE: ABOVE OBJECT MODULES ARE CONTAINED IN STATPAC.LIB AND
C***        NEED NOT BE RECOMPILED OR REASSEMBLED.
C***        COMPILE SS2DBLOK WITH FILE BKKRIG.INC PRESENT.
C***  NOTE: FSTDSP WRITES TO VIDEO RAM
C***
C***  PROGRAM REQUIRES AN INPUT STATPAC DATA FILE.  THE INPUT FILE
C***  MUST CONTAIN CARTESIAN X- AND Y-COORDINATES (EASTINGS AND NORTH-
C***  INGS) AND AN OBSERVED VALUE OF A REGIONALIZED VARIABLE.
C***
C***  FILE NAME REQUIREMENTS
C***       THE STATPAC INPUT FILE MUST HAVE THE EXTENSION .UPR
C***       THE CHARACTER OUTPUT FILE IS AUTOMATICALLY GIVEN THE
C***       EXTENSION .FBL
C***       THE OPTIONAL CHARACTER BOUNDARY FILE MUST HAVE THE
C***       EXTENSION .FDI
C***
C***  COORDINATES OF THE BLOCK NODES MAY BE ENTERED
C***  MANUALLY FROM THE TERMINAL OR THEY MAY BE ENTERED
C***  FROM A STORED CHARACTER FILE.  THE BLOCK NODES ARE
C***  ENTERED WITH THE NORTHING FIRST AND THE EASTING SECOND.
C***  THEY ARE ENTERED IN A CLOCKWISE DIRECTION.
C***
C***  PROGRAM CREATES A CHARACTER OUTPUT FILE CONTAINING THE VARIOGRAM/
C***  DRIFT PARAMETERS USED, THE COMPUTED AREA OF THE KRIGED BLOCK,
C***  THE BLOCK VARIANCE, THE COORDINATES OF THE CENTROID OF THE BLOCK,
C***  AND THE KRIGED MEAN VALUE OF THE REGIONALIZED VARIABLE WITHIN THE
C***  BLOCK TOGETHER WITH ITS ERROR VARIANCE.  OPTIONALLY, THE OUTPUT
C***  FILE CONTAINS A LISTING OF THE NORTHINGS AND EASTINGS OF THE
C***  NODES DEFINING THE BLOCK, AND A LISTING OF DRILL HOLES USED IN
C***  THE ESTIMATION ALONG WITH THEIR COMPUTED KRIGING WEIGHTS.
C***
C***  MAJOR VARIABLES AND DIMENSION REQUIREMENTS
C***
C***    NEST         NUMBER OF VARIOGRAMS IN MODEL (MAX=5)
C***    C0           NUGGET EFFECT
C***    C(NEST)      C-VALUES OF VARIOGRAMS
C***    AVALU(NEST)  A-VALUES OF VARIOGRAMS
C***                 IF VARIOGRAM IS OF FORM GAM(H)=C0+SLOPE*H**A
C***                 AVALU REPRESENTS THE POWER A.
C***                 FOR CUBIC AND SPHERICAL VARIOGRAMS, AVALU
C***                 REPRESENTS THE VARIOGRAM RANGE.
C***                 FOR EXPLANATION OF THIS PARAMETER FOR GAUSSIAN
C***                 AND EXPONENTIAL VARIOGRAMS, SEE JOURNEL AND
C***                 HUIJBREGTS, 1978, PAGE 164.
C***    ANGLE(NEST)  ANGLES OF ROTATION OF ANISOTROPIC VARIOGRAMS ABOUT
C***                 EAST-WEST AXIS.  EAST IS TAKEN AS ZERO DEGREES.
C***                 ANGLES ARE ENTERED IN DECIMAL DEGREES.
C***                 NEGATIVE ANGLE INDICATES CLOCKWISE ROTATION.
C***                 POSITIVE ANGLE INDICATES COUNTER-CLOCKWISE ROTATION
C***                 IF GEOMETRIC ANISOTROPY IS PRESENT, ANGLE IS THE
C***                 BEARING OF THE MAJOR AXIS OF THE ELLIPSE.
C***    AFH(NEST)    ANISOTROPY RATIOS OF VARIOGRAMS
C***                 AFH.NE.1   INDICATES GEOMETRIC ANISOTROPY
C***                 AFH.EQ.1   INDICATES ISOTROPIC VARIOGRAM
C***    SILL         C0 PLUS SUM OF C'S
C***    ITYPE(NEST) VARIOGRAM TYPE
C***                 =1 INDICATES SPHERICAL VARIOGRAM
C***                 =2 INDICATES EXPONENTIAL VARIOGRAM
C***                 =3 INDICATES LINEAR VARIOGRAM
C***                 =4 INDICATES GAUSSIAN VARIOGRAM
C***                 =5 INDICATES CUBIC VARIOGRAM
C***    AISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    BISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    CISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    SLOPE(NEST)  SLOPES OF LINEAR VARIOGRAMS
C***                 ZERO SLOPE INDICATES VARIO-
C***                 GRAM IS NOT A LINEAR TYPE.
C***
C***    NT           NUMBER OF DRIFT TERMS (MAX=10)
C***    EX(NT)       X-EXPONENTS OF DRIFT TERMS
C***    EY(NT)       Y-EXPONENTS OF DRIFT TERMS
C***
C***    MAXH         MAXIMUM NUMBER OF DRILL HOLES PROGRAM WILL PROCESS.
C***                 MAXH IS CURRENTLY SET TO 1500.
C***    IHOLID(MAXH) ALPHAMERIC DRILL HOLE IDENTIFIERS
C***    XE(MAXH)     ARRAY OF X-COORDINATES (EASTINGS) OF DRILL HOLES
C***    YN(MAXH)     ARRAY OF Y-COORDINATES (NORTHINGS) OF DRILL HOLES
C***    GD(MAXH)     ARRAY OF SAMPLE MEASUREMENTS OF DRILL HOLES
C***    ERMEAS(MAXH) ARRAY OF ERROR VARIANCES OF MEASUREMENTS GD
C***
C***  /COV4/
C***    MAXK         MAXIMUM NUMBER OF HOLES IN NEIGHBORHOOD WHICH CAN
C***                 BE USED IN KRIGING A POINT.  IN THIS PROGRAM MAXK
C***                 IS SET TO 150.
C***    P(KDIM)      COVARIANCE AND DRIFT TERM MATRIX
C***                 KDIM=(MAXK+NT)*(MAXK+NT+1)/2
C***    ID(MAXK+NT)  AN ARRAY OF POINTERS TO DIAGONAL ELEMENTS OF P
C***    W(MAXK+NT)   ARRAY OF KRIGING WEIGHTS AND LAGRANGIAN MULTIPLIERS
C***    IHOL(MAXK+1) ARRAY OF POINTERS TO HOLES TO BE USED IN KRIGING
C***                 A PARTICULAR LOCATION
C***    R(MAXK+NT)   COLUMN VECTOR OF KRIGING SYSTEM
C***
C***    IPT           STATPAC INPUT FILE
C***    IOUT         STATPAC OUTPUT FILE (NOT USED IN PROGRAM KRIGBLK4)
C***    IPR           CHARACTER OUTPUT FILE
C***    NHOL         ACTUAL NUMBER OF DRILL HOLES OBTAINED FROM STATPAC
C***    IVIB(2,99)   ARRAY OF IDENTIFIERS OF STATPAC FILE VARIABLES
C***    ILIST        OPTION FOR PRINTING BLOCK NODES, AND DRILL HOLE
C***                 LISTINGS WITH KRIGING WEIGHTS.
C***                 = 0  DO NOT PRINT NODES OR DRILL HOLE INFORMATION
C***                 = 1  PRINT DRILL HOLE INFORMATION ONLY
C***                 = 2  PRINT BLOCK NODES ONLY
C***                 = 3  PRINT BLOCK NODES AND DRILL HOLE INFORMATION
C***
C***  /CONT2/
C***    IX           NUMBER OF GRID POINTS IN BLOCK IN X-DIRECTION
C***    IY           NUMBER OF GRID POINTS IN BLOCK IN Y-DIRECTION
C***    YMAX         HIGHEST Y-COORDINATE OF THE BLOCK
C***    YMIN         LOWEST  Y-COORDINATE OF THE BLOCK
C***    XMAX         HIGHEST X-COORDINATE OF THE BLOCK
C***    XMIN         LOWEST  X-COORDINATE OF THE BLOCK
C***    PGRID(2,2,
C***    MAXPT)       ARRAY OF GRID POINTS USED IN BLOCK KRIGING.
C***    YNO(MAXN)    NORTH-COORDINATES OF NODES OF THE BLOCK
C***    XEA(MAXN)    EAST-COORDINATES OF NODES OF THE BLOCK
C***    NODES        NUMBER OF NODES (VERTICES) OF THE BLOCK
C***    AR           AREA OF BLOCK IN SQUARE UNITS
C***    IREG         = 1  INDICATES A RECTANGULAR BLOCK WITH SIDES
C***                      PARALLEL TO THE COORDINATE AXES.
C***                 = 2  INDICATES AN IRREGULAR BLOCK.
C***    MAXN         MAXIMUM NUMBER OF NODES OF POLYGONAL BLOCK. THE
C***                 VALUE OF MAXN IS CURRENTLY SET TO 1000.
C***    ISEED        STARTING SEED (INTEGER) FOR RANDOM NUMBER GENERATOR
C***    DSEED        SEED FOR RANDOM NUMBER GENERATOR USED IN KRIGING A
C***                 NON-RECTANGULAR BLOCK
C***    DIST(MAXK+1) ARRAY OF DISTANCES OF DRILL HOLES FROM CENTROID
C***                 OF BLOCK.
C***    NK           MAXIMUM NUMBER OF HOLES TO BE USED
C***                 IN KRIGING A POINT (USER SPECIFIED)
C***    RMAX         SEARCH RADIUS ABOUT CENTROID OF BLOCK
C***    MAXPT        MAXIMUM NUMBER OF POINTS ALLOWED IN THE KRIGING
C***                 GRID.  VALUE OF MAXPT IS CURRENTLY SET TO 1000.
C***
C -----------------------------------------------------------------------
$INCLUDE:'BKKRIG.INC'
      INTEGER*4 ITIME1,ITIME2
      CHARACTER*32 UNIT10,UNIT21
      CHARACTER*4 EXT
      DATA EXT/'.FBL'/
C -----------------------------------------------------------------------
C***
C***
C***  INITIALIZE TIMING OF RUN AND OTHER NECESSARY VARIABLES
C -----------------------------------------------------------------------
      CALL TICKER(ITIME1)
      IPT=10
      IPR=21
C -----------------------------------------------------------------------
C***  IBOUND IS OPTIONAL BOUNDARY FILE.  IF IT IS NOT PRESENT,
C***  COORDINATES OF POLYGONAL NODES WILL HAVE TO BE ENTERED
C***  FROM THE KEYBOARD AT RUN TIME.
C -----------------------------------------------------------------------
      IBOUND=14
      MAXH=1500
      MAXK=150
      MAXN=1000
      MAXPT=1000
C -----------------------------------------------------------------------
C***
C***
C***  GET NAMES OF INPUT AND OUTPUT FILES FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL NAMFIL(IPT,IPR,UNIT10,UNIT21,EXT)
C -----------------------------------------------------------------------
C***
C***
C***  CHECK IF UNIVERSAL KRIGING IS WANTED AND GET POSSIBLE DRIFT
C***  TERMS FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL DRFTRM(0)
C -----------------------------------------------------------------------
C***
C***
C***  GET VARIOGRAM PARAMETERS FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL VARIN4
C -----------------------------------------------------------------------
C***
C***
C***  GET NEIGHBORHOOD LIMITS FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL NEIGH(RMAX,NK,NT,MAXK)
C -----------------------------------------------------------------------
C***
C***
C***  GET BOUNDARY NODES OF BLOCKS FROM DISK FILE OR FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL IRREG
C -----------------------------------------------------------------------
C***
C***
C***  GET USER'S SPCIFICATIONS FOR GRID NEEDED FOR COVARIANCE
C***  CALCULATIONS FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL GRDPTS
C -----------------------------------------------------------------------
C***
C***
C***  GET DESCRIPTION OF STATPAC INPUT FILE FROM KEYBOARD AND
C***  READ STATPAC INPUT FILE FROM DISK
C -----------------------------------------------------------------------
      CALL GETFLE(IGD)
C -----------------------------------------------------------------------
C***
C***
C***  SET UP GRID OF POINTS FOR BLOCK KRIGING
C -----------------------------------------------------------------------
      WRITE(*,'(//,'' EXECUTING...'',/,'' SETTING UP GRID OF '',
     1'' POINTS FOR COVARIANCE CALCULATIONS'')')
      CALL GRIDER(M,MAXPT)
C -----------------------------------------------------------------------
C***
C***
C***  GET COORDINATES OF CENTROID OF BLOCK
C -----------------------------------------------------------------------
      WRITE(*,'('' COMPUTING COORDINATES OF CENTROID OF BLOCK'')')
      CALL CTROID(M,XC,YC)
C -----------------------------------------------------------------------
C***
C***
C***  GET BLOCK VARIANCE.  THIS STEP CAN TAKE UP TO 20 MINUTES OF
C***  TIME IF THE BLOCK IS IRREGULAR AND THE NUMBER OF GRID POINTS
C***  SELECTED BY THE USER APPROACHES 1000.  IF THE BLOCK IS RECT-
C***  ANGULAR WITH SIDES PARALLEL TO THE COORDINATE AXES, THIS COMPUTA-
C***  TION TAKES ABOUT 1 SECOND.
C -----------------------------------------------------------------------
      WRITE(*,'('' COMPUTING BLOCK VARIANCE'')')
      CALL VRBLK4(M,BLKVAR)
C -----------------------------------------------------------------------
C***
C***
C***  FIND NEAREST HOLES WITHIN DISTANCE RMAX OF CENTROID.  NUMBER OF
C***  HOLES, NS, CANNOT EXCEED NK.
C -----------------------------------------------------------------------
      WRITE(*,'('' SEARCHING FOR DRILL HOLES WITHIN SEARCH RADIUS'')')
      CALL HLSRCH(XC,YC,NS,NK,RMAX*RMAX,RMAX,1,NHOL,1,0)
C -----------------------------------------------------------------------
C***
C***
C***  GET KRIGING WEIGHTS
C -----------------------------------------------------------------------
      CALL KRIG4B(M,NS)
C -----------------------------------------------------------------------
C***
C***
C***  GET KRIGED MEAN GRADE OF THE BLOCK AND ITS ERROR VARIANCE
C -----------------------------------------------------------------------
      WRITE(*,'('' COMPUTING MEAN AND VARIANCE'')')
      CALL MNVAR(NS,BLKVAR,GR,VARKG)
C -----------------------------------------------------------------------
C***
C***
C***  WRITE OUT RESULTS OF BLOCK ESTIMATION TO CHARACTER OUTPUT FILE
C -----------------------------------------------------------------------
      WRITE(*,'('' WRITING RESULTS TO OUTPUT FILE'')')
      CALL KRGOT4(M,XC,YC,RMAX,NS,BLKVAR,GR,VARKG)
C -----------------------------------------------------------------------
C***
C***
C***  INFORM USER PROGRAM IS FINISHED
C***
C -----------------------------------------------------------------------
      WRITE(*,'(//)')
      WRITE(*,'('' COMPUTATIONS COMPLETE'',/,'' THE STATPAC INPUT '',
     1''FILE WAS '',A32,/,'' THE CHARACTER OUTPUT FILE IS '',A32)')
     2UNIT10,UNIT21
         DO 10 I=1,5
         CALL SOUND(1000,25)
   10    CONTINUE
      CALL TICKER(ITIME2)
      RTIME=FLOAT(ITIME2-ITIME1)/6000.
      WRITE(*,'('' PROGRAM ACTIVE TIME WAS'',F6.2,'' MINUTES'')')
     1RTIME
      STOP 'NORMAL END OF PROGRAM'
      END
      SUBROUTINE NAMFIL(IPT,IPR,UNIT10,UNIT21,EXT)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE OPENS A STATPAC INPUT FILE AND A
C***  CHARACTER OUTPUT FILE FOR PROGRAM SS2DBLOK.
C -----------------------------------------------------------------------
      CHARACTER*4 EXT,EXT0
      CHARACTER*32 UNIT10,UNIT21
      LOGICAL FLAG
      DATA EXT0/'.UPR.'/
   10 WRITE(*,'(5X,''WHAT IS THE ROOT NAME OF THE STATPAC INPUT '',
     1''FILE? '',\)')
      READ(*,20) UNIT10
   20 FORMAT(A32)
      CALL CHEKER(UNIT10,EXT0,FLAG)
      IF(FLAG) GO TO 10
C -----------------------------------------------------------------------
C***  CHECK FOR EXISTENCE OF INPUT FILE
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT10,2,FLAG)
      IF(FLAG) GO TO 10
      OPEN(IPT,FILE=UNIT10,STATUS='OLD',FORM='UNFORMATTED')
   30 WRITE(*,'(5X,''GIVE A ROOT NAME FOR THE CHARACTER OUTPUT '',
     1''FILE: '',\)')
      READ(*,20) UNIT21
      CALL CHEKER(UNIT21,EXT,FLAG)
      IF(FLAG) GO TO 10
C -----------------------------------------------------------------------
C***  CHECK FOR EXISTENCE OF OUTPUT FILE.  IF IT ALREADY EXISTS,
C***  NOTIFY THE USER.
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT21,1,FLAG)
      IF(FLAG) GO TO 30
      OPEN(IPR,FILE=UNIT21,STATUS='NEW',FORM='FORMATTED')
      RETURN
      END
      SUBROUTINE IRREG
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE INPUTS THE COORDINATES OF THE BLOCK TO BE
C***  KRIGED AND COMPUTES THE AREA OF THE BLOCK.
C***  COORDINATES CAN BE ENTERED MANUALLY FROM THE TERMINAL
C***  OR MAY BE READ FROM UNIT14.
C***  IF A BOUNDARY FILE IS USED, IT MUST BE A FORMATTED FILE WITH
C***  THE FILE NAME EXTENSION ".FDI".  THE COORDINATES OF THE NODES
C***  MUST BE ENTERED IN A CLOCKWISE ORDER WITH THE NORTHING FIRST
C***  AND THE EASTING SECOND.  THE NORTHING AND EASTING MUST BE
C***  SEPARATED BY EITHER A COMMA OR A SPACE.  EMBEDDED BLANKS IN
C***  THE NORTHINGS OR EASTINGS WILL CAUSE AN ERROR.
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C***  REVISED BY W. D. GRUNDY  JULY 1985
C***
C***  EXTERNAL REFERENCES: AREA, CHEKER, CKSAFE, ERRMSG, SOUND
C -----------------------------------------------------------------------
      CHARACTER*32 UNIT14
      CHARACTER*1 IANS
      INTEGER*4 ISEED
      LOGICAL IFLAG
      COMMON /CONT2/ IY,IX,YMAX,YMIN,XMAX,XMIN,PGRID(2,2,1000),
     1YNO(1000),XEA(1000),NODES,AR,IREG,MAXN,ISEED
      WRITE(*,'(//,5X,''ENTER DESCRIPTORS OF BLOCK TO BE KRIGED'',//)')
   10 WRITE(*,'(5X,''IS BLOCK RECTANGULAR WITH SIDES PARALLEL TO'',/,
     15X,''COORDINATE AXES? ANSWER YES OR NO :'',\)')
      READ (*,'(A1)') IANS
C -----------------------------------------------------------------------
C***  MAKE SURE ANSWER WAS SPECIFICALLY YES OR NO
C -----------------------------------------------------------------------
      IF(IANS.EQ.'Y'.OR.IANS.EQ.'y'.OR.IANS.EQ.
     1'N'.OR.IANS.EQ.'n') GO TO 20
      CALL SOUND(256,100)
      GO TO 10
   20    IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') THEN
C -----------------------------------------------------------------------
C***  RECTANGULAR BLOCK WITH SIDES PARALLEL TO COORDINATE AXES
C -----------------------------------------------------------------------
         IREG=1
   30    WRITE(*,'(5X,''ENTER MINIMUM NORTHING OF RECTANGLE : '',\)')
         READ(*,40,IOSTAT=IERR) YMIN
   40    FORMAT(BN,F10.0)
         IF(IERR.NE.0) CALL ERRMSG(2,*30)
   50    WRITE(*,'(5X,''ENTER MAXIMUM NORTHING OF RECTANGLE : '',\)')
         READ(*,40,IOSTAT=IERR) YMAX
         IF(IERR.NE.0) CALL ERRMSG(2,*50)
   60    WRITE(*,'(5X,''ENTER MINIMUM  EASTING OF RECTANGLE : '',\)')
         READ(*,40,IOSTAT=IERR) XMIN
         IF(IERR.NE.0) CALL ERRMSG(2,*60)
   70    WRITE(*,'(5X,''ENTER MAXIMUM  EASTING OF RECTANGLE : '',\)')
         READ(*,40,IOSTAT=IERR) XMAX
         IF(IERR.NE.0) CALL ERRMSG(2,*70)
            IF(XMAX.GT.XMIN.AND.YMAX.GT.YMIN) THEN
            AR=(YMAX-YMIN)*(XMAX-XMIN)
            NODES=4
            YNO(1)=YMAX
            YNO(2)=YMAX
            YNO(3)=YMIN
            YNO(4)=YMIN
            XEA(1)=XMIN
            XEA(2)=XMAX
            XEA(3)=XMAX
            XEA(4)=XMIN
            GO TO 200
            ELSE
            CALL SOUND(256,100)
            WRITE(*,'('' MAXIMUM VALUES MUST BE GREATER THAN MINIMUM '',
     1      ''VALUES; REDO OR BREAK'')')
            GO TO 30
            ENDIF
         ENDIF
C -----------------------------------------------------------------------
C***  PROCEDURE FOR IRREGULAR BLOCK
C -----------------------------------------------------------------------
      IREG=2
      YMIN=99999999.
      YMAX=0.
      XMIN=99999999.
      XMAX=0.
   80 WRITE(*,'(5X,''ARE COORDINATES OF NODES TO BE ENTERED'',/,
     15X,''(1) FROM THE TERMINAL OR (2) FROM A STORED FILE?'',/,
     220X,''ANSWER 1 OR 2: '',\)')
      READ(*,'(A1)') IANS
         IF(IANS.NE.'1'.AND.IANS.NE.'2') THEN
         CALL SOUND(256,100)
         GO TO 80
         ENDIF
         IF(IANS.EQ.'1') THEN
C -----------------------------------------------------------------------
C***  INPUT OF BOUNDARY NODES IS FROM THE TERMINAL
C -----------------------------------------------------------------------
   90    WRITE(*,'(5X,''ENTER THE NUMBER OF VERTICES OF THE POLYGON '',\
     1   )')
         READ(*,'(BN,I5)',IOSTAT=IERR) NODES
         IF(IERR.NE.0) CALL ERRMSG(1,*90)
            IF(NODES.LT.3.OR.NODES.GT.MAXN) THEN
            CALL SOUND(256,100)
            WRITE(*,'(5X,''THE NUMBER OF VERTICES CANNOT BE LESS '',
     1      ''THAN 3 OR MORE THAN '',I5)') MAXN
            GO TO 90
            ENDIF
            DO 120 I=1,NODES
  100       WRITE(*,'(5X,''ENTER THE '',I4,'' NORTH COORDINATE: '',\)')
     1      I
            READ(*,40,IOSTAT=IERR) YNO(I)
            IF(IERR.NE.0) CALL ERRMSG(2,*100)
  110       WRITE(*,'(5X,''ENTER THE '',I4,''  EAST COORDINATE: '',\)')
     1      I
            READ(*,40,IOSTAT=IERR) XEA(I)
            IF(IERR.NE.0) CALL ERRMSG(2,*110)
  120       CONTINUE
         ELSE
C -----------------------------------------------------------------------
C***  ENTRY OF POLYGONAL NODES IS FROM A STORED FILE
C -----------------------------------------------------------------------
  130    WRITE(*,'(5X,''WHAT IS THE ROOT NAME OF THE BOUNDARY '',
     1   ''NODE FILE? '',\)')
         READ(*,'(A32)') UNIT14
         CALL CHEKER(UNIT14,'.FDI',IFLAG)
         IF(IFLAG) GO TO 130
         CALL CKSAFE(UNIT14,2,IFLAG)
         IF(IFLAG) GO TO 130
         OPEN(14,FILE=UNIT14,STATUS='OLD',FORM='FORMATTED')
         NODES=1
  140    READ(14,*,IOSTAT=IERR) YNO(NODES),XEA(NODES)
         IF(IERR) 170,160,150
  150    WRITE(*,'('' FORMAT PROBLEM AT NODE '',I4,'' OF BOUNDARY '',
     1   ''FILE'',/,A32)') NODES, UNIT14
         STOP
  160    NODES=NODES+1
         GO TO 140
  170    NODES=NODES-1
         ENDIF
C -----------------------------------------------------------------------
C***  TEST NODES FOR VALIDITY AND FINISH
C -----------------------------------------------------------------------
  180 IF(NODES.LT.3.OR.NODES.GT.MAXN)
     1STOP 'PROGRAM STOPPED BECAUSE OF ERROR IN NUMBER OF NODES'
C -----------------------------------------------------------------------
C***  CLOSE BOUNDARY ARRAY,IF NECESSARY
C -----------------------------------------------------------------------
      IF(YNO(NODES).NE.YNO(1).OR.
     1   XEA(NODES).NE.XEA(1)) THEN
         NODES=NODES+1
         YNO(NODES)=YNO(1)
         XEA(NODES)=XEA(1)
         WRITE(*,'(5X,''ADDITIONAL NODE WAS ADDED TO CLOSE BOUNDARY'')')
         GO TO 180
         ENDIF
C -----------------------------------------------------------------------
C***  GET MINIMUM AND MAXIMUM COORDINATES OF THE BLOCK
C -----------------------------------------------------------------------
         DO 190 I=1,NODES
         IF(YNO(I).LT.YMIN) YMIN=YNO(I)
         IF(YNO(I).GT.YMAX) YMAX=YNO(I)
         IF(XEA(I).LT.XMIN) XMIN=XEA(I)
         IF(XEA(I).GT.XMAX) XMAX=XEA(I)
  190    CONTINUE
C -----------------------------------------------------------------------
C***  CALCULATE AREA OF POLYGONAL BLOCK
C -----------------------------------------------------------------------
      AR=AREA(NODES,YNO,XEA)
  200 RETURN
      END
      SUBROUTINE GRDPTS
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE INPUTS DESCRIPTION OF KRIGING GRID IN
C***  THE BLOCK OF GROUND TO BE KRIGED.  THE GRID IS USED FOR ESTI-
C***  THE BLOCK VARIANCE (AVERAGE VALUE OF THE COVARIANCE BETWEEN
C***  ALL POSSIBLE PAIRS OF POINTS IN THE BLOCK) AND TO ESTIMATE
C***  THE DATA POINT TO BLOCK COVARIANCE COLUMN VECTOR IN THE KRIGING
C***  SYSTEM.  SEE COMMENTS IN VRBLK4.FOR.
C***  FOR GUIDLINES ON THE NUMBER OF GRID POINTS TO USE, SEE
C***  DAVID (1977, PP. 280,281) AND JOURNEL AND HUIJBREGTS (1978, P97).
C***  FOR DISCUSSION OF THE THEORY INVOLVED IN ESTIMATING BLOCK
C***  VARIANCE, SEE JOURNEL AND HUIJBREGTS (1978, PP. 95-103).
C***  THE GRIDDING PROCEDURES USED FOR PROGRAM SS2DBLOK ARE DE-
C***  SIGNED TO REDUCE THE ZERO-EFFECT (DEFINED IN JOURNEL AND
C***  HUIJBREGTS, 1978, P. 96).
C***
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C***  REVISED BY W. D. GRUNDY  JULY 1985
C***
C***  EXTERNAL REFERENCE: ERRMSG
C***
C -----------------------------------------------------------------------
$INCLUDE:'BKKRIG.INC'
      CHARACTER*1 NILIST
      WRITE(*,'(//,5X,''ENTER DESCRIPTORS OF THE KRIGING GRID'',//)')
   10 WRITE(*,'(5X,''ENTER NUMBER OF GRID POINTS IN NORTH DIRECTION : ''
     1,\)')
      READ (*,20,IOSTAT=IERR) IY
   20 FORMAT(BN,I8)
      IF(IERR.NE.0) CALL ERRMSG(1,*10)
         IF(IY.LE.0) THEN
         CALL SOUND(256,100)
         WRITE(*,30)
   30    FORMAT(' NUMBER OF GRID POINTS MUST NOT BE ZERO.  REDO.')
         GO TO 10
         ENDIF
   40 WRITE(*,'(5X,''ENTER NUMBER OF GRID POINTS IN  EAST DIRECTION : ''
     1,\)')
      READ (*,20,IOSTAT=IERR) IX
      IF(IERR.NE.0) CALL ERRMSG(1,*40)
         IF(IX.LE.0) THEN
         CALL SOUND(256,100)
         WRITE(*,30)
         GO TO 40
         ENDIF
      WRITE(*,'(5X,''IS A DRILL HOLE LIST WANTED? YES OR NO  : '',\)')
      READ (*,'(A1)') NILIST
      ILIST=0
      IF(NILIST.EQ.'Y'.OR.NILIST.EQ.'y') ILIST=1
      WRITE(*,'(5X,''IS LISTING OF BOUNDARY NODES WANTED? YES OR NO  : '
     1',\)')
      READ(*,'(A1)') NILIST
      IF(NILIST.EQ.'Y'.OR.NILIST.EQ.'y') ILIST=ILIST+2
         IF(IREG.NE.1) THEN
   50    WRITE(*,'(5X,''ENTER STARTING SEED FOR RANDOM NUMBER '',
     1   ''GENERATOR: '',\)')
         READ (*,20,IOSTAT=IERR) ISEED
         IF(IERR.NE.0) CALL ERRMSG(1,*50)
         ENDIF
      RETURN
      END
      SUBROUTINE GRIDER(M,MAXPT)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE CREATES A GRID OF POINTS FOR KRIGING
C***  A BLOCK OF GROUND
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C***  SUBPROGRAMS REQUIRED: RAND, PTLOC
C -----------------------------------------------------------------------
      INTEGER*4 ISEED
      COMMON /CONT2/ IY,IX,YMAX,YMIN,XMAX,XMIN,PGRID(2,2,1000),
     &YNO(1000),XEA(1000),NODES,AR,IREG,MAXN,ISEED
      INTEGER*2 PTLOC
      WIDY=YMAX-YMIN
      WIDX=XMAX-XMIN
      WIDX=WIDX-WIDX*1.E-05
      WIDY=WIDY-WIDY*1.E-05
C -----------------------------------------------------------------------
C***  CALCULATE POINTS WITHIN THE BLOCK
C -----------------------------------------------------------------------
      DX=WIDX/FLOAT(IX)
      DY=WIDY/FLOAT(IY)
      M=0
         DO 40 I=1,IX
            DO 40 J=1,IY
            X=XMIN+DX/2.+FLOAT(I-1)*DX
            Y=YMAX-DY/2.-FLOAT(J-1)*DY
            GO TO(20,10),IREG
   10       IF(PTLOC(NODES,XEA,YNO,X,Y)-1) 40,20,40
   20       M=M+1
            IF(M-MAXPT) 30,30,110
   30       PGRID(1,1,M)=X
            PGRID(2,1,M)=Y
   40       CONTINUE
C -----------------------------------------------------------------------
C***  GET RANDOMLY DISPLACED POINTS AROUND GRIDDED POINTS
C -----------------------------------------------------------------------
      GO TO(100,50),IREG
   50 DX1=0.5*DX
      DY1=0.5*DY
         DO 90 I=1,M
         IBAD=0
C -----------------------------------------------------------------------
C***  RAND IS A UNIFORM RANDOM NUMBER GENERATOR IN
C***  THE RANGE 0-1.
C -----------------------------------------------------------------------
   60    X=PGRID(1,1,I)+RAND(ISEED)*DX-DX1
         Y=PGRID(2,1,I)+RAND(ISEED)*DY-DY1
C -----------------------------------------------------------------------
C***  TRY TO KEEP DISPLACED POINT WITHIN BLOCK
C -----------------------------------------------------------------------
         IF(PTLOC(NODES,XEA,YNO,X,Y)) 70,80,80
   70    IBAD=IBAD+1
         IF(IBAD-5) 60,80,80
   80    PGRID(1,2,I)=X
         PGRID(2,2,I)=Y
   90    CONTINUE
  100 RETURN
  110 WRITE(*,120)
  120 FORMAT(5X,'TOO MANY GRID POINTS - PROGRAM STOPPED')
      STOP
      END
      SUBROUTINE CTROID(M,XC,YC)
C -----------------------------------------------------------------------
C***  COMPUTES X AND Y COORDINATES OF CENTROID OF POLYGON
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C -----------------------------------------------------------------------
      INTEGER*4 ISEED
      COMMON /CONT2/ IY,IX,YMAX,YMIN,XMAX,XMIN,PGRID(2,2,1000),
     1YNO(1000),XEA(1000),NODES,AR,IREG,MAXN,ISEED
      SUMX=0.
      SUMY=0.
         DO 10 I=1,M
         SUMX=SUMX+PGRID(1,1,I)
         SUMY=SUMY+PGRID(2,1,I)
   10    CONTINUE
      XC=SUMX/FLOAT(M)
      YC=SUMY/FLOAT(M)
      RETURN
      END
      SUBROUTINE VRBLK4(M,BLKVAR)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE COMPUTES THE VARIANCE OF A BLOCK
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C***  SUBPROGRAMS REQUIRED: CBAR4, COVAR4
C***
C***
C***               ESTIMATION OF BLOCK VARIANCE
C***
C***  FOR IRREGULAR BLOCK:
C***
C***  THE COMPUTER CREATES A GRID OF POINTS IN A RECTANGULAR AREA WHICH
C***  ENCLOSES THE POLYGONAL BLOCK BEING KRIGED. THE NODES OF THE RECT-
C***  ANGLE ARE DEFINED BY THE MINIMUM AND MAXIMUM X- AND Y-COORDINATES
C***  OF THE POLYGONAL BLOCK.  AS EACH GRID POINT IS COMPUTED, IT IS
C***  TESTED BY AN INSIDE-OUTSIDE ROUTINE TO DETERMINE WHETHER IT LIES
C***  WITHIN THE POLYGON ITSELF.  POINTS LYING WITHIN THE POLYGONAL
C***  BLOCK ARE STORED IN RAM.  NEXT, EACH RETAINED GRID POINT IS RAN-
C***  DOMLY OFFSET BY A SMALL AMOUNT AND THE RESULTING RANDOMLY DIS-
C***  PLACED GRID IS ALSO STORED IN RAM.  THE BLOCK VARIANCE IS THEN
C***  ESTIMATED BY COMPUTING THE MEAN VALUE OF THE COVARIANCE BETWEEN
C***  ALL PAIRINGS OF POINTS OF THE REGULAR GRID WITH THE RANDOMLY-
C***  DISPLACED GRID.  THIS TECHNIQE WAS ADAPTED TO AVOID A BIASSED
C***  ESTIMATE OF THE BLOCK VARIANCE (SEE JOURNEL AND HUIJBREGRS,
C***  1978, PP. 96-97).
C***
C***  FOR RECTANGULAR BLOCK (WITH SIDES PARALLEL TO THE COORDINATE
C***  AXES):
C***
C***  BLOCK VARIANCE FOR A RECTANGULAR BLOCK IS DONE BY AN ANALYTICAL
C***  TECHNIQUE ADAPTED FROM JOURNEL AND HUIJBREGTS (SEE COMMENTS
C***  IN SUBROUTINE CBAR4).  THE RESULTING ESTIMATE OF BLOCK VARIANCE
C***  IS MORE PRECISE THAN THE METHOD USED FOR IRREGULAR BLOCKS (SEE
C***  ABOVE).
C***
C***
C -----------------------------------------------------------------------
      INTEGER*4 ISEED
      COMMON /CONT2/ IY,IX,YMAX,YMIN,XMAX,XMIN,PGRID(2,2,1000),
     1YNO(1000),XEA(1000),NODES,AR,IREG,MAXN,ISEED
C -----------------------------------------------------------------------
C***  COMPUTE THE BLOCK VARIANCE
C -----------------------------------------------------------------------
      GO TO(10,20),IREG
   10 BLKVAR=CBAR4(XMIN,XMAX,YMIN,YMAX)
      GO TO 40
   20 BLKVAR=0.
C -----------------------------------------------------------------------
C***  THE FOLLOWING LINES OF CODE ARE BASED ON A ALGORITHM DESCRIBED
C***  BY DAVID (1977, PP 187-189)
C -----------------------------------------------------------------------
         DO 30 I=1,M
            DO 30 J=1,M
            BLKVAR=BLKVAR+COVAR4(PGRID(1,1,I),PGRID(1,2,J))
   30       CONTINUE
      BLKVAR=BLKVAR/FLOAT(M*M)
   40 RETURN
      END
      SUBROUTINE KRIG4B(M,NS)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE OBTAINS THE KRIGING WEIGHTS FOR BLOCK KRIGING
C***  THIS SUBROUTINE IS ADAPTED FROM THE KRIGING PACKAGE OF
C***  SKRIVAN AND KARLINGER AND MODIFIED BY W.D. GRUNDY, MARCH 1981
C***  MODIFICATION INVOLVED CODING TO CONVERT FROM POINT- TO
C***  BLOCK-KRIGING.
****  SUBPROGRAMS REQUIRED: COVAR4, DOLITL, MAXCOR, PSET4, TERM
C -----------------------------------------------------------------------
$INCLUDE:'BKKRIG.INC'
      DIMENSION P1(2)
C -----------------------------------------------------------------------
C***  BE SURE THERE ARE ENOUGH SAMPLE POINTS
C -----------------------------------------------------------------------
      IF(NS.LE.NT) GO TO 90
      NTOT=NS+NT
C -----------------------------------------------------------------------
C***  GET MAXIMUM COORDINATES OF DRILL HOLES FOR SCALING PURPOSES
C***  IF NT=1, THERE IS NO NEED FOR CALLING MAXCOR
C -----------------------------------------------------------------------
      XMAX=0.
      YMAX=0.
      IF(NT.EQ.1) GO TO 10
      CALL MAXCOR(XMAX,YMAX,NS)
C -----------------------------------------------------------------------
C***  SET UP LOWER TRIANGULAR MATRIX OF COVARIANCES AND DRIFT TERMS
C -----------------------------------------------------------------------
   10 WRITE(*,'(A)')' SETTING UP SYSTEM OF SIMULTANEOUS EQUATIONS'
      CALL PSET4(NS,XMAX,YMAX)
C -----------------------------------------------------------------------
****  SET UP RIGHT HAND SIDE
C -----------------------------------------------------------------------
      XM=FLOAT(M)
         DO 30 I=1,NS
         J=IHOL(I)
         P1(1)=XE(J)
         P1(2)=YN(J)
         SUM=0.
C -----------------------------------------------------------------------
C***  NOTE: PGRID(1,1,K) REPRESENTS POINTS OF A REGULAR GRID WITHIN
C***  THE BLOCK BEING KRIGED.  THIS ARRAY WAS CREATED IN SUBROUTINE
C***  GRIDER WITH USER-SPECIFICATIONS OBTAINED IN SUBROUTINE GRDPTS.
C -----------------------------------------------------------------------
            DO 20 K=1,M
            SUM=SUM+COVAR4(P1,PGRID(1,1,K))
   20       CONTINUE
         R(I)=SUM/XM
   30    CONTINUE
C***  GET MEAN VALUES OF DRIFT MONOMIALS OF GRID
      IF(NT-1) 40,40,50
   40 R(NS+1)=1.
      GO TO 80
   50 K=NS+1
         DO 70 I=K,NTOT
         J=I-NS
         SUM=0.
            DO 60 L=1,M
            SUM=SUM+TERM(PGRID(1,1,L),EX(J),PGRID(2,1,L),EY(J),XMAX,
     1      YMAX)
   60       CONTINUE
         R(I)=SUM/XM
   70    CONTINUE
C -----------------------------------------------------------------------
C***  GET WEIGHTS WHICH WILL BE STORED IN W
C -----------------------------------------------------------------------
   80 WRITE(*,'(A)')' SOLVING SYSTEM OF SIMULTANEOUS EQUATIONS'
      CALL DOLITL(NTOT,0,IER)
      IF(IER.NE.0) GO TO 110
      RETURN
   90 WRITE(*,'(5X,''ONLY'',I5,'' SAMPLE POINTS - PROGRAM STOPPED.'',
     1)') NS
  100 STOP
  110 WRITE(*,'(A)')' ERROR IN MATRIX SOLUTION'
         DO 120 MM=1,NS
         LL=IHOL(MM)
         WRITE(*,'('' '',A8,1X,5(F10.2,1X))') IHOLID(LL),YN(LL),XE(LL)
  120    CONTINUE
      GO TO 100
      END
      SUBROUTINE KRGOT4(M,XC,YC,RMAX,NS,BLKVAR,GR,VARKG)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE PRINTS OUT PARAMETERS AND BLOCK KRIGING RESULTS
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, MARCH 1981
C -----------------------------------------------------------------------
$INCLUDE:'BKKRIG.INC'
      CHARACTER*4 IVRTYP
      DIMENSION IVRTYP(5)
      DATA IVRTYP/'SPHR','EXPO','LIN ','GAUS','CUB '/
      IF(NEST.NE.0) GO TO 20
      WRITE(IPR,10) C0
   10 FORMAT(//,' NUGGET ONLY MODEL --- NUGGET= ',F10.4)
      GO TO 50
   20 WRITE(IPR,30) C0,SILL,NEST,(I,IVRTYP(ITYPE(I)),C(I),ANGLE(I),
     1AFH(I),AVALU(I),SLOPE(I),I=1,NEST)
   30 FORMAT(29X,'VARIOGRAM PARAMETERS',//,14X,'NUGGET = ',F7.3,5X,'SILL
     1= ',F8.3,3X,'NO.STRUCTURES = ',I1,//,
     2' VARIOGRAM  C-VALUE    ANGLE     ANISOTROPY  A-VALUE      SLOPE',
     3/,'  NO. TYPE           ANISOTROPY    FACTOR              (IF LINE
     4AR)',//,(I4,2X,A4,1X,F9.3,1X,F8.3,7X,F5.3,3X,F9.3,2X,F8.3))
   40 FORMAT(29X,'VARIOGRAM PARAMETERS',//,14X,'NUGGET = ',F7.3,5X,'SILL
     1= ',F8.3,3X,'NO.STRUCTURES = ',I1,//,
     2' VARIOGRAM NO.  C-VALUE     RANGE   ANGLE ANISOTROPY   ANISOTROPY
     3FACTOR',//,(6X,I1,8X,F9.4,1X,F9.4,5X,F7.3,15X,F7.3))
C -----------------------------------------------------------------------
C***  IF ORDINARY KRIGING IS WANTED, DO NOT PRINT OUT THE DRIFT TERMS
C -----------------------------------------------------------------------
   50 IF(NT.EQ.1) GO TO 70
      WRITE(IPR,60) NT,(I,EX(I),EY(I),I=1,NT)
   60 FORMAT(//,24X,'NUMBER OF DRIFT TERMS =  ',I2,//,
     120X,'TERM NO.',3X,'X-EXPONENT',3X,'Y-EXPONENT',/,
     2(23X,I2,11X,I2,11X,I2))
   70 WRITE(IPR,80) IY,IX
   80 FORMAT(//,1X,'NO. OF Y-GRID POINTS   ',I5,/
     11X,'NO. OF X-GRID POINTS   ',I5)
      IF(ILIST.LE.1) GO TO 110
      WRITE(IPR,90)
      WRITE(IPR,100) (YNO(I),XEA(I),I=1,NODES)
   90 FORMAT(1X,'DEFINITION OF BLOCK - NODES',/,
     1'     NORTHING        EASTING')
  100 FORMAT(1X,'(',E15.8,',',E15.8,')')
  110 WRITE(IPR,120) AR,RMAX,NHOL
  120 FORMAT(1X,'AREA OF BLOCK = ',E15.8,/,1X,'SEARCH RADIUS = ',E15.8,/
     1,1X,'NUMBER OF HOLES READ FROM DATA FILE = ',I5)
  130 WRITE(IPR,140) BLKVAR
  140 FORMAT(//,' BLOCK VARIANCE =',19X,F12.5,//)
      WRITE(IPR,150) M
  150 FORMAT(1X,'NUMBER OF GRID POINTS USED',I5)
      WRITE(IPR,160) NS
  160 FORMAT(1X,'NUMBER OF HOLES USED',I5)
      WRITE(IPR,170) XC,YC,GR,VARKG
  170 FORMAT(//,25X,'***OUTPUT OF KRIGED RESULTS***',//,19X,
     1'X-COORD.  Y-COORD.   KRIGING    KRIGING',/,42X,'MEAN',
     25X,'VARIANCE',//,16X,2F10.3,2(1X,F10.3))
      IF(ILIST.EQ.0.OR.ILIST.EQ.2) GO TO 210
      WRITE(IPR,180) (IHOLID(IHOL(I)),YN(IHOL(I)),XE(IHOL(I)),
     1GD(IHOL(I)),ERMEAS(IHOL(I)),W(I),I=1,NS)
  180 FORMAT(//,3X,'HOLE',6X,'NORTH',8X,'EAST',7X,'SAMPLE',7X,'ERROR',
     14X,'KRIGING',/,4X,'NO.',6X,'COORD.',6X,'COORD.',6X,'VALUE',7X,
     2'MEASURE',3X,'WEIGHT',/,(1X,A8,4F12.3,F9.4))
C -----------------------------------------------------------------------
C***  SUM THE KRIGING WEIGHTS
C -----------------------------------------------------------------------
      SUMWTS=0.
         DO 190 I=1,NS
         SUMWTS=SUMWTS+W(I)
  190    CONTINUE
      WRITE(IPR,200) SUMWTS
  200 FORMAT(/,41X,'SUM OF WEIGHTS =',F9.4)
  210 RETURN
      END

