$STORAGE:2
      PROGRAM CARD2STP
C -----------------------------------------------------------------------
C***  PROGRAM CREATES AN UNFORMATTED U.S.G.S  STATPAC DATA FILE FROM A
C***  CARD-IMAGE ASCII INPUT FILE.  THE FORMAT OF THE ASCII INPUT FILE
C***  IS SPECIFIED AT RUN TIME IN ONE OF TWO WAYS:
C***       1) THE USER TYPES IN THE REQUIRED FORMAT; OR
C***       2) THE USER TYPES IN A DESCRIPTION OF THE ASCII FILE,
C***          FROM WHICH THIS PROGRAM "BUILDS" THE PROPER FORMAT
C***          FOR INPUTTING THE DATA.
C***
C***  EACH RECORD OF THE ASCII FILE MUST CONTAIN:
C***       1) AN ALPHAMERIC IDENTIFIER CONSISTING OF ONE TO
C***          SIXTEEN PRINTABLE CHARACTERS (E.G. DIGITS, ALPHA-
C***          BETIC CHARACTERS, SPECIAL CHARACTERS);
C***       2) AT LEAST ONE AND NO MORE THAN 10 STATPAC VARIABLES.
C***
C***  AS OPTIONS, EACH ASCII RECORD MAY ALSO CONTAIN:
C***       1) SINGLE CHARACTER STATPAC QUALIFIER CODES (E.G.
C***          B,L,N,G,T,H,*, AND BLANK (ASCII 32 CODE);
C***       2) LATITUDE AND LONGITUDE MEASURES EXPRESSED IN INTEGER
C***          DEGREES, MINUTES, SECONDS.  THE VALUES MUST BE SPECIFIED
C***          AS DDDMMSS WITH NO INTERVENING SPACES.  FOR EXAMPLE,
C***          A LATITUDE OF 30 DEGREES, 14 MINUTES, 27 SECONDS MUST
C***          BE PRESENT ON THE FILE AS 301427. (WITHOUT THE DECIMAL
C***          POINT)
C***
C***  THE USER SHOULD REMEMBER THAT THE ALPHAMERIC IDENTIFIERS AND
C***  INTEGER LATITUDE-LONGITUDE VALUES ARE NOT CONSIDERED TO BE
C***  STATPAC VARIABLES.
C***
C***  THE INDIVIDUAL RECORDS OF THE ASCII FILE MUST NOT EXCEED
C***  A LENGTH OF 80 CHARACTERS.  THE INFORMATION FIELDS (IDEN-
C***  TIFIERS, LAT-LONG, AND STATPAC VARIABLES AND QUALIFIERS)
C***  MUST EXIST IN THE SAME LOCATIONS FROM RECORD TO RECORD.
C***  THE ARRANGEMENT OF THE FIELDS DOES NOT MATTER, SO LONG AS
C***  IT IS CONSISTENT FROM RECORD-TO-RECORD.  FOR EXAMPLE, THE
C***  ALPHAMERIC IDENTIFIER CAN BE AT THE BEGINNING OF EACH RECORD,
C***  OR AT THE END OF EACH RECORD, OR AT A CONSISTENT POSITION
C***  SOMEWHERE IN BETWEEN.  THIS FLEXIBILITY IS POSSIBLE BECAUSE
C***  THE POSITION OF EACH FIELD IN THE RECORD CAN BE SPECIFIED
C***  BY THE FORTRAN  T-FORMAT.  THUS A VARIABLE ON THE RIGHT-HAND
C***  SIDE OF A RECORD CAN BE READ IN BEFORE A VARIABLE ON THE LEFT-
C***  HAND SIDE.
C***
C***  THIS PROGRAM HAS A LIMIT OF 2500 RECORDS (STATPAC ROWS).
C***  IT PROVIDES SOME ERROR TRAPS AND WARNINGS TO THE USER.
C***
C***  SUBPROGRAMS REQUIRED: ASCIIZ, BYTMOV, CHEKER, CHRKNT, CKSAFE,
C***  CLEAR, CLREOL, CONCAT, CRSOFF, CRSON, DELAY, ERRCHK, FNDCUR,
C***  FSTDSP, INT21H, INTR, IOFS, IREGHI, IREGLO, ISEG, MOVCUR, ODOMTR,
C***  PACK, PCHCEN, PCHVAR, SAVRST, SCROLL, SOUND, TICKER
C***
C***  CODED BY W. D. GRUNDY      MAY  1987
C -----------------------------------------------------------------------
      CHARACTER*80 RECORD,TMPLT1,TMPLT2,FIRST,SECOND,THIRD
      CHARACTER*240,IFRMT
      CHARACTER*32 INFILE,OUTFIL
      CHARACTER*16 ROWID
      CHARACTER*8 VARIDS(10),ID
      CHARACTER*8 ROWDAT,LATDAT,LONDAT
      CHARACTER*1 RIGHT
      CHARACTER*1 JFRMT(240),KFRMT(240),IANS,IA
      INTEGER*4 LOC
      LOGICAL FILERR,QCODE,LLCODE
      COMMON /VARS/ X(10,2500)
      COMMON /IDS/ ROWID(2500)
      DIMENSION IPACK(2),LOC(2,2500),IA(10,2500)
      EQUIVALENCE(IFRMT,JFRMT(1))
      EQUIVALENCE (FIRST,KFRMT(1)),(SECOND,KFRMT(81)),(THIRD,KFRMT(161))
      DATA IA /25000*' '/,LOC /5000*0/
      DATA ID /'XXXXXXXX'/
      DATA RIGHT /')'/
      DATA TMPLT1 /
     1'0        1         2         3         4         5         6
     2    7         8'/
      DATA TMPLT2 /
     1'12345678901234567890123456789012345678901234567890123456789012345
     2678901234567890'/
C -----------------------------------------------------------------------
C***  GET DESCRIPTION OF WHAT IS ON THE FILE
C -----------------------------------------------------------------------
      CALL CLEAR
      WRITE(*,'('' '',''DOES THE DATA FILE CONTAIN INTEGER LAT-LONG '',
     1''VALUES? Y/N: '',\)')
      READ(*,'(A)') IANS
      LLCODE=IANS.EQ.'Y'.OR.IANS.EQ.'y'
      WRITE(*,'('' '',''DOES THE DATA FILE CONTAIN QUALIFIED VALUES? '',
     1''Y/N: '',\)')
      READ(*,'(A)') IANS
      QCODE=IANS.EQ.'Y'.OR.IANS.EQ.'y'
   10 WRITE(*,'('' '',''HOW MANY VARIABLES WILL BE ON EACH RECORD? '',
     1''(MAX=10): '',\)')
      CALL ERRCHK(1,NVAR,DUMMY)
      IF(NVAR.LE.0.OR.NVAR.GT.10) THEN
      CALL SOUND(256,100)
      WRITE(*,'(A)') ' NUMBER OF VARIABLES MUST BE BETWEEN 1 AND 10'
      GO TO 10
      ENDIF
      WRITE(*,'('' '',''ENTER '',I1,'' VARIABLE IDS - FOLLOW EACH '',
     1''WITH A CARRIAGE RETURN'')') NVAR
      DO 20 I=1,NVAR
      READ(*,'(A)') VARIDS(I)
   20 CONTINUE
C -----------------------------------------------------------------------
C***  GET NAMES OF INPUT AND OUTPUT FILES
C -----------------------------------------------------------------------
      KI=20
      KO=11
   30 WRITE(*,'(A\)') ' ENTER NAME OF ASCII INPUT FILE: '
      READ(*,'(A)') INFILE
      CALL CKSAFE(INFILE,2,FILERR)
      IF(FILERR) GO TO 30
      OPEN(KI,FILE=INFILE)
   40 WRITE(*,'('' GIVE A ROOT NAME FOR THE STATPAC OUTPUT FILE: '',
     1\)')
      READ(*,'(A)') OUTFIL
      CALL CHEKER(OUTFIL,'.STP',FILERR)
      IF(FILERR) GO TO 40
      CALL CKSAFE(OUTFIL,1,FILERR)
      IF(FILERR) GO TO 40
      OPEN(KO,FILE=OUTFIL,STATUS='UNKNOWN',FORM='UNFORMATTED')
C -----------------------------------------------------------------------
C***  DISPLAY A TEMPLATE AND ALSO THE FIRST RECORD OF THE INPUT FILE
C -----------------------------------------------------------------------
   50 CALL CLEAR
      CALL PCHVAR(TMPLT1,1,80,0,0,0)
      CALL PCHVAR(TMPLT2,1,80,1,0,0)
      READ(KI,'(A)') RECORD
      CALL PCHVAR(RECORD,2,80,2,0,0)
      IFRMT='(BN,'
      FIRST=' '
      SECOND=' '
      THIRD=' '
      REWIND KI
   60 WRITE(*,70)
C -----------------------------------------------------------------------
C***  FIND OUT HOW USER WANTS TO SPECIFY THE FORMAT OF THE INPUT RECORD
C -----------------------------------------------------------------------
   70 FORMAT(//,1H ,'DO YOU WANT TO',/,6X,'1) ENTER THE FORMAT DIRECT',
     1'LY FROM THE KEYBOARD, OR',/,6X,'2) BUILD THE FORMAT USING AN',
     2' INTERACTIVE DESCRIPTION OF THE FILE?',/,11X,'ANSWER 1 OR 2: ',\)
      READ(*,'(A)') IANS
      IF(IANS.NE.'1'.AND.IANS.NE.'2') THEN
      WRITE(*,'('' '',''YOU MUST ANSWER 1 OR 2'')')
      CALL WARN
      CALL SCROLL(4,24,0,79,20,1)
      WRITE(*,'(\)')
      CALL MOVCUR(5,0,0)
      GO TO 60
      ENDIF
      IF(IANS.EQ.'1') THEN
C -----------------------------------------------------------------------
C***  USER WANTS TO ENTER FORMAT DIRECTLY FROM THE KEYBOARD
C -----------------------------------------------------------------------
      WRITE(*,'('' '',''ENTER THE FORMAT BELOW WITH 80 CHARACTERS '',
     1''OR LESS.'',/,'' REMEMBER THE FORMAT MUST START WITH A LEFT'',
     2'' PARENTHESIS'',/,'' AND END WITH A RIGHT PARENTHESIS.'')')
      READ(*,'(A)') FIRST
      CALL SCROLL(4,24,0,79,20,1)
      WRITE(*,'(\)')
      CALL MOVCUR(10,0,0)
      GO TO 150
      ENDIF
C -----------------------------------------------------------------------
C***  USER WANTS COMPUTER TO CREATE THE FORMAT
C -----------------------------------------------------------------------
      WRITE(*,'(//,'' BY DEFAULT, ALL ASCII BLANK CHARACTERS CONTAIN'',
     1''ED IN NUMERIC FIELDS WILL BE'',/,'' TREATED AS ASCII NULL'',
     2'' (BN) CHARACTERS.  DO YOU WANT TO CHANGE THIS DEFAULT'',/,
     3'' SO THAT THEY WILL BE TREATED AS ASCII ZERO (BZ) '',
     4'' CHARACTERS?'',/,11X,''ANSWER YES OR NO: '',\)')
      READ(*,'(A)') IANS
      IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') IFRMT='(BZ,'
      CALL SCROLL(4,24,0,79,20,1)
      CALL MOVCUR(4,0,0)
C -----------------------------------------------------------------------
C***  GET LOCATIONS AND WIDTHS OF FIELDS STARTING WITH ROWID
C -----------------------------------------------------------------------
   80 WRITE(*,'(//,'' '',''ENTER STARTING CHARACTER POSITION OF '',
     1''ROWID: '',\)')
      CALL ERRCHK(1,IR,DUMMY)
      IF(IR.LT.1.OR.IR.GT.80) THEN
      CALL SOUND(256,100)
      WRITE(*,90)
   90 FORMAT(' VALUE MUST BE BETWEEN 1 AND 80.  REDO.')
      GO TO 80
      ENDIF
  100 WRITE(*,'(A\)') ' ENTER WIDTH OF ROWID IN CHARACTERS (MAX=16): '
      CALL ERRCHK(1,IRW,DUMMY)
      IF(IRW.LT.1.OR.IRW.GT.16) THEN
      CALL SOUND(256,100)
      WRITE(*,'('' '',''VALUE MUST BE BETWEEN 1 AND 16.  REDO.'')')
      GO TO 100
      ENDIF
      WRITE(ROWDAT,'(''T'',I2,'',A'',I2)') IR,IRW
      CALL CONCAT(IFRMT,240,ROWDAT,8,FILERR)
      IF(LLCODE) THEN
C -----------------------------------------------------------------------
C***  GET POSITION AND WIDTH OF LAT-LONG FIELDS
C -----------------------------------------------------------------------
  110 WRITE(*,'(A\)') ' ENTER STARTING CHARACTER POSITION OF LATITUDE
     1: '
      CALL ERRCHK(1,ILAT,DUMMY)
      IF(ILAT.LT.1.OR.ILAT.GT.80) THEN
      CALL SOUND(256,100)
      WRITE(*,90)
      GO TO 110
      ENDIF
      WRITE(*,'(A\)') ' ENTER WIDTH OF LATITUDE FIELD IN CHARACTERS:
     1'
      CALL ERRCHK(1,ILATW,DUMMY)
  120 WRITE(*,'(A\)')' ENTER STARTING CHARACTER POSITION OF LONGITUDE
     1: '
      CALL ERRCHK(1,ILONG,DUMMY)
      IF(ILONG.LT.1.OR.ILONG.GT.80) THEN
      CALL SOUND(256,100)
      WRITE(*,90)
      GO TO 120
      ENDIF
      WRITE(*,'(A\)') ' ENTER WIDTH OF LONGITUDE FIELD IN CHARACTERS:
     1'
      CALL ERRCHK(1,ILONGW,DUMMY)
      WRITE(LATDAT,'('',T'',I2,'',I'',I2)') ILAT,ILATW
      WRITE(LONDAT,'('',T'',I2,'',I'',I2)') ILONG,ILONGW
      CALL CONCAT(IFRMT,240,LATDAT,8,FILERR)
      CALL CONCAT(IFRMT,240,LONDAT,8,FILERR)
      ENDIF
C -----------------------------------------------------------------------
C***  GET FORMAT SPECIFICATION FOR EACH STATPAC VARIABLE
C -----------------------------------------------------------------------
      DO 130 I=1,NVAR
      CALL SCROLL(4,24,0,79,20,1)
      WRITE(*,'(\)')
      CALL MOVCUR(5,0,0)
      CALL BUILD(IFRMT,VARIDS(I),QCODE)
  130 CONTINUE
C -----------------------------------------------------------------------
C***  CLOSE IFRMT WITH A RIGHT PARENTHESIS
C -----------------------------------------------------------------------
      CALL CONCAT(IFRMT,240,RIGHT,1,FILERR)
C -----------------------------------------------------------------------
C***  NOW SQUEEZE OUT THE BLANKS IN THE FORMAT TO MAKE IT LOOK PRETTY
C -----------------------------------------------------------------------
      K=0
      DO 140 I=1,240
      IF(JFRMT(I).NE.' ') THEN
      K=K+1
      KFRMT(K)=JFRMT(I)
      ENDIF
  140 CONTINUE
C -----------------------------------------------------------------------
C***  LET USER TAKE A LOOK AT THE FORMAT THAT WAS CREATED
C -----------------------------------------------------------------------
  150 WRITE(*,'(/,'' THE FOLLOWING FORMAT WILL BE USED:'')')
      CALL PCHVAR(FIRST,1,80,12,0,0)
      CALL PCHVAR(SECOND,1,80,13,0,0)
      CALL PCHVAR(THIRD,1,80,14,0,0)
      WRITE(*,'('' DO YOU WANT TO CHANGE THIS? Y/N: '',\)')
      READ(*,'(A)') IANS
      IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') GO TO 50
C -----------------------------------------------------------------------
C***  NOW READ THE INPUT FILE AND STORE INFORMATION IN RAM
C -----------------------------------------------------------------------
      K=1
      WRITE(*,'(A)') '       ...READING DATA'
      CALL FNDCUR(IRRR,ICCC,0)
  160 READ(KI,'(A)',IOSTAT=IERR) RECORD
      IF(IERR.EQ.0.AND.K.GT.2500) THEN
      CALL WARN
      WRITE(*,'('' '',''FILE CONTAINS MORE THAN 2500 RECORDS'',/,
     1'' RECORDS PAST 2500 WILL NOT BE INCLUDED IN THE OUTPUT FILE'')')
      GO TO 180
      ENDIF
      IF(IERR) 180,170,200
  170 IF(LLCODE.AND.QCODE) THEN
      READ(RECORD,KFRMT,ERR=200) ROWID(K),LOC(1,K),LOC(2,K),(X(I,K),
     1IA(I,K),I=1,NVAR)
      ELSE IF(LLCODE) THEN
      READ(RECORD,KFRMT,ERR=200) ROWID(K),LOC(1,K),LOC(2,K),(X(I,K),
     1I=1,NVAR)
      ELSE IF(QCODE) THEN
      READ(RECORD,KFRMT,ERR=200) ROWID(K),(X(I,K),IA(I,K),I=1,NVAR)
      ELSE
      READ(RECORD,KFRMT,ERR=200) ROWID(K),(X(I,K),I=1,NVAR)
      END IF
      IF(MOD(K,10).EQ.0) CALL ODOMTR(IRRR,K)
      K=K+1
      GO TO 160
  180 K=K-1
C -----------------------------------------------------------------------
C***  THE OUTPUT FILE IS WRITTEN HERE
C -----------------------------------------------------------------------
      WRITE(*,'(''+'',I5)') K
      WRITE(*,'(A)') '       ...WRITING OUTPUT FILE'
      CALL FNDCUR(IRRR,ICCC,0)
      WRITE(KO) ID,K,NVAR,(VARIDS(I),I=1,NVAR)
      NPACK=(NVAR+4)/5
      DO 190 I=1,K
      CALL PACK(IA(1,I),NVAR,IPACK,NPACK)
      WRITE(KO) I,ROWID(I),LOC(1,I),LOC(2,I),(X(L,I),L=1,NVAR),
     1(IPACK(J),J=1,NPACK)
      IF(MOD(I,10).EQ.0) CALL ODOMTR(IRRR,I)
  190 CONTINUE
      WRITE(*,'(''+'',I5)') K
      WRITE(*,'('' THE STATPAC OUTPUT FILE IS '',A32)') OUTFIL
      WRITE(*,'('' N= '',I4,'', M= '',I2)') K,NVAR
      STOP 'NORMAL END OF PROGRAM'
  200 CALL SOUND(256,100)
C -----------------------------------------------------------------------
C***  IF FILE CONTAINS A RECORD IT CAN'T READ IT IS DISPLAYED
C -----------------------------------------------------------------------
      WRITE(*,'('' FATAL ERROR IN READING RECORD NUMBER '',I4,/,
     1'' THIS BAD RECORD IS DISPLAYED BENEATH THE TEMPLATE.'')') K
      CALL FNDCUR(IRRR,ICCC,0)
      CALL PCHVAR(RECORD,3,80,3,0,0)
      CALL MOVCUR(IRRR,ICCC,0)
      STOP
      END
      SUBROUTINE BUILD(IFRMT,VAR,QCODE)
C -----------------------------------------------------------------------
C***  GETS DESCRIPTION OF EACH FIELD CONTAINING A STATPAC VARIABLE
C -----------------------------------------------------------------------
      CHARACTER*80 IFRMT
      CHARACTER*8 VAR
      CHARACTER*18 FIELD
      LOGICAL FILERR,QCODE
   10 WRITE(*,'(/,'' ENTER STARTING CHARACTER POSITION OF '',A8,
     1'': '',\)') VAR
      CALL ERRCHK(1,IX,DUMMY)
      IF(IX.LT.1.OR.IX.GT.80) THEN
      CALL SOUND(256,100)
      WRITE(*,20)
   20 FORMAT(' NEED A NUMBER FROM 1 TO 80.')
      GO TO 10
      ENDIF
      WRITE(*,'('' ENTER WIDTH OF '',A8,'' FIELD IN CHARACTERS: '',\)')
     1VAR
      CALL ERRCHK(1,IWX,DUMMY)
      WRITE(*,'(A\)') ' ENTER NUMBER OF DIGITS TO RIGHT OF DECIMAL: '
      CALL ERRCHK(1,IXD,DUMMY)
      IF(IXD.GT.IWX) THEN
      CALL SOUND(256,100)
      WRITE(*,'('' '',''NUMBER OF DIGITS TO RIGHT OF DECIMAL CANNOT'',
     1'' EXCEED WIDTH OF THE FIELD'',/,'' REDO FROM START'')')
      GO TO 10
      ENDIF
      IF(QCODE) THEN
   30 WRITE(*,'('' '',''ENTER CHARACTER POSITION OF QUALIFYING '',
     1''CODE FOR '',A8,'': '',\)') VAR
      CALL ERRCHK(1,ICODE,DUMMY)
      IF(ICODE.LT.1.OR.ICODE.GT.80) THEN
      CALL SOUND(256,100)
      WRITE(*,20)
      GO TO 30
      ENDIF
      WRITE(FIELD,'('',T'',I2,'',F'',I2,''.'',I2,'',T'',I2,'',A1'')')
     1IX,IWX,IXD,ICODE
      ELSE
      WRITE(FIELD,'('',T'',I2,'',F'',I2,''.'',I2)')
     1IX,IWX,IXD
      ENDIF
      CALL CONCAT(IFRMT,240,FIELD,18,FILERR)
      RETURN
      END
