$STORAGE:2
C  EQMODE PROGRAM FOR Q-MODE FACTOR ANALYSIS OF DATA MATRICES WITH
C   CONSTANT ROW-SUMS.  PRINCIPAL FUNCTION OF PROGRAM IS TO WRITE
C   A MASTER(E97) FILE TO USE AS INPUT TO OTHER INTERACTIVE EXTENDED
C   Q-MODE FACTOR ANALYSIS PROGRAMS.  FIVE OPTIONS ARE GIVEN FOR
C   SCALING THE COLUMNS OF THE DATA MATRIX PRIOR TO ROW-NORMALIZATION
C   AND EXTRACTION OF THE EIGENVALUES AND EIGENVECTORS.
C
C  BY A.T. MIESCH, USGS, DENVER.
C   THE MAIN PROGRAM AND SUBROUTINE TRANS ARE MODELED AFTER THE
C   CABFAC PROGRAM OF KLOVAN AND IMBRIE (1971).  SUBROUTINE VARMAX2
C   IS ALSO FROM KLOVAN AND IMBRIE (1971), BUT HAS BEEN MODIFIED
C   TO PRODUCE ONLY THE VARIMAX TRANSFORMATION MATRIX FOR WRITING
C   ON THE MASTER(E97) FILE.  SUBROUTINE EIGEN IS FROM PROGRAM D0097
C   OF THE U.S. GEOLOGICAL SURVEY, BY D.S. HANDWERKER.
C   PROGRAM EQMODE REQUIRES A STATPAC INPUT FILE.  THIS CAN BE
C   CREATED FROM A CARD DECK OR CHARACTER INPUT FILE WITH
C   PROGRAM CARFIL.  PROGRAM EQMODE IS USED IN LIEU OF PROGRAMS CABFAC
C   AND EQTRANS REFERRED TO IN COMPUTERS & GEOSCIENCES, VOL.2, P.439-
C   492.
C
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*12 UNIT10,UNIT11,UNIT14
      CHARACTER*4 IVID(37,2),ID(2),NAME(4),ITITLE(8)
      CHARACTER*1 IA(37),IANS,JY,IY
      INTEGER*4 LOC(2)
      REAL*4 XX(37)
      DIMENSION XMAX(37),XMIN(37),XB(37),XS(37),
     1X(37),A(37),B(37),E(37),
     2T(10,10),X1(37),X2(37),Y(37),Y1(37),Y2(37),
     3XY(37),C(10,37)
      COMMON/X111/ F(800,10)
      COMMON/X222/ R(37,37),RX(703),AX(1369),SUMSQ(800),RR(37,37)
C
C     KT=INPUT/OUTPUT TERMINAL
C     KI=STATPAC FILE OF INPUT DATA
C     KS1 AND KS2= SCRATCH FILES
C     KO=OUTPUT MASTER(E97) FILE (BINARY)
C     KST=STATPAC OUTPUT FILE FOR PLOTTING THE FACTOR-VARIANCE DIAGRAM
C         WITH PROGRAM CVDGRM$.BAS
C
C
      DATA IY/'Y'/,JY/'y'/,IBLANK/'    '/
      KT=0
      KI=10
      KO=11
      KS1=12
      KS2=13
      KST=14
      WRITE(KT,100)
  100 FORMAT(/,' GIVE 30-CHARACTER JOB TITLE')
      READ(KT,110) (ITITLE(I),I=1,8)
  110 FORMAT(7A4,A2)
      WRITE(KT,120)
  120 FORMAT(' HOW MANY SAMPLES ?')
      READ(KT,*) N
      FN=FLOAT(N)
      IF(N.LE.800) GO TO 150
      WRITE(KT,140)
  140 FORMAT(' PROGRAM IS PRESENTLY LIMITED TO 800 SAMPLES')
      STOP
  150 WRITE(KT,160)
  160 FORMAT(' HOW MANY VARIABLES ?')
      READ(KT,*) M
      IF(M.LE.37) GO TO 180
      WRITE(KT,170)
  170 FORMAT(' PROGRAM IS PRESENTLY LIMITED TO 37 VARIABLES')
      STOP
  180 WRITE(KT,190)
  190 FORMAT(' WHAT IS THE CONSTANT ROW-SUM IN THE DATA MATRIX ?')
      READ(KT,*) CONST
      IF(CONST.LE.0.0) GO TO 180
  200 WRITE(KT,210)
  210 FORMAT(' TYPE NUMBER 0 TO 5 TO SPECIFY A DATA TRANSFORMATION',/,
     1' 0  NO TRANSFORMATION',/,' 1  TRANSFORM TO PROPORTIONS OF MAXIMUM
     2 VALUES',/,' 2  TRANSFORM TO PROPORTIONS OF THE RANGES',/,' 3  TRA
     3NSFORM TO A SPECIFIED STD DEV',/,' 4  TRANSFORM TO SPECIFIED RELAT
     4IVE STD DEVS',/,' 5  VALUES OF B AND A WILL BE SUPPLIED FOR TRANSF
     5ORMATION (X-B)/(A-B)')
      READ(KT,*) ITRANS
      IF(ITRANS.LT.0.OR.ITRANS.GT.5) GO TO 200
C     WRITE(KT,1080) M
C1080 FORMAT(' ENTER',I3,' 8-CHARACTER VARIABLE NAMES',/,' END EACH WITH
C    * CR')
C     DO 120 I=1,M
C 120 READ(KT,1090) IVID(I,1),IVID(I,2)
C1090 FORMAT(2A4)
C**IF A TRANSFORMATION (SCALING) IS TO BE USED, GO TO SUBROUTINE TRANS
C  AND READ IN RAW DATA.  OTHERWISE, READ IN RAW DATA IN THE MAJOR LOOP
C  BELOW.
      IF(ITRANS.GE.1) CALL TRANS(N,M,ITRANS,XMIN,XMAX,CONST,A,B,IVID,
     *ID,LOC,IA)
      WRITE(*,215)
  215 FORMAT(' ENTER A NAME FOR THE MASTER(E97) FILE: ',\)
      READ(*,226) UNIT11
      OPEN(KO,FILE=UNIT11,STATUS='NEW',FORM='UNFORMATTED')
      WRITE(KO) ITITLE,N,M,ITRANS
      IF(ITRANS.GT.0) GO TO 240
      WRITE(KT,225)
  225 FORMAT(' ENTER NAME OF THE STATPAC INPUT FILE: ',\)
      READ(KT,226) UNIT10
  226 FORMAT(A12)
      OPEN(KI,FILE=UNIT10,STATUS='OLD',FORM='UNFORMATTED')
      READ(KI) ID,N1,M1,(IVID(I,1),IVID(I,2),I=1,M)
      IF(N1.EQ.N.AND.M1.EQ.M) GO TO 240
      WRITE(KT,230)
  230 FORMAT(' YOU HAVE THE WRONG INPUT FILE !!!')
      STOP
  240 WRITE(KO) ((IVID(I,J),J=1,2),I=1,M)
      DO 260 I=1,M
      XB(I)=0.0
      XS(I)=0.0
      XMAX(I)=-1.0E30
      XMIN(I)=1.0E30
      DO 260 J=1,M
      RR(I,J)=0.0
  260 R(I,J)=0.0
C**BEGIN MAJOR LOOP COMPUTING MEANS, STD DEVS, CORRELATIONS, AND COSINE
C  THETA OF RAW OR SCALED DATA, DEPENDING ON ITRANS.  WRITE RAW DATA
C  ON THE E97 FILE.
      OPEN(KS1,FILE='SCRATCH1',STATUS='NEW',FORM='UNFORMATTED')
      WRITE(*,265)
  265 FORMAT(' READING DATA.....',/)
      DO 370 I=1,N
      WRITE(*,134) I
  134 FORMAT('+',I5)
      SUMSQ(I)=0.0
      IF(ITRANS.GE.1) GO TO 280
      CALL JKGTLS(KI,IROW,NAME,LOC,XX,IA,M,IERR)
      IF(IERR.NE.0) STOP 'UNEXPECTED END-OF-FILE'
      DO 135 J=1,M
  135 X(J)=DBLE(XX(J))
C     READ(KI,1120) NAME(1),NAME(2),(X(J),J=1,M)
C1120 FORMAT(2A4,2X,10F7.0,/,(10X,10F7.0))
C     --------------------
C     STATEMENT 1120 CONTAINS THE INPUT FILE FORMAT
C     (ALSO SEE STATEMENT 1005 IN SUBROUTINE TRANS)
C     --------------------
C     SUM=0.0
C     DO 140 J=1,M
C 140 SUM=SUM+X(J)
C     SUM=SUM/CONST
C     DO 150 J=1,M
C 150 X(J)=X(J)/SUM
      DO 270 J=1,M
      DO 270 K=1,M
  270 RR(J,K)=RR(J,K)+X(J)*X(K)
      GO TO 290
  280 READ(KS2) (NAME(J),J=1,2),(X(J),J=1,M)
  290 DO 300 J=1,M
      XB(J)=XB(J)+X(J)
      XS(J)=XS(J)+X(J)**2
      IF(X(J).GT.XMAX(J)) XMAX(J)=X(J)
      IF(X(J).LT.XMIN(J)) XMIN(J)=X(J)
  300 SUMSQ(I)=SUMSQ(I)+X(J)**2
      SUMSQ(I)=DSQRT(SUMSQ(I))
      DO 310 J=1,M
  310 X(J)=X(J)/SUMSQ(I)
      WRITE(KS1) (X(J),J=1,M)
      DO 320 J=1,M
      DO 320 K=J,M
  320 R(J,K)=R(J,K)+X(J)*X(K)
      DO 330 J=1,M
  330 X(J)=X(J)*SUMSQ(I)
      IF(ITRANS.EQ.0) GO TO 350
      DO 340 J=1,M
  340 X(J)=X(J)*(A(J)-B(J))+B(J)
  350 WRITE(KO) (NAME(J),J=1,2),(X(J),J=1,M)
  370 CONTINUE
C**WRITE THE GENERAL STATISTICS FOR THE RAW OR TRANSFORMED DATA,
C  DEPENDING ON ITRANS.
      IF(ITRANS.GE.1) REWIND KS2
      REWIND KS1
      IF(ITRANS.GE.1) GO TO 390
      WRITE(KT,380)
  380 FORMAT(/,20X,' GENERAL STATISTICS FOR RAW DATA',/,
     14X,'VARIABLE',22X,'STANDARD',7X,'MINIMUM',8X,'MAXIMUM',/,
     220X,'AVERAGE',7X,'DEVIATION',7X,'VALUE',10X,'VALUE')
      GO TO 410
  390 WRITE(KT,400)
  400 FORMAT(/,20X,' GENERAL STATISTICS FOR TRANSFORMAED DATA',/,
     14X,'VARIABLE',22X,'STANDARD',7X,'MINIMUM',8X,'MAXIMUM',/,
     220X,'AVERAGE',7X,'DEVIATION',7X,'VALUE',10X,'VALUE')
  410 TRACE=0.0
      DO 440 I=1,M
      DO 420 J=I,M
  420 R(J,I)=R(I,J)
      XB(I)=XB(I)/FN
      XS(I)=DSQRT(XS(I)/FN-XB(I)**2)
      WRITE(KT,430) I,(IVID(I,J),J=1,2),XB(I),XS(I),XMIN(I),XMAX(I)
  430 FORMAT(1X,I2,1X,2A4,4F15.6)
  440 TRACE=TRACE+R(I,I)
      IF(ITRANS.NE.0) GO TO 500
C**COMPUTE THE CORRELATION MATRIX AND DETERMINE THE EIGENVALUES.
      DO 450 I=1,M
      DO 450 J=1,M
  450 RR(I,J)=(RR(I,J)/FN-XB(I)*XB(J))/(XS(I)*XS(J))
      DO 460 I=1,M
      DO 460 J=1,M
      IF(I.GT.J) GO TO 460
      L=I+(J*J-J)/2
      RX(L)=RR(I,J)
  460 CONTINUE
      CALL EIGEN(RX,AX,M,0)
      FM=FLOAT(M)
      CPM=0.0
      WRITE(KT,470)
  470 FORMAT(/,' THE EIGENVALUES OF THE CORRELATION MATRIX ARE',/,
     1' NO. EIGENVALUE      CPM')
      L=0
      DO 490 I=1,M
      L=L+I
      IF(RX(L).LT.0.0) RX(L)=0.0
      CPM=CPM+RX(L)/FM
      WRITE(KT,480) I,RX(L),CPM
  480 FORMAT(I3,2X,F11.6,1X,F11.6)
  490 CONTINUE
C**DETERMINE THE EIGENVALUES OF THE PSUEDO-COSINE THETA MATRIX.
  500 DO 510 I=1,M
      DO 510 J=1,M
      IF(I.GT.J) GO TO 510
      L=I+(J*J-J)/2
      RX(L)=R(I,J)
  510 CONTINUE
      CALL EIGEN(RX,AX,M,0)
      L=0
      DO 520 I=1,M
      L=L+I
  520 E(I)=RX(L)
      L=0
      DO 530 J=1,M
      DO 530 I=1,M
      L=L+1
  530 R(I,J)=AX(L)
      WRITE(KT,540)
  540 FORMAT(/,' THE NON-ZERO EIGENVALUES OF THE COSINE THETA MATRIX ARE
     1',/,' NO.   EIGENVALUE    CPN')
      SUM=0.0
      DO 550 I=1,M
      NF=I
      SUM=SUM+E(I)/TRACE
      WRITE(KT,480) I,E(I),SUM
      IF(E(I).LT.0.00001) GO TO 560
      IF(I.EQ.10.OR.I.EQ.M) GO TO 570
  550 CONTINUE
C**WRITE THE SCALING VALUES AND THE CONSTANT SUM ON THE E97 FILE.
  560 NF=NF-1
  570 IF(ITRANS.GT.0) GO TO 600
      DO 590 I=1,M
      B(I)=0.0
  590 A(I)=1.0
  600 WRITE(KO) (A(I),I=1,M)
      WRITE(KO) (B(I),I=1,M)
      WRITE(KO) CONST
      WRITE(KO) NF
C**COMPUTE PRINCIPAL COMPONENTS LOADINGS FROM NORMALIZED DATA AND MATRIX
C  OF EIGENVECTORS.  WRITE THESE ON E97 FILE.
      OPEN(KS2,FILE='SCRATCH2',STATUS='NEW',FORM='UNFORMATTED')
      WRITE(*,265)
      DO 660 I=1,N
      WRITE(*,134) I
      READ(KS1) (X(J),J=1,M)
      DO 640 J=1,NF
      F(I,J)=0.0
      DO 640 K=1,M
  640 F(I,J)=F(I,J)+X(K)*R(K,J)
      WRITE(KS2) (F(I,J),J=1,NF)
      WRITE(KO) (F(I,J),J=1,NF)
  660 CONTINUE
C**PREPARE TO WRITE COEFFICIENTS OF DETERMINATION ON AN OUTPUT FILE FOR
C  AUTOMATIC PLOTTING OF THE FACTOR-VARIANCE DIAGRAM.
      REWIND KS1
      SMIN=0.0
      WRITE(KT,670)
  670 FORMAT(/,' DO YOU WANT TO WRITE A STATPAC OUTPUT FILE FOR PLOTTING
     1 THE',/,' FACTOR-VARIANCE DIAGRAM WITH PROGRAM FVCPLT ?')
      READ(KT,680) IANS
  680 FORMAT(A1)
      IRITE=0
      IF(IANS.EQ.IY.OR.IANS.EQ.JY) IRITE=1
      NG=NF-1
      LOC(1)=0
      LOC(2)=0
      NAME(3)=IBLANK
      NAME(4)=IBLANK
      IF(IRITE.NE.1) GO TO 688
      WRITE(*,685)
  685 FORMAT(' ENTER A NAME FOR THE FV-DIAGRAM FILE: ',\)
      READ(*,226) UNIT14
      OPEN(KST,FILE=UNIT14,STATUS='NEW',FORM='UNFORMATTED')
      WRITE(KST) ID,NG,M,(IVID(I,1),IVID(I,2),I=1,M)
  688 DO 690 I=1,M
  690 SMIN=SMIN+B(I)
      CONST=CONST-SMIN
C**WRITE PRINCIPAL FACTOR SCORES (EIGENVECTORS) ON E97 FILE.
      DO 710 I=1,M
  710 WRITE(KO) (R(I,J),J=1,NF)
      WRITE(KT,720) NF
  720 FORMAT(' 2 TO',I3,' FACTORS WILL BE ROTATED')
      WRITE(KO) NF
C**BEGIN MAJOR LOOP COMPUTING RAW DATA (X), RECOMPUTED DATA (Y),
C  COEFFICIENTS OF DETERMINATION (C), AND ALL POSSIBLE VARIMAX
C  TRANSFORMATION MATRICES (T).
      DO 890 I=2,NF
      WRITE(KT,740) I
  740 FORMAT(' NOW ROTATING',I3,' FACTORS')
      NFF=I
      DO 750 J=1,M
      X1(J)=0.0
      X2(J)=0.0
      Y1(J)=0.0
      Y2(J)=0.0
  750 XY(J)=0.0
      DO 810 J=1,N
      READ(KS1) (X(K),K=1,M)
      DO 760 K=1,M
  760 X(K)=X(K)*SUMSQ(J)*(A(K)-B(K))+B(K)
      DO 770 K=1,M
      Y(K)=0.0
      DO 770 L=1,NFF
  770 Y(K)=Y(K)+F(J,L)*R(K,L)
      SFAC=0.0
      DO 780 K=1,M
  780 SFAC=SFAC+Y(K)*(A(K)-B(K))
      SFAC=CONST/SFAC
      DO 790 K=1,M
  790 Y(K)=Y(K)*SFAC*(A(K)-B(K))+B(K)
      DO 800 K=1,M
      X1(K)=X1(K)+X(K)
      X2(K)=X2(K)+X(K)**2
      Y1(K)=Y1(K)+Y(K)
      Y2(K)=Y2(K)+Y(K)**2
  800 XY(K)=XY(K)+X(K)*Y(K)
  810 CONTINUE
      DO 830 J=1,M
      X1(J)=X1(J)/FN
      X2(J)=DSQRT(X2(J)/FN-X1(J)**2)
      Y1(J)=Y1(J)/FN
      Y2(J)=Y2(J)/FN-Y1(J)**2
      IF(Y2(J).GT.0.0) GO TO 820
      C(I,J)=0.0
      GO TO 830
  820 Y2(J)=DSQRT(Y2(J))
      C(I,J)=((XY(J)/FN-X1(J)*Y1(J))/(X2(J)*Y2(J)))**2
  830 CONTINUE
      IF(IRITE.EQ.0) GO TO 850
      DO 840 J=1,M
  840 XX(J)=SNGL(C(I,J))
      NN=I-1
      CALL JKPTLS(KST,NN,NAME,LOC,XX,IA,M)
C     WRITE(KST,1278) I,(X(J),J=1,M)
C1278 FORMAT(I4,6X,10F7.4,/,(10X,10F7.4))
  850 REWIND KS1
      CALL VARMAX2(NFF,N,M,T)
      WRITE(KO) NFF
      DO 870 J=1,NFF
  870 WRITE(KO) (T(J,K),K=1,NFF)
      REWIND KS2
      DO 880 J=1,N
  880 READ(KS2) (F(J,K),K=1,NF)
  890 CONTINUE
C**WRITE COEFFICIENTS OF DETERMINATION FOR CONSTRUCTING THE
C  FACTOR-VARIANCE DIAGRAM.
      WRITE(KT,900)
  900 FORMAT(/,' DATA FOR THE FACTOR-VARIANCE DIAGRAM (COEFFS. OF DETERM
     1INATION)')
      WRITE(KT,910) (IVID(I,1),IVID(I,2),I=1,M)
  910 FORMAT(5(5X,2A4))
      FM=FLOAT(M)
      DO 960 I=2,NF
      WRITE(KT,920) I
  920 FORMAT(' FOR',I3,' FACTOR MODEL')
      WRITE(KT,930) (C(I,J),J=1,M)
  930 FORMAT(5(1X,F12.5))
      SUM=0.0
      DO 940 J=1,M
  940 SUM=SUM+C(I,J)
      SUM=SUM/FM
      WRITE(KT,950) SUM
  950 FORMAT(' AVERAGE = ',F12.5)
  960 CONTINUE
      WRITE(KT,970)
  970 FORMAT(' NORMAL END-OF-JOB  -  PROGRAM EQMODE')
      STOP
      END
