TITLE       SUBROUTINE FSTDSP(IROW,ICOL,STRING,LENGTH)
PAGE 66,132
;WRITES A FORTRAN CHARACTER STRING DIRECTLY INTO VIDEO RAM.
;CAN BE USED EITHER WITH A COLOR GRAPHICS ADAPTER OR A MONO-
;CHROME ADAPTER.
;ADAPTED BY W. D. GRUNDY FROM A TURBOPASCAL ROUTINE WRITTEN BY
;STEVE HALL, BEAVERTON, OREGON, AND PUBLISHED IN THE OCTOBER 1, 1985
;ISSUE OF PC MAGAZINE (PP 210,213).  HALL'S ALGORITHM IS "SNOW-FREE".
;
;      FORTRAN USAGE
;      INTEGER*2 IROW,ICOL,LENGTH
;      CHARACTER*N STRING           OR, ALTERNATIVELY,
;      CHARACTER*1 STRING(N)
;       .........
;       .........
;      CALL FSTDSP(IROW,ICOL,STRING,LENGTH)
;      WHERE 
;      IROW IS THE SCREEN ROW NUMBER
;      ICOL IS THE SCREEN COLUMN NUMBER
;      STRING IS THE FORTRAN TEXT TO BE DISPLAYED ON THE SCREEN
;      LENGTH = NUMBER OF FORTRAN CHARACTERS TO BE DISPLAYED.
;      LENGTH MUST BE GT 0
;      IROW MUST BE GE 0 AND LE 24
;      ICOL MUST BE GE 0 AND LE 79
;
BIOS_DATA   EQU        40H           ;SEGMENT FOR ROM BIOS DATA
EQUIP_FLAG  EQU        10H           ;OFFSET FOR EQUIPMENT STATUS
CRT_COLS    EQU        4AH           ;OFFSET TO GET NUMBER OF COLUMNS
                                     ;ON THE SCREEN.
ADDR_6845   EQU        63H           ;BASE ADDRESS FOR ACTIVE DISPLAY CARD
COLOR_CARD  EQU        0B800H        ;SEGMENT FOR COLOR/GRAPHICS CARD
MONO_CARD   EQU        0B000H        ;SEGMENT FOR MONOCHROME CARD
;
FRAME       STRUC
SAVESI      DW         ?             ;SAVE AREA FOR CALLER'S SI
SAVEDI      DW         ?             ;SAVE AREA FOR CALLER'S DI
SAVEDS      DW         ?             ;SAVE AREA FOR CALLER'S DS
SAVEBP      DW         ?             ;SAVE AREA FOR CALLER'S BP
SAVERET     DD         ?             ;SAVE AREA FOR RETURN ADDRESS
D           DD         ?             ;POINTER TO LENGTH
C           DD         ?             ;POINTER TO STRING
B           DD         ?             ;POINTER TO SCREEN COLUMN
A           DD         ?             ;POINTER TO SCREEN ROW
FRAME       ENDS
;
_DATA       SEGMENT    'DATA'
_DATA       ENDS
;
FSTD_TEXT   SEGMENT    'CODE'
DGROUP      GROUP      _DATA
            ASSUME     CS:FSTD_TEXT,DS:DGROUP,SS:DGROUP
            PUBLIC     FSTDSP
FSTDSP      PROC       FAR
            PUSH       BP            ;SAVE CALLER'S BP
            PUSH       DS            ;SAVE CALLER'S DS
            PUSH       DI
            PUSH       SI
            MOV        BP,SP         ;POINT TO ADDRESSES PASSED BY CALLER
            LES        BX,[BP]+B     ;GET ADDRESS OF COLUMN
            MOV        DI,ES:[BX]    ;STORE COLUMN NUMBER
            LES        BX,[BP]+A     ;GET ADDRESS OF ROW
            MOV        AX,ES:[BX]    ;STORE ROW NUMBER
            LES        BX,[BP]+D     ;GET ADDRESS OF LENGTH
            MOV        CX,ES:[BX]    ;STORE LENGTH
            CMP        CX,0          ;TEST FOR VALID LENGTH
            JE         EXIT
            LDS        SI,[BP]+C     ;POINT TO STRING
            MOV        BX,BIOS_DATA  ;CHECK VIDEO CARD TYPE
            MOV        ES,BX         ;CURRENT COLUMN SETTING
            MUL        ES:CRT_COLS   ;SET CARD MODE
            ADD        DI,AX
            SHL        DI,1          ;ATTRIBUTE BYTE
            MOV        DX,ES:ADDR_6845
            ADD        DX,6          ;POINT TO STATUS PORT
            MOV        AX,COLOR_CARD ;FIRST TRY COLOR CARD
            MOV        BX,ES:EQUIP_FLAG
            AND        BX,30H        ;CHECK FOR CARD TYPE
            CMP        BX,30H        ;TEST FOR MONO CARD
            JNE        SETCARD
            MOV        AX,MONO_CARD  ;ELSE IT IS A MONO CARD
SETCARD:    MOV        ES,AX         ;POINT ES TO VIDEO
TEST_LO:    IN         AL,DX         ;GET STATUS
            TEST       AL,1          ;IS IT LOW?
            JNZ        TEST_LO       ;NO, KEEP CHECKING
            CLI                      ;TURN OFF INTERRUPTS
TEST_HI:    IN         AL,DX         ;GET STATUS
            TEST       AL,1          ;IS IT HIGH?
            JZ         TEST_HI       ;NO, KEEP CHECKING
            MOVSB                    ;PROPER TIME TO DISPLAY
            INC        DI            ;SKIP ATTRIBUTE BYTE
            LOOP       TEST_LO
EXIT:       STI                      ;TURN ON INTERRUPTS
            POP        SI
            POP        DI
            POP        DS            ;RESTORE CALLER'S DS
            POP        BP            ;RESTORE CALLER'S BP
            RET        16            ;FAR RETURN.  RELEASE 16 BYTES
FSTDSP      ENDP
FSTD_TEXT   ENDS
            END
