TITLE     SUBROUTINE INTR(INTNO,REGS)
; THIS SUBROUTINE MIMICS THE ACTION OF TURBOPASCAL FUNCTION "INTR".
; EXECUTES BIOS AND DOS INTERRUPTS USING REGISTER VALUES SPECIFIED
; BY A FORTRAN CALLING PROGRAM.  THE FORTRAN CALLING PROGRAM MUST
; CONTAIN THE FOLLOWING "INCLUDE" FILE:
;
;C***  INCLUDE FILE FOR FORTRAN ROUTINES WHICH CALL DOS OR BIOS SERVICES
;      COMMON /RGSTRS/ AX,BX,CX,DX,BP,SI,DI,DS,ES,FLAGS
;      INTEGER*4 AX,BX,CX,DX,BP,SI,DI,DS,ES,FLAGS
;      EQUIVALENCE (AX,REGS)
;C***  END OF INCLUDE FILE PCREGS.INC
;
;      SAMPLE FORTRAN CALLS
;      AX=#0600                {REQUEST SCREEN SCROLL UP ENTIRE WINDOW}
;      CX=#0000                {FROM ROW 0, COLUMN 0}
;      DX=#184F                {TO ROW 24, COLUMN 79}
;      BX=#0700                {NORMAL WHITE ON BLACK}
;      CALL INTR(#10,REGS)     {INVOKE BIOS VIDEO SERVICES}
;      AX=#0200                {REQUEST SET CURSOR POSITION}
;      DX=#0000                {PLACE AT ROW 0, COLUMN 0}
;      BX=#0000                {ON SCREEN PAGE 0}
;      CALL INTR(#10,REGS)
;
; CODED BY W. D. GRUNDY  AUGUST  1986, REVISED APRIL  1987
;
FRAME          STRUC
SAVEBP         DW              ?
SAVERET        DD              ?            ; CALLER'S RETURN
REGS           DD              ?            ; POINTS TO REGISTERS
INTNO          DD              ?            ; POINTS TO INTERRUPT NO.
FRAME          ENDS
;
_DATA          SEGMENT         WORD PUBLIC 'DATA'
_DATA          ENDS
;
INTR_TEXT      SEGMENT         BYTE PUBLIC 'CODE'
DGROUP         GROUP           _DATA
               ASSUME          CS:INTR_TEXT,DS:DGROUP,SS:DGROUP
               PUBLIC          INTR  
INTR           PROC            FAR
               PUSH            BP
               MOV             BP,SP
               PUSH            DS
               PUSH            DI
               PUSH            SI
               LES             DI,[BP][INTNO]   ; POINT TO INTNO.
               MOV             AX,ES:[DI]       ; INTNO IS NOW IN AL
               XCHG            AH,AL            ; BUILD UP THE
               MOV             AL,0CDH          ; REQUESTED INTERRUPT
               LEA             BX,REPLACE       ; AND STORE IT
               MOV             CS:[BX],AX       ; AT LOCATION "REPLACE"
;
               LES             BX,[BP][REGS]    ; POINT TO REGS.
               MOV             AX,ES:[BX]       ; LOAD THE REGISTERS
               MOV             CX,ES:[BX]+8     ; WITH VALUES SUPPLIED
               MOV             DX,ES:[BX]+12    ; BY CALLING PROGRAM.
               MOV             SI,ES:[BX]+20
               MOV             DI,ES:[BX]+24
               MOV             DS,ES:[BX]+28
               MOV             BX,ES:[BX]+4
               PUSH            BX
               LES             BX,[BP][REGS]
               MOV             ES,ES:[BX]+32
               POP             BX
;
               PUSH            BP               ; MAKE SURE BP IS NOT
                                                ; CLOBBERED.
REPLACE:       DW              ?                ; ACTUAL INTERRUPT HERE
               POP             BP
;
               PUSH            ES               ; NEED ES AND BX REGS
               PUSH            BX               ; TO RETURN VALUES.
;
; RETURN VALUES TO CALLING PROGRAM REGISTER-BY-REGISTER
;
               LES             BX,[BP]+REGS
               MOV             ES:[BX],AX       ; AX RETURNED
               POP             AX
               MOV             ES:[BX]+4,AX     ; BX
               MOV             ES:[BX]+8,CX
               MOV             ES:[BX]+12,DX
               MOV             ES:[BX]+16,BP
               MOV             ES:[BX]+20,SI
               MOV             ES:[BX]+24,DI
               MOV             ES:[BX]+28,DS
               POP             AX
               MOV             ES:[BX]+32,AX    ; ES
               PUSHF
               POP             AX
               MOV             ES:[BX]+36,AX    ; FLAGS
               POP             SI
               POP             DI
               POP             DS
               POP             BP
               RET             8                ; RELEASE 8 BYTES
INTR           ENDP
INTR_TEXT      ENDS
               END
