$STORAGE:2
$NOFLOATCALLS
C     PROGRAM LNCLUST&.FOR OF R.G. GARRETT.
C     GEOCHEMISTRY SECTION, GEOLOGICAL SURVEY OF CANADA, OTTAWA
C
C     PROGRAM TO CLUSTER DATA INTO HOMOGENOUS GROUPS AFTER THE
C     ALGORITHM OF SPATH (COMPUTING 11, 175-177 (1973)).  BASED ON THE
C     COMPLETE ENUMERATION METHOD OF THE N-1, K-1 PARTITIONS AS
C     DESCRIBED BY FISHER (J. AM. STAT. ASS. 58, 789-798 (1958)).
C
C     THE PROGRAM WILL GROUP UP TO 5000 DATA ITEMS (SAMPLES) INTO ANY
C     NUMBER OF GROUPS BETWEEN 1 AND 20.  THE RAW DATA MAY BE LOG10 OR
C     RANGE TRANSFORMED, OR BOTH.  THE RANGE TRANSFORM LEADS TO OPTIMAL
C     NUMERICAL STABILITY AND IS RECOMMENDED.
C
C    ORIGINALLY PROGRAMMED FOR CDC 6400 (FORTRAN 4, SCOPE 3.4)
C     R.G. GARRETT, FEBRUARY 1974.
C     ADAPTED TO USGS STATPAC AND IBM PC BY A.T. MIESCH, USGS,
C       GRAND JUNCTION, COLO.  FEBRUARY, 1985
C
C     LINK WITH: IOS, RSORT
C
      CHARACTER*32 UNIT15
      CHARACTER*4 ID(2),IVID(60,2),NAME(4)
      CHARACTER*1 IANS,IY,JY,IA(60)
      INTEGER W(40)
      INTEGER*4 LOC(2)
      DIMENSION X(5000),XX(60)
C
      COMMON X,N,W,DIFF,XLOW,IFL,IFR,IFT
C
C
      DATA IY/'Y'/,JY/'y'/
      KI=15
      KP=16
      WRITE(*,10)
   10 FORMAT(' ENTER NAME OF THE STATPAC INPUT FILE: ',\)
      READ(*,20) UNIT15
   20 FORMAT(A32)
      OPEN(KI,FILE=UNIT15,STATUS='OLD',FORM='UNFORMATTED')
      OPEN(KP,FILE='LPT1',STATUS='NEW')
C
   30 READ(KI) ID,N,MSTP,(IVID(I,1),IVID(I,2),I=1,MSTP)
      IF(N.GT.5000) STOP 'PROGRAM IS LIMITED TO 5,000 SAMPLES'
      IF(MSTP.GT.60) STOP 'PROGRAM IS LIMITED TO 60 VARIABLES'
   35 WRITE(*,40)
   40 FORMAT(' WHICH VARIABLE DO YOU WANT TO EXAMINE? ',\)
      READ(*,*) MM
      IF(MM.LT.1.OR.MM.GT.MSTP) GO TO 35
      WRITE(*,50)
   50 FORMAT(' THE DATA SHOULD BE SPLIT INTO AT LEAST HOW MANY GROUPS? '
     *,\)
      READ(*,*) MING
      IF(MING.LT.1) MING=1
      IF(MING.GT.20) MING=20
      IF(MING.GT.N) MING=N
      WRITE(*,60)
   60 FORMAT(' WHAT IS THE MAXIMUM NO. OF GROUPS? ',\)
      READ(*,*) MAXG
      IF(MAXG.GT.20) MAXG=20
      IF(MAXG.GT.N) MAXG=N
      IF(MING.GT.MAXG) MING=MAXG
      WRITE(*,70)
   70 FORMAT(' DO YOU WANT THE LOG10 TRANSFORMATION? ',\)
      READ(*,80) IANS
   80 FORMAT(A1)
      IFL=0
      IF(IANS.EQ.IY.OR.IANS.EQ.JY) IFL=1
      WRITE(*,90)
   90 FORMAT(' DO YOU WANT THE RANGE TRANSFORMATION? ',\)
      READ(*,80) IANS
      IFR=0
      IF(IANS.EQ.IY.OR.IANS.EQ.JY) IFR=1
      IFT=0
      IF(IFL.EQ.1.OR.IFR.EQ.1) IFT=1
C***  CLEAR EPSON OF PREVIOUS MODES
      WRITE(KP,'('' '',2A1,\)') CHAR(27),'@'
      WRITE(KP,'('' '',2A1,\)') CHAR(27),'G'
      WRITE(KP,'('' '',A1,10X,''UNIVARIATE GROUPING'')') CHAR(14)
      WRITE(KP,'(/,'' '',A1,10X,''BY LINEAR CLUSTERING'')') CHAR(14)
      WRITE(KP,'(/,'' '',A1,14X,''USGS STATPAC'')') CHAR(14)
      WRITE(KP,'('' '',2A1,\)') CHAR(27),'H'
C***  PUT EPSON PRINTER INTO COMPRESSED MODE
      WRITE(KP,'('' '',A1\)') CHAR(15)
      WRITE(KP,230) UNIT15,IVID(MM,1),IVID(MM,2),N,MING,MAXG
C
C     READ IN DATA, ORDER, COMPUTE AND PRINT SUMMARY STATISTICS
      WRITE(*,110)
  110 FORMAT(' READING DATA.....',/)
      DO 150 I=1,N
      WRITE(*,120) I
  120 FORMAT('+',I5)
      CALL JKGTLS(KI,IROW,NAME,LOC,XX,IA,MSTP,IERR)
      IF(IERR.NE.0) STOP 'UNEXPECTED END-OF-FILE'
      IF(IA(MM).EQ.' ') GO TO 140
      WRITE(*,130)
  130 FORMAT(' THIS VARIABLE IS QUALIFIED!!!')
      GO TO 210
  140 X(I)=XX(MM)
  150 CONTINUE
      WRITE(*,155)
  155 FORMAT(' SORTING.....')
      CALL RSORT(X,N)
      WRITE (KP,240) X(1),X(N)
      CALL STATS(1,N,1,0)
C
C     TRANSFORM DATA, COMPUTE AND PRINT SUMMARIES OF STATISTICS
      IF(IFL.EQ.0)GO TO 170
      DO 160 I=1,N
  160 X(I)=ALOG10(X(I))
      CALL STATS(1,N,2,0)
  170 IF(IFR.EQ.0)GO TO 190
      XLOW=X(1)
      XHIH=X(N)
      DIFF=XHIH-XLOW
      DO 180 I=1,N
  180 X(I)=(X(I)-XLOW)/DIFF
      CALL STATS(1,N,3,0)
C
C CLUSTER DATA INTO HOMOGENOUS GROUPS
  190 DO 200 K=MING,MAXG
      WRITE(KP,250) K,IVID(MM,1),IVID(MM,2)
      WRITE(KP,260)
      IF(IFT.EQ.1)WRITE (KP,270)
      WRITE(*,195) K
  195 FORMAT(' LOOKING FOR THE ',I2,' BEST CLUSTERS.....')
      CALL CLUSTER(K)
      DO 200 KK=1,K
      K2=KK*2
      K1=K2-1
      N1=W(K1)
      N2=W(K2)
      CALL STATS(N1,N2,4,KK)
  200 CONTINUE
C
  210 REWIND KI
      WRITE(KP,215)
  215 FORMAT(1X)
      WRITE(*,220)
  220 FORMAT(' ANOTHER VARIABLE? ',\)
      READ(*,80) IANS
C***  RESTORE EPSON TO NORMAL MODE
      WRITE(KP,'('' '',A1\)') CHAR(18)
      IF(IANS.EQ.IY.OR.IANS.EQ.JY) GO TO 30
      STOP 'NORMAL END OF PROGRAM'
C
  230 FORMAT (///,' FILE=',
     1A32,/,5X,
     2'VARIABLE',22X,2A4,2X,/,5X,'NO. OF ITEMS',18X,I10,/,5X,'MINIMUM NO
     3. OF GROUPS',9X,I10,/,5X,'MAXIMUM NO. OF GROUPS',9X,I10)
  240 FORMAT (/,5X,'MINIMUM OBSERVED RAW VALUE',4X,F10.2,/,5X,'MAXIMUM O
     1BSERVED RAW VALUE',4X,F10.2,//,20X,'SUMMARY STATISTICS FOR DATA',/
     2,5X,'TRANSFORM',6X,'MEAN',5X,'VARIANCE',5X,'STD DEV',4X,'CO OF VAR
     3')
  250 FORMAT (/,44X,'UNIVARIATE GROUPING BY LINEAR CLUSTERING',
     1,/,23X,I2,' GROUP MODEL FOR: ',2A4)
  260 FORMAT(/,50X,'SUMMARY STATISTICS FOR GROUPS',/,
     35X,'GROUP',5X,'ITEMS',5X,'FREQUENCY',5X,'GROUP LIMITS',6X,'MEAN',
     44X,'VARIANCE',5X,'STD DEV',5X,'CO OF VAR')
  270 FORMAT ('+',103X,'UNTRANSFORMED LIMITS AND MEAN')
      END
      SUBROUTINE STATS(N1,N2,NP,NC)
C     CALLED BY LNCLUST PROGRAM OF R.G. GARRETT.
C
      CHARACTER*5 HOL(3)
      DIMENSION X(5000),IDUM(40)
      COMMON X,NN,IDUM,DIFF,XLOW,IFL,IFR,IFT
      DATA HOL/'NONE ','LOG10','RANGE'/
      KP=16
      SUMXS=0.0
      SUMX=0.0
      N=N2-N1+1
      IF(N.LE.1)GO TO 3
      DO 1 I=N1,N2
      SUMX=SUMX+X(I)
    1 SUMXS=SUMXS+X(I)*X(I)
      BAR=SUMX/N
      VAR=(SUMXS-BAR*SUMX)/(N-1.0)
      IF(VAR.GE.0.0) SDV=SQRT(VAR)
      IF(VAR.LT.0.0) SDV=0.0
      CV=SDV/BAR
      GO TO (2,2,2,4),NP
    2 WRITE (KP,101) HOL(NP),BAR,VAR,SDV,CV
      GO TO 8
    3 BAR=X(N1)
      VAR=0.0
      SDV=0.0
      CV=0.0
    4 FR=100.0*N/NN
      IF(IFT.EQ.1)GO TO 5
      WRITE (KP,102) NC,N,FR,X(N1),X(N2),BAR,VAR,SDV,CV
      GO TO 8
    5 XL=X(N1)
      XH=X(N2)
      WRITE (KP,103) NC,N,FR,XL,XH,BAR,VAR,SDV,CV
      IF(IFR.EQ.0)GO TO 6
      XL=(XL*DIFF)+XLOW
      XH=(XH*DIFF)+XLOW
      BAR=(BAR*DIFF)+XLOW
    6 IF(IFL.EQ.0)GO TO 7
      XL=10**XL
      XH=10**XH
      BAR=10**BAR
    7 WRITE (KP,104) XL,XH,BAR
    8 RETURN
  101 FORMAT (6X,A5,4F12.4)
  102 FORMAT (1X,I7,I11,F13.2,2F10.2,F9.2,F11.4,F12.4,F13.4)
  103 FORMAT (1X,I7,I11,F13.2,F11.4,F10.4,F8.4,F10.4,2F13.4)
  104 FORMAT ('+',98X,2F12.2,F10.2)
      END
      SUBROUTINE CLUSTER(K)
C     CALLED BY LNCLUST PROGRAM OF R.G. GARRETT.
C     LINEAR CLUSTERING ALGORITHM OF SPATH.  ORIGINAL ALGOL CODING
C     TRANSLATED TO FORTRAN 4 BY GWYNNETH FREEMAN, COMPUTER SCIENCE
C     CENTRE, E.M.R., OTTAWA.
C
C    A SET OF N ORDERED REAL NUMBERS IS PARTITIONED BY COMPLETE
C     ENUMERATION INTO K CLUSTERS SUCH THAT THE SUM OF THE SUM OF
C     SQUARED DEVIATIONS FROM THE MEAN WITHIN EACH CLUSTER IS A MINIMUM.
C
      DIMENSION X(5000),V(20),W(40),U(5000)
      COMMON X,N,W
      INTEGER A,B,C,D,U,V,W,AP1
      E=1.E30
      DO 1 J=1,K
    1 V(J)=1
      V(K)=N-K+1
      L=N
    2 J=K
      IF(L.NE.N)GO TO 6
      A=0
      H=0
      DO 4 I=1,K
      C=V(I)
      B=A+C
      F=0.0
      G=0.0
      AP1=A+1
      DO 3 D=AP1,B
      Y=X(D)
      U(D)=I
      F=F+Y
    3 G=G+Y*Y
      H=H+G-F*F/C
      IF(H.GT.E)GO TO 6
    4 A=B
      E=H
      KK=1
      K1=1
      KN=K*2
      W(1)=1
      W(KN)=N
      DO 5 I=1,N
      IF(U(I).EQ.KK)GO TO 5
      KK=KK+1
      K1=K1+2
      K2=K1-1
      W(K1)=I
      W(K2)=I-1
    5 CONTINUE
    6 M=V(J)+1
      V(J)=V(J)+1
      L=L+1
      IF(L.LE.N)GO TO 2
      IF(J.EQ.1)RETURN
      L=L-M+1
      V(J)=1
      J=J-1
      GO TO 6
      END
