$NOFLOATCALLS
$STORAGE:2
      PROGRAM SS2DDRES
C***  THIS PROGRAM COMPUTES DRIFT RESIDUALS BASED ON AN ASSUMED
C***  VARIOGRAM AND DRIFT FORM WHICH ARE INPUT INTERACTIVELY BY
C***  THE USER.
C***  THIS PROGRAM WAS ADAPTED BY W. D. GRUNDY FROM THE KRIGING
C***  PACKAGE OF SKRIVAN AND KARLINGER AND PROGRAM UKRIG OF KIM
C***  AND KNUDSEN.
C***  THE STATPAC INPUT FILE FOR THIS PROGRAM MUST HAVE BEEN CREATED
C***  BY PROGRAM SS2DPREP.
C***  THIS PROGRAM IS USED TO COMPUTE GENERALIZED LEAST SQUARES
C***  DRIFT RESIDUALS WHICH ARE TO BE USED AS INPUT TO PROGRAM 
C***  SS2DGAMH.  THE VARIOGRAM OF RESIDUALS OBTAINED FROM SS2DGAMH
C***  IS THEN ENTERED BACK INTO SS2DDRES TO OBTAIN A NEW SET OF
C***  DRIFT RESIDUALS, FROM WHICH ANOTHER EXPERIMENTAL VARIOGRAM
C***  IS OBTAINED.  THIS ITERATIVE PROCESS IS REPEATED UNTIL THE
C***  EXPERIMENTAL VARIOGRAM "STABILIZES" OR CHANGES ONLY VERY 
C***  SLIGHTLY WITH ADDITIONAL ITERATIONS.  THIS FINAL VARIOGRAM
C***  IS THE VARIOGRAM TO BE USED FOR UNIVERSAL KRIGING.  THIS
C***  PROGRAM ALSO REQUIRES USER INPUT OF THE EXPONENTS OF THE
C***  DRIFT TERMS TO BE USED.
C***
C***  A SUGGESTED STARTING VARIOGRAM TO GET GOING IN THE ITERATIVE
C***  PROCESS IS AN ALL NUGGET MODEL USING THE EXPERIMENTAL VARIANCE
C***  OF THE REGIONALIZED VARIABLE AS A STARTING POINT.
C***
C***  LINKS REQUIRED: ASCIIZ, CHEKER, CHRKNT, CKSAFE, CLEAR, CLREOL,
C***                  CONCAT, COVAR4, CRSOFF, CRSON, DATE, DELAY,
C***                  DOLITL, DRFTRM, DSORT, ERRCHK, FNDCUR, FSTDSP,
C***                  GETFLE, INT21H, INTR, HEADER, KRIG4C, MDYHMS,
C***                  MOVCUR, NEIGH, ODOMTR, OUTGEO, PACK, PCHCEN,
C***                  PRECHK, PSET4, SAVRST, SCREAD, SOUND, SRCHPB,
C***                  TERM, TICKER, TIME, UNPK, VALDOP, VARIN4
C***  NOTE: ABOVE OBJECT MODULES ARE CONTAINED IN STATPAC.LIB AND
C***        NEED NOT BE RECOMPILED OR REASSEMBLED.
C***  NOTE:SUBROUTINE FSTDSP WRITES DIRECTLY TO VIDEO RAM
C***        COMPILE SS2DDRES WITH FILE PTKRIG.INC PRESENT.
C***  MAJOR VARIABLES AND DIMENSION REQUIREMENTS
C***
C***    NEST         NUMBER OF VARIOGRAMS IN MODEL (MAX=5)
C***    C0           NUGGET EFFECT
C***    C(NEST)      C-VALUES OF VARIOGRAMS
C***    AVALU(NEST)  A-VALUES OF VARIOGRAMS
C***                 IF VARIOGRAM IS OF FORM GAM(H)=C)+SLOPE*H**A
C***                 AVALU REPRESENTS THE POWER A.  OTHERWISE
C***                 IT IS THE VAROGRAM RANGE.
C***    ANGLE(NEST)  ANGLES OF ROTATION OF ANISOTROPIC VARIOGRAMS ABOUT
C***                 EAST-WEST AXIS.  EAST IS TAKEN AS ZERO DEGREES.
C***                 ANGLES ARE ENTERED IN DECIMAL DEGREES.
C***                 NEGATIVE ANGLE INDICATES CLOCKWISE ROTATION.
C***                 POSITIVE ANGLE INDICATES COUNTER-CLOCKWISE ROTATION
C***                 IF GEOMETRIC ANISOTROPY IS PRESENT, ANGLE IS THE
C***                 BEARING OF THE MAJOR AXIS OF THE ELLIPSE.
C***    AFH(NEST)    ANISOTROPY RATIOS OF VARIOGRAMS
C***                 AFH.NE.1  INDICATES GEOMETRIC ANISOTROPY
C***                 AFH.EQ.1   INDICATES ISOTROPIC VARIOGRAM
C***    SILL         C0 PLUS SUM OF C'S
C***    ITYPE(NEST) VARIOGRAM TYPE
C***                =1 INDICATES SPHERICAL VARIOGRAM
C***                =2 INDICATES EXPONENTIAL VARIOGRAM
C***                =3 INDICATES LINEAR VARIOGRAM
C***                =4 INDICATES GAUSSIAN VARIOGRAM
C***                =5 INDICATES CUBIC VARIOGRAM
C***    AISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    BISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    CISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    SLOPE(NEST) SLOPES OF LINEAR VARIOGRAMS
C***                ZERO SLOPE INDICATES VARIO-
C***                GRAM IS NOT A LINEAR TYPE.
C***
C***    NT           NUMBER OF DRIFT TERMS (MAX=10)
C***    EX(NT)       X-EXPONENTS OF DRIFT TERMS
C***    EY(NT)       Y-EXPONENTS OF DRIFT TERMS
C***    COEF(NT)     COEFFICIENTS OF DRIFT TERMS
C***
C***    MAXH         MAXIMUM NUMBER OF DRILL HOLES WHICH ARE ALLOWED TO
C***                 BE READ IN FROM THE STATPAC FILE.  IN THIS PROGRAM
C***                 MAXH IS SET TO 1500
C***    IHOLID(MAXH) ALPHAMERIC DRILL HOLE IDENTIFIERS
C***    XE(MAXH)     ARRAY OF X-COORDINATES (EASTINGS) OF DRILL HOLES
C***    YN(MAXH)     ARRAY OF Y-COORDINATES (NORTHINGS) OF DRILL HOLES
C***    GD(MAXH)     ARRAY OF SAMPLE MEASUREMENTS OF DRILL HOLES
C***    ERMEAS(MAXH) ARRAY OF MEASUREMENT VARIANCES OF GD
C***
C***  /COV4/
C***    MAXK         MAXIMUM NUMBER OF HOLES IN NEIGHBORHOOD WHICH CAN
C***                 BE USED IN KRIGING A POINT.  IN THIS PROGRAM MAXK
C***                 IS SET TO 150.
C***    P(KDIM)      COVARIANCE AND DRIFT TERM MATRIX
C***                 KDIM=(MAXK+NT)*(MAXK+NT+1)/2
C***    ID(MAXK+NT)  AN ARRAY OF POINTERS FOR MANIPULATING MATRIX P
C***    W(MAXK+NT)   ARRAY OF KRIGING WEIGHTS AND LAGRANGIAN MULTIPLIERS
C***    IHOL(MAXK+1) ARRAY OF POINTERS TO HOLES TO BE USED IN KRIGING
C***                 A PARTICULAR LOCATION
C***    R(MAXK+NT)   COLUMN VECTOR OF KRIGING SYSTEM
C***
C***    IPT          STATPAC INPUT FILE
C***    IOUT         STATPAC OUTPUT FILE
C***    IPR          CHARACTER OUTPUT FILE
C***    NHOL         ACTUAL NUMBER OF DRILL HOLES OBTAINED FROM STATPAC
C***    IVIB(2,99)   ARRAY OF IDENTIFIERS OF STATPAC FILE VARIABLES
C***
C***    DIST(MAXK+1) ARRAY OF DISTANCES OF DRILL HOLES FROM POINT BEING
C***                 KRIGED.
C***    B(2)         TEMPORARY STORAGE FOR COORDINATES OF POINT BEING
C***                 KRIGED
C***    NK           USER-SPECIFIED ACTUAL MAXIMUM NUMBER OF HOLES TO BE
C***                 USED IN KRIGING A POINT
C***    RMAX         SEARCH RADIUS (NEIGHBORHOOD) ABOUT POINT BEING KRIGED.
C***
$INCLUDE:'PTKRIG.INC'
      INTEGER*4 ITIME1,ITIME2
      CHARACTER*32 UNIT10,UNIT14,UNIT21
      CHARACTER*4 EXT1,EXT2,WKDAY
      CHARACTER*10 HMS
      CHARACTER*8 IDSV(6),IDST,MDY
      LOGICAL FLAG
      DIMENSION B(2),DUMMY(6)
      DATA IDSV/'X-COORD.','Y-COORD.','ASAY VAL','ERMEASUR',
     1'VAL DRFT','RESIDUAL'/
      DATA EXT1/'.FDR'/,EXT2/'.UDR'/
C***  INITIATE TIMING OF RUN
      CALL DATE(WKDAY,MDY)
      CALL TICKER(ITIME1)
      CALL TIME(HMS)
      IPT=10
      IPR=21
      IOUT=14
      ISCRCH=13
      MAXH=1500
      MAXK=150
C***  OPEN IO FILES
      CALL VALDOP(ISCRCH,UNIT10,UNIT14,UNIT21,EXT1,EXT2,FLAG)
C***  GET DRIFT TERMS
      CALL DRFTRM(1)
C***  GET VARIOGRAM PARAMETERS
      CALL VARIN4
C***  GET NEIGHBORHOOD LIMITS
      CALL NEIGH(RMAX,NK,NT,MAXK)
C***  GET STATPAC FILE
      CALL GETFLE(IGD)
C***  CHECK TO SEE IF STATPAC INPUT FILE WAS PREPROCESSED; IF NOT,
C***  HALT EXECUTION OF THE PROGRAM
      CALL PRECHK(YN,XE,NHOL)
C***  WRITE OUT HEADERS FOR VARIOGRAMS AND DRIFT TERMS
      IF(FLAG) CALL HEADER(IPR,RMAX,NK,WKDAY,MDY,HMS)
C***  ALLOW USER TWO SECONDS TO VIEW THE SCREEN BEFORE CLEARING
      CALL DELAY(200)
      CALL CLEAR
      CALL HEADER(0,RMAX,NK,WKDAY,MDY,HMS)
C***  WRITE HEADERS FOR ORIGINAL DATA AND RESIDUALS
      IF(FLAG) WRITE(IPR,10) IVIB(1,IGD),IVIB(2,IGD)
   10 FORMAT(//,6X,'HOLE',5X,'X-COORD.',3X,'Y-COORD.',3X,2A4,2X,
     1'ERMEAS',3X,'VDRIFT',3X,'RESIDUAL',//)
C***  START ESTIMATING THE DRIFT RESIDUALS
      R2=RMAX*RMAX
      WRITE(*,20)
   20 FORMAT(//)
      KOUNT=0
      KSTART=0
      WRITE(*,'(A)')'           COMPUTING DRIFT RESIDUALS'
      CALL FNDCUR(IRRR,ICCC,0)
      DO 100 J=1,NHOL
      B(1)=XE(J)
      B(2)=YN(J)
C***  FIND NEAREST NK HOLES FROM POINT J
      ISTART=KSTART+1
      CALL SRCHPB(B(1),B(2),NS,NK,R2,RMAX,ISTART,KSTART)
C***  INSURE THERE IS AT LEAST ONE DEGREE OF FREEDOM
      IF(NS.LE.NT) GO TO 100
C***  ESTIMATE THE DRIFT COEFFICIENTS IF THERE ARE ENOUGH SAMPLE POINTS
      CALL KRIG4C(NS,IFERR)
      IF(IFERR.EQ.0) GO TO 60
      IF(FLAG) WRITE(IPR,30) J
   30 FORMAT(' ERROR IN MATRIX SOLUTION POINT ',I5)
      DO 50 M=1,NS
      LL=IHOL(M)
      IF(FLAG) WRITE(IPR,40) IHOLID(LL),YN(LL),XE(LL)
   40 FORMAT(1X,A8,3F10.2)
   50 CONTINUE
      GO TO 100
C***  CALCULATE RESIDUALS
   60 SUM=0.
      DO 70 M=1,NT
      SUM=SUM+COEF(M)*TERM(XE(J),EX(M),YN(J),EY(M),0.,0.)
   70 CONTINUE
      DIFF1=GD(J)-SUM
      IF(FLAG)
     1WRITE(IPR,80) IHOLID(J),XE(J),YN(J),GD(J),ERMEAS(J),SUM,DIFF1
   80 FORMAT(2X,A8,6X,F7.2,4X,F7.2,4X,F7.2,2X,F7.2,2X,F7.2,3X,F7.2)
      WRITE(ISCRCH) IHOLID(J),XE(J),YN(J),GD(J),ERMEAS(J),SUM,DIFF1
      KOUNT=KOUNT+1
      IF(MOD(KOUNT,10).EQ.0) CALL ODOMTR(IRRR,KOUNT)
   90 FORMAT('+',I4)
  100 CONTINUE
      WRITE(*,'(''+'',I5)') KOUNT
      IF(KOUNT.EQ.0) GO TO 140
      WRITE(*,110)
  110 FORMAT(//,' NOW WRITING STATPAC OUTPUT FILE',/)
      CALL OUTGEO(ISCRCH,MDY,IDSV,IGD,KOUNT,6,DUMMY,UNIT10,UNIT14,
     1UNIT21)
      DO 120 I=1,5
      CALL SOUND(1000,25)
  120 CONTINUE
      CALL TICKER(ITIME2)
      RTIME=FLOAT(ITIME2-ITIME1)/6000.
      WRITE(*,130) RTIME
  130 FORMAT(' ','RUN TIME WAS ',F6.2,' MINUTES')
      STOP 'NORMAL END OF PROGRAM'
  140 STOP 'NO RESIDUALS COULD BE COMPUTED'
      END
      SUBROUTINE KRIG4C(N,IER)
C***  THIS SUBROUTINE IS A MODIFICATION OF A SUBROUTINE WRITTEN
C***  BY J.A. SKRIVAN AND M.A. KARLINGER OF THE USGS
C***  KRIG4C OBTAINS THE DRIFT COEFFICIENTS FOR PROGRAM DRIFTER
C***  SUBROUTINES REQUIRED: PSET4,DOLITL
$INCLUDE:'PTKRIG.INC'
      NTOT=N+NT
C***  SET UP P MATRIX
      CALL PSET4(N,0.,0.)
      IFSKIP=0
      DO 30 LOOP=1,NT
C***  SET UP RIGHT HAND SIDE
      DO 10 I=1,NTOT
      R(I)=0.
   10 CONTINUE
      R(N+LOOP)=1.
      CALL DOLITL(NTOT,IFSKIP,IER)
      IF(IER.EQ.1) GO TO 40
      IFSKIP=1
      COEF(LOOP)=0.
      DO 20 I=1,N
      IND=IHOL(I)
      COEF(LOOP)=COEF(LOOP)+W(I)*GD(IND)
   20 CONTINUE
   30 CONTINUE
   40 RETURN
      END
