$NOFLOATCALLS
$STORAGE:2
      PROGRAM SS2DGRID
C -----------------------------------------------------------------------
C***  PROGRAM ESTIMATES POINT-VALUES ALONG A GRID USING KRIGING
C***
C***  REFERENCES:
C***  Skrivan, J.A., and Karlinger, M.R., 1980, Semi-variogram estima-
C***      tion and universal kriging program, computer contribution,
C***      U.S. Geological Survey, Water Resources Division, Tacoma,
C***      Washington, 98 p.
C***
C***  Knudsen, H.P., and Kim, Y.C., 1977, A short course on geostatist-
C***      ical ore reserve estimation: College of mines, University of
C***      Arizona, Tucson, Arizona, 202 pages and appendices.
C***
C***  Journel, A.G., and Huijbregts, Ch. J., 1978, Mining geo-
C***      statistics: Academic Press, New York, 600 p.
C***
C***  THIS IS A TWO-DIMENSIONAL POINT-KRIGING PROGRAM USED TO KRIG A
C***  GRID OF POINTS.  IT WAS SYNTHESIZED BY W.D. GRUNDY FROM PROGRAM
C***  UKRIG OF KNUDSEN AND KIM AND THE KRIGING PACKAGE OF SKRIVAN AND
C***  KARLINGER.  ALGORITHMS PUBLISHED BY JOURNEL AND HUIJBREGTS WERE
C***  ALSO USED.
C***  THIS PROGRAM HAS A CHOICE OF UNIVERSAL OR ORDINARY KRIGING AND
C***  ALLOWS THE USER TO SPECIFY THE NEIGHBORHOOD SIZE.  FIVE
C***  VARIOGRAM MODELS, THE SPHERICAL, EXPONENTIAL, LINEAR,
C***  GAUSSIAN, AND CUBIC ARE INCLUDED.  AS MANY AS FIVE NESTED
C***  VARIOGRAMS MAY BE USED.  ANISOTROPIES, IF PRESENT,
C***  MUST BE THE GEOMETRIC TYPE.
C***
C***
C***                       INPUT AND OUTPUT FILES
C***
C***  PROGRAM REQUIRES AN INPUT STATPAC DATA FILE.  THE INPUT FILE
C***  MUST CONTAIN CARTESIAN X- AND Y-COORDINATES (EASTINGS AND NORTH-
C***  INGS) AND AN OBSERVED VALUE OF A REGIONALIZED VARIABLE.  THE
C***  STATPAC INPUT FILE MUST BE PREPROCESSED BY PROGRAM SS2DPREP.
C***
C***  PROGRAM CREATES A STATPAC OUTPUT FILE CONTAINING THE FOLLOWING
C***  VARIABLES FOR EACH GRID POINT: 1)X-COORDINATE, 2)Y-COORDINATE,
C***  3)KRIGED VALUE, AND 4)KRIGING VARIANCE.
C***
C***  ALSO CREATES AN OPTIONAL CHARACTER OUTPUT FILE CONTAINING THE
C***  SAME INFORMATION AS THE STATPAC OUTPUT FILE, BUT ALSO CONTAINING
C***  A HEADER SHOWING THE PARAMETERS USED IN THE GRIDDING.
C***
C***
C*** LINKS REQUIRED: ASCIIZ, BYTMOV, CHEKER, CHRKNT, CKSAFE, CLEAR,
C***                 CLREOL, CONCAT, CRSOFF, CRSON, COVAR4, DATE,
C***                 DELAY, DOLITL, DRFTRM, DSORT, ERRCHK, FNDCUR,
C***                 FSTDSP, GETFLE, HEADER, INT21H, INTR, KRIG4P,
C***                 MAXCOR, MDYHMS, MNVAR, MOVCUR, NEIGH, ODOMTR,
C***                 OUTGEO, PACK, PCHCEN, PRECHK, PSET4, SAVRST,
C***                 SCREAD, SOUND, SRCHPB, TERM, TICKER, TIME,
C***                 UNPK, VALDOP, VARIN4
C***  NOTE: SUBROUTINE FSTDSP WRITES DIRECTLY TO VIDEO RAM
C***  NOTE: ABOVE OBJECT MODULES ARE CONTAINED IN STATPAC.LIB AND
C***        NEED NOT BE RECOMPILED OR REASSEMBLED.
C***        COMPILE SS2DGRID AND ITS SUBPROGRAMS WITH FILES PTKRIG.INC
C***        AND PCREGS.INC PRESENT.
C***
C***  FILE NAME REQUIREMENTS
C***       THE STATPAC INPUT FILE MUST HAVE THE EXTENSION .UPR
C***       THE CHARACTER OUTPUT FILE IS AUTOMATICALLY GIVEN THE
C***       EXTENSION .FGR
C***       THE STATPAC OUTPUT FILE IS AUTOMATICALLY GIVEN THE
C***       EXTENSION .UGR
C***
C***
C***  MAJOR VARIABLES AND DIMENSION REQUIREMENTS
C***
C***    NEST         NUMBER OF VARIOGRAMS IN MODEL (MAX=5)
C***    C0           NUGGET EFFECT
C***    C(NEST)      C-VALUES OF VARIOGRAMS
C***    AVALU(NEST)  A-VALUES OF VARIOGRAMS
C***                 IF VARIOGRAM IS OF FORM GAM(H)=C0+SLOPE*H**A
C***                 AVALU REPRESENTS THE POWER A.
C***                 FOR CUBIC AND SPHERICAL VARIOGRAMS, AVALU
C***                 REPRESENTS THE VARIOGRAM RANGE.
C***                 FOR EXPLANATION OF THIS PARAMETER FOR GAUSSIAN
C***                 AND EXPONENTIAL VARIOGRAMS, SEE JOURNEL AND
C***                 HUIJBREGTS, 1978, PAGE 164.
C***    ANGLE(NEST)  ANGLES OF ROTATION OF ANISOTROPIC VARIOGRAMS ABOUT
C***                 EAST-WEST AXIS.  EAST IS TAKEN AS ZERO DEGREES.
C***                 ANGLES ARE ENTERED IN DECIMAL DEGREES.
C***                 NEGATIVE ANGLE INDICATES CLOCKWISE ROTATION.
C***                 POSITIVE ANGLE INDICATES COUNTER-CLOCKWISE ROTATION
C***                 IF GEOMETRIC ANISOTROPY IS PRESENT, ANGLE IS THE
C***                 BEARING OF THE MAJOR AXIS OF THE ELLIPSE.
C***    AFH(NEST)    ANISOTROPY RATIOS OF VARIOGRAMS
C***                 AFH.NE.1  INDICATES GEOMETRIC ANISOTROPY
C***                 AFH.EQ.1   INDICATES ISOTROPIC VARIOGRAM
C***    SILL         C0 PLUS SUM OF C'S
C***    ITYPE(NEST) VARIOGRAM TYPE
C***                =1 INDICATES SPHERICAL VARIOGRAM
C***                =2 INDICATES EXPONENTIAL VARIOGRAM
C***                =3 INDICATES LINEAR VARIOGRAM
C***                =4 INDICATES GAUSSIAN VARIOGRAM
C***                =5 INDICATES CUBIC VARIOGRAM
C***    AISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    BISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    CISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    SLOPE(NEST) SLOPES OF LINEAR VARIOGRAMS
C***                ZERO SLOPE INDICATES VARIO-
C***                GRAM IS NOT A LINEAR TYPE.
C***
C***  /DRIFT/
C***    NT           NUMBER OF DRIFT TERMS (MAX=10)
C***    EX(NT)       X-EXPONENTS OF DRIFT TERMS
C***    EY(NT)       Y-EXPONENTS OF DRIFT TERMS
C***    COEF(NT)     COEFFICIENTS OF DRIFT TERMS (THIS ARRAY IS NOT USED
C***
C***    MAXH         MAXIMUM NUMBER OF DRILL HOLES WHICH ARE ALLOWED TO BE
C***                 READ IN FROM THE STATPAC FILE.  IN THIS PROGRAM MAXH IS
C***                 SET TO 1500.
C***    IHOLID(MAXH) ALPHAMERIC DRILL HOLE IDENTIFIERS
C***    XE(MAXH)     ARRAY OF X-COORDINATES (EASTINGS) OF DRILL HOLES
C***    YN(MAXH)     ARRAY OF Y-COORDINATES (NORTHINGS) OF DRILL HOLES
C***    GD(MAXH)     ARRAY OF SAMPLE MEASUREMENTS OF DRILL HOLES
C***    ERMEAS(MAXH) ARRAY OF MEASUREMENT VARIANCES OF GD
C***
C***  /COV4/
C***    MAXK         MAXIMUM NUMBER OF HOLES IN NEIGHBORHOOD WHICH CAN
C***                 BE USED IN KRIGING A POINT.  IN THIS PROGRAM MAXK
C***                 IS SET TO 150.
C***    P(KDIM)      COVARIANCE AND DRIFT TERM MATRIX
C***                 KDIM=(MAXK+NT)*(MAXK+NT+1)/2
C***    ID(MAXK+NT)  AN ARRAY OF POINTERS FOR MANIPULATING MATRIX P
C***    W(MAXK+NT)   ARRAY OF KRIGING WEIGHTS AND LAGRANGIAN MULTIPLIERS
C***    IHOL(MAXK+1) ARRAY OF POINTERS TO HOLES TO BE USED IN KRIGING
C***                 A PARTICULAR LOCATION
C***    R(MAXK+NT)   COLUMN VECTOR OF KRIGING SYSTEM
C***
C***    IPT          STATPAC INPUT FILE
C***    IOUT         STATPAC OUTPUT FILE
C***    IPR          CHARACTER OUTPUT FILE
C***    NHOL         ACTUAL NUMBER OF DRILL HOLES OBTAINED FROM STATPAC
C***    IVIB(2,99)   ARRAY OF IDENTIFIERS OF STATPAC FILE VARIABLES
C***
C***    DIST(MAXK+1) ARRAY OF DISTANCES OF DRILL HOLES FROM POINT BEING
C***                 KRIGED.
C***    B(2)         TEMPORARY STORAGE FOR COORDINATES OF POINT BEING
C***                 KRIGED
C***    NK           USER-SPECIFIED ACTUAL MAXIMUM NUMBER OF HOLES TO BE
C***                 USED IN KRIGING A POINT
C***    RMAX         SEARCH RADIUS (NEIGHBORHOOD) ABOUT POINT BEING KRIGED
C***
C***    NER          NUMBER OF NORTH-SOUTH GRID POINTS
C***    WIDY         DISTANCE BETWEEN POINTS IN NORTH-SOUTH DIRECTION
C***    NEC          NUMBER OF EAST-WEST GRID POINTS
C***    WIDX         DISTANCE BETWEEN POINTS IN EAST-WEST DIRECTION
C***    YMN          NORTHING OF SOUTHWEST CORNER OF GRID OF POINTS
C***    XMN          EASTING OF SOUTHWEST CORNER OF GRID OF POINTS
C***
C***
C***  COMMON STATEMENTS ARE INCLUDED IN DISKFILE 'PTKRIG.INC'
C***  PTKRIG.INC MUST BE PRESENT WHEN THIS PROGRAM AND ITS SUBROUTINES
C***  ARE COMPILED.
C -----------------------------------------------------------------------
$INCLUDE:'PTKRIG.INC'
      INTEGER*4 ITIME1,ITIME2
      CHARACTER*32 UNIT10,UNIT14,UNIT21
      CHARACTER*8 MDY
      CHARACTER*10 HMS
      CHARACTER*4 IDSV,EXT1,EXT2,WKDAY
      DIMENSION B(2),IDSV(2,4),DUMMY(4)
      LOGICAL FLAG
      DATA IDSV/'X-CO','ORD.','Y-CO','ORD.','KRIG',' VAL','KRIG',' VAR'/
      DATA EXT1/'.FGR'/, EXT2/'.UGR'/
C -----------------------------------------------------------------------
C***  INITIALIZE TIMING OF PROGRAM RUN AND OTHER VARIABLES
C -----------------------------------------------------------------------
      CALL TICKER(ITIME1)
      CALL TIME(HMS)
      IPT=10
      IPR=21
      IOUT=14
      ISCRCH=13
      MAXH=1500
      MAXK=150
      NEGERR=0
      ISOLVE=0
      CALL DATE(WKDAY,MDY)
C -----------------------------------------------------------------------
C***  GET NAMES OF INPUT/OUTPUT FILES FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL VALDOP(ISCRCH,UNIT10,UNIT14,UNIT21,EXT1,EXT2,FLAG)
C -----------------------------------------------------------------------
C***  CHECK IF UNIVERSAL KRIGING IS WANTED AND GET POSSIBLE DRIFT TERMS
C***  FROM THE KEY BOARD
C -----------------------------------------------------------------------
      CALL DRFTRM(0)
C -----------------------------------------------------------------------
C***  GET VARIOGRAM PARAMETERS
C -----------------------------------------------------------------------
      CALL VARIN4
C -----------------------------------------------------------------------
C***  GET NEIGHBORHOOD LIMITS (SEARCH RADIUS AND MAXIMUM NUMBER OF
C***  SAMPLE POINTS TO BE USED) FROM THE KEYBOARD
C -----------------------------------------------------------------------
      CALL NEIGH(RMAX,NK,NT,MAXK)
C -----------------------------------------------------------------------
C***  GET DESCRIPTORS OF GRID TO BE KRIGED FROM THE KEYBOARD
C -----------------------------------------------------------------------
      CALL INGRD(NER,WIDY,NEC,WIDX,YMN,XMN)
C -----------------------------------------------------------------------
C***  GET DESCRIPTION OF STATPAC INPUT FILE FROM THE KEYBOARD AND
C***  READ THE FILE FROM DISK
C -----------------------------------------------------------------------
      CALL GETFLE(IGD)
C -----------------------------------------------------------------------
C***  CHECK FOR PREPROCESSED STATPAC INPUT DATA FILE AND STOP EXECUTION
C***  IF INPUT FILE WAS NOT SORTED IN ASCENDING ORDER OF Y-COORDINATES.
C -----------------------------------------------------------------------
      CALL PRECHK(YN,XE,NHOL)
C -----------------------------------------------------------------------
C***  WRITE OUT HEADER FOR VARIOGRAM(S) AND ANY DRIFT TERMS.
C***
C***  CLEAR THE SCREEN AFTER A TWO SECOND DELAY
C -----------------------------------------------------------------------
      CALL DELAY(200)
      CALL CLEAR
      CALL HEADER(0,RMAX,NK,WKDAY,MDY,HMS)
      IF(FLAG) CALL HEADER(IPR,RMAX,NK,WKDAY,MDY,HMS)
C -----------------------------------------------------------------------
C***  WRITE OUT USER'S SPECIFICATIONS OF THE GRID AND HEADER FOR THE
C***  KRIGING ESTIMATES.
C -----------------------------------------------------------------------
      WRITE(0,10) NER,WIDY,NEC,WIDX,YMN,XMN,RMAX,NK
      IF(FLAG) WRITE(IPR,10) NER,WIDY,NEC,WIDX,YMN,XMN,RMAX,NK
   10 FORMAT(//,1X,'NUMBER OF POINTS IN NORTH-SOUTH DIRECTION = ',I5,/,
     11X,'DISTANCE BETWEEN POINTS IN NORTH-SOUTH DIRECTION = ',F7.2,/,
     21X,'NUMBER OF POINTS IN  EAST-WEST  DIRECTION = ',I5,/,
     31X,'DISTANCE BETWEEN POINTS IN  EAST-WEST  DIRECTION = ',F7.2,/,
     41X,'NORTHING OF SOUTHWEST CORNER OF GRID OF POINTS = ',F7.2,/,
     51X,'EASTING  OF SOUTHWEST CORNER OF GRID OF POINTS = ',F7.2,/,
     61X,'SEARCH RADIUS = ',F7.2,/,1X,'MAXIMUM NUMBER OF HOLES USED = '
     7,I5)
      IF(FLAG) WRITE(IPR,20)
   20 FORMAT(//,16X,'***OUTPUT OF KRIGED RESULTS***',//,
     13X,'COLUMN  ROW     X-COORD.  Y-COORD.    VALUE      KRG VAR.',/)
C -----------------------------------------------------------------------
C***  START THE KRIGING PROCESS
C -----------------------------------------------------------------------
      WRITE(*,30)
   30 FORMAT(/,'          NOW KRIGING GRID POINTS')
      CALL FNDCUR(IROW,ICOL,0)
      R2=RMAX*RMAX
      KOUNT=0
      IMISS=0
      YM=YMN-WIDY
      XM=XMN-WIDX
         DO 90 J=1,NEC
         B(1)=XM+FLOAT(J)*WIDX
         KSTART=0
            DO 90 K=1,NER
            B(2)=YM+FLOAT(K)*WIDY
C -----------------------------------------------------------------------
C***  FIND NEAREST NS HOLES FROM POINT J,K
C -----------------------------------------------------------------------
            ISTART=KSTART+1
            CALL SRCHPB(B(1),B(2),NS,NK,R2,RMAX,ISTART,KSTART)
C -----------------------------------------------------------------------
C***  BE SURE THERE ARE AT LEAST NT SAMPLE POINTS
C -----------------------------------------------------------------------
               IF(NS.LT.NT) THEN
               IMISS=IMISS+1
               GO TO 90
               ENDIF
C -----------------------------------------------------------------------
C***  KRIG THE POINT IF THERE ARE ENOUGH DRILL HOLES
C***  WITHIN THE SEARCH RADIUS.
C***  GET KRIGING WEIGHTS WHICH WILL BE STORED IN ARRAY W
C -----------------------------------------------------------------------
            CALL KRIG4P(B,NS,IFERR)
            IF(IFERR.EQ.0) GO TO 70
            ISOLVE=ISOLVE+1
            IF(FLAG) WRITE(IPR,40) J,K
   40       FORMAT(' ERROR IN MATRIX SOLUTION POINT ',2I5)
               DO 60 M=1,NS
               LL=IHOL(M)
               IF(FLAG) WRITE(IPR,50) IHOLID(LL),YN(LL),XE(LL)
   50          FORMAT(1X,A8,1X,F10.2,1X,F10.2)
   60          CONTINUE
            GO TO 90
C -----------------------------------------------------------------------
C***  COMPUTE THE KRIGED VALUE AND THE KRIGING VARIANCE
C***  KRIGED VALUE WILL BE STORED IN GR
C -----------------------------------------------------------------------
   70       CALL MNVAR(NS,SILL,GR,VARKG)
            IF(VARKG.LT.0) NEGERR=NEGERR+1
            IF(FLAG) WRITE(IPR,80) J,K,B(1),B(2),GR,VARKG
   80       FORMAT(4X,2I5,3X,2F10.3,F8.2,F15.4)
            WRITE(ISCRCH)B(1),B(2),GR,VARKG
            KOUNT=KOUNT+1
            IF(MOD(KOUNT,10).EQ.0) CALL ODOMTR(IROW,KOUNT)
   90       CONTINUE
      WRITE(*,100) KOUNT
  100 FORMAT('+',I5)
C -----------------------------------------------------------------------
C***  KRIGING ESTIMATES HAVE BEEN COMPLETED.  IF POINTS HAVE BEEN
C***  ESTIMATED WRITE STATPAC OUTPUT FILE AND FINISH.
C -----------------------------------------------------------------------
      IF(KOUNT.EQ.0) STOP ' NO POINTS COULD BE KRIGED'
C -----------------------------------------------------------------------
C***  WRITE STATPAC OUTPUT FILE
C -----------------------------------------------------------------------
      CALL OUTGEO(ISCRCH,MDY,IDSV,IGD,KOUNT,4,DUMMY,UNIT10,UNIT14,
     1UNIT21)
C -----------------------------------------------------------------------
C***  INFORM USER OF ANY PROBLEMS ENCOUNTERED IN KRIGING THE GRID.
C -----------------------------------------------------------------------
      IF(ISOLVE.NE.0) WRITE(*,110) ISOLVE
  110 FORMAT(' ',I3,' POINTS COULD NOT BE GRIDDED BECAUSE OF ERRORS',
     1' IN THE MATRIX SOLUTION',/,' EXAMINE THE CHARACTER OUTPUT FILE',
     2' AND LOOK FOR DUPLICATED SAMPLE LOCATIONS.')
      IF(NEGERR.NE.0) WRITE(*,120) NEGERR
  120 FORMAT(' ',I3,' GRID POINTS HAVE NEGATIVE KRIGING VARIANCES.  ',
     1'THIS CAN',/,' RESULT FROM HAVING MORE THAN 6 SIGNIFICANT DIG',
     2'ITS IN THE NORTH OR',/,' EAST COORDINATES WHEN USING UNIVERSAL',
     3' KRIGING.')
      IF(IMISS.NE.0) WRITE(*,130) IMISS
  130 FORMAT(' ',I4,' GRID POINTS COULD NOT BE KRIGED BECAUSE OF TOO ',
     1'FEW POINTS IN THE SEARCH',/,' ','RADIUS.  YOU MAY WISH TO RE',
     2'RUN THIS PROGRAM USING A LARGER SEARCH RADIUS.')
C -----------------------------------------------------------------------
C***  INFORM USER OF END OF PROGRAM WITH FIVE BEEPS OF THE SPEAKER.
C -----------------------------------------------------------------------
         DO 140 I=1,5
         CALL SOUND(1000,25)
  140    CONTINUE
C -----------------------------------------------------------------------
C***  GET CURRENT TIME FROM SYSTEM CLOCK
C -----------------------------------------------------------------------
      CALL TICKER(ITIME2)
      RTIME=FLOAT(ITIME2-ITIME1)/6000.
      WRITE(*,150) RTIME
  150 FORMAT(' ','RUN TIME WAS ',F6.2,' MINUTES')
      STOP 'NORMAL END OF PROGRAM'
      END
      SUBROUTINE INGRD(NER,WIDY,NEC,WIDX,YMN,XMN)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE INPUTS DESCRIPTORS OF THE GRID OF POINTS
C***  FOR POINT-KRIGING PROGRAM SS2DGRID.
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY, JANUARY, 1980
C -----------------------------------------------------------------------
      WRITE(*,10)
   10 FORMAT(//,5X,'ENTER DESCRIPTORS OF GRID OF POINTS TO BE KRIGED',/)
   20 WRITE(*,30)
   30 FORMAT(5X,'ENTER NUMBER OF POINTS IN NORTH-SOUTH DIRECTION : ',\)
      CALL ERRCHK(1,NER,DUM)
      IF(NER.GE.1) GO TO 60
      CALL SOUND(256,100)
      WRITE(*,40)
   40 FORMAT(' ','YOU MUST SPECIFY AT LEAST ONE POINT')
      GO TO 20
   50 FORMAT(BN,I3)
   60 WRITE(*,70)
   70 FORMAT(5X,'ENTER DISTANCE BETWEEN POINTS IN NORTH-SOUTH DIRECTION
     1: ',\)
      CALL ERRCHK(2,IVAL,WIDY)
   80 FORMAT(BN,F10.0)
      IF(WIDY.GT.0.0) GO TO 90
      CALL SOUND(256,100)
      WRITE(*,130)
      GO TO 60
   90 WRITE(*,100)
  100 FORMAT(5X,'ENTER NUMBER OF POINTS IN EAST-WEST DIRECTION : ',\)
      CALL ERRCHK(1,NEC,DUM)
      IF(NEC.GE.1) GO TO 110
      CALL SOUND(256,100)
      WRITE(*,40)
      GO TO 90
  110 WRITE(*,120)
  120 FORMAT(5X,'ENTER DISTANCE BETWEEN POINTS IN EAST-WEST DIRECTION :
     1',\)
      CALL ERRCHK(2,IVAL,WIDX)
      IF(WIDX.GT.0.) GO TO 140
      CALL SOUND(256,100)
      WRITE(*,130)
  130 FORMAT(' ','DISTANCE MUST BE GREATER THAN ZERO; REENTER VALUE')
      GO TO 110
  140 WRITE(*,150)
  150 FORMAT(5X,'ENTER NORTHING OF SOUTHWEST CORNER OF GRID OF POINTS :
     1',\)
      CALL ERRCHK(2,IVAL,YMN)
  160 WRITE(*,170)
  170 FORMAT(5X,'ENTER EASTING OF SOUTHWEST CORNER OF GRID OF POINTS :
     1',\)
      CALL ERRCHK(2,IVAL,XMN)
      RETURN
      END

