$NOFLOATCALLS
$STORAGE:2
      PROGRAM SS2DPREP
C -----------------------------------------------------------------------
C***  WRITTEN BY W.D. GRUNDY  APRIL  1984
C***
C***  PRE-PROCESSING PROGRAM WHICH MUST BE RUN PRIOR TO RUNNING
C***  PROGRAMS SS2DGAMH, SS2DGRID, AND SS2DDRES.  THE STATPAC FILE
C***  CREATED BY THIS PROGRAM IS USED AS AN INPUT FILE TO THESE
C***  PROGRAMS.
C***
C***  THIS PROGRAM SORTS THE SOURCE STATPAC DATA FILE IN ASCENDING
C***  ORDER OF THE HOLE COORDINATES AND CHECKS FOR DUPLICATED DATA
C***  LOCATIONS.  AN OUTPUT FILE IS NOT PRODUCED IF ANY DUPLICATES
C***  ARE FOUND.  A REPORT OF THE PRIMARY ROWIDS OF DUPLICATED
C***  POINTS AND ROW NUMBERS IS PRINTED ON THE SCREEN.  THE DUPLICATES
C***  MUST BE RECONCILED AND REMOVED FROM THE INPUT FILE BEFORE THIS
C***  PROGRAM WILL PRODUCE THE NEEDED OUTPUT FILE FOR PROGRAM SS2DGAMH.
C***
C***  IF THE INPUT FILE HAS MISSING DATA WHICH ARE NOT QUALIFIED,
C***  AND IF THESE MISSING DATA HAVE A UNIQUE NUMERIC VALUE (SUCH
C***  AS -999), THIS PROGRAM REQUESTS THE UNIQUE VALUE, AND ATTACHES
C***  THE STATPAC QUALIFIER 'B' TO THAT MISSING DATA ITEM.  THIS
C***  QUALIFIER IS NECESSARY TO PREVENT PROGRAM SS2DGAMH AND THE
C***  KRIGING PROGRAMS FROM USING INVALID DATA POINTS.
C***
C***  LINKS REQUIRED: ASCIIZ, BYTMOV, B2SORT, CHEKER, CHGATR, CHRKNT,
C***                  CKSAFE, CONCAT, DATE, DOSREN, FNDCUR, FSTDSP,
C***                  FSORT, MDYHMS, MOVCUR, ODOMTR, PACK, SOUND,
C***                  TICKER, UNPK
C***  NOTE: ABOVE OBJECT MODULES ARE CONTAINED IN STATPAC.LIB AND
C***        NEED NOT BE RECOMPILED OR REASSEMBLED.
C***
C***  FILE NAME REQUIREMENTS:
C***       THE STATPAC INPUT FILE MUST HAVE THE EXTENSION .STP
C***       THE STATPAC OUTPUT FILE WILL AUTOMATICALLY BE GIVEN
C***       THE EXTENSION .UPR (Unformatted PRep).
C -----------------------------------------------------------------------
      INTEGER*4 LOC(2)
      INTEGER*4 ITIME1,ITIME2
      CHARACTER*32 UNIT10,UNIT14
      CHARACTER*4 EXT0,EXT1,WKDAY
      CHARACTER*8 IDS,IVID(99),NAME(2),IHOLID(1500),TEMP,MDY
      CHARACTER*1 IA(99),IANS
      LOGICAL FLAG
      LOGICAL IFLAGQ
      DIMENSION XE(1500),YN(1500),IPOINT(1500),X(99),IY(20)
      EQUIVALENCE (TEMP,NAME(1))
      DATA EXT0/'.STP'/,EXT1/'.UPR'/
C -----------------------------------------------------------------------
C***  INITIALIZE TIMING OF PROGRAM RUN AND OTHER NECESSARY VARIABLES
C -----------------------------------------------------------------------
      CALL TICKER(ITIME1)
      CALL DATE(WKDAY,MDY)
C -----------------------------------------------------------------------
C***  KI IS INPUT UNIT; KO IS OUTPUT UNIT; KS IS SCRATCH UNIT
C -----------------------------------------------------------------------
      KI=10
      KO=14
      KS=13
C -----------------------------------------------------------------------
C***  INITIALIZE COUNT OF DUPLICATES
C -----------------------------------------------------------------------
      IDUP=0
C -----------------------------------------------------------------------
C***  GET NAMES OF INPUT/OUTPUT FILES
C -----------------------------------------------------------------------
   10 WRITE(*,'('' '',''WHAT IS THE ROOT NAME OF THE STATPAC '',
     1''INPUT FILE? '',\)')
      READ(*,20) UNIT10
   20 FORMAT(A32)
      CALL CHEKER(UNIT10,EXT0,FLAG)
      IF(FLAG) GO TO 10
C -----------------------------------------------------------------------
C***  CHECK FOR EXISTENCE OF INPUT FILE
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT10,2,FLAG)
      IF(FLAG) GO TO 10
      OPEN(KI,FILE=UNIT10,STATUS='OLD',FORM='UNFORMATTED')
   30 WRITE(*,'('' '',''GIVE A ROOT NAME FOR THE STATPAC OUTPUT '',
     1''FILE: '',\)')
      READ(*,20) UNIT14
      CALL CHEKER(UNIT14,EXT1,FLAG)
      IF(FLAG) GO TO 30
C -----------------------------------------------------------------------
C***  CHECK FOR EXISTENCE OF OUTPUT FILE.  IF IT ALREADY EXISTS,
C***  DO NOT ALLOW IT TO BE OVERWRITTEN.
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT14,1,FLAG)
      IF(FLAG) GO TO 30
      OPEN(KO,FILE=UNIT14,STATUS='NEW',FORM='UNFORMATTED')
C -----------------------------------------------------------------------
C***  READ STATPAC HEADER RECORD
C -----------------------------------------------------------------------
      READ(KI) IDS,N,M
         IF(N.GT.1500) THEN
         CALL SOUND(256,100)
         WRITE(*,'('' WARNING*** THERE ARE '',I5,'' ROWS IN THE '',
     1   ''STATPAC INPUT FILE.'',/,'' THE LIMIT FOR KRIGING IS 1500 '',
     2   ''UNQUALIFIED DATA POINTS.'')') N
         ENDIF
         IF(M.GT.99) THEN
         CALL SOUND(256,100)
         WRITE(*,'('' THERE ARE '',I3,'' COLS IN THE STATPAC INPUT '',
     1   ''FILE.'',/,'' THE LIMIT FOR KRIGING IS 99 COLUMNS.'')') M
         STOP
         ENDIF
      REWIND (KI)
      READ(KI) IDS,N,M,(IVID(I),I=1,M)
      NPACK=(M+4)/5
C -----------------------------------------------------------------------
C***  COMPUTE RECORD LENGTH FOR DIRECT ACCESS SCRATCH FILE
C -----------------------------------------------------------------------
      LENREC=24+5*M
C -----------------------------------------------------------------------
C***  OPEN DIRECT ACCESS SCRATCH FILE
C -----------------------------------------------------------------------
      OPEN(KS,FILE='SCRATCH',ACCESS='DIRECT',STATUS='NEW',
     1FORM='UNFORMATTED',RECL=LENREC)
C -----------------------------------------------------------------------
C***  GET COLUMN NUMBERS OF NORTH- AND EAST-COORDINATES
C***  AND GET SPECIFICATIONS FOR POSSIBLE MISSING DATA
C -----------------------------------------------------------------------
      WRITE(*,40) IDS
   40 FORMAT(' ',//,5X,'STATPAC INPUT FILE ID IS: ',A8,
     1' ;   COLUMN ID''S ARE:')
      WRITE(*,'('' '')')
      WRITE(*,50) (I,IVID(I),I=1,M)
   50 FORMAT(5(1X,I2,')',A8))
      WRITE(*,60)
   60 FORMAT(/,5X,'ENTER STATPAC FILE DESCRIPTORS',/)
   70 IFLAG=1
      WRITE(*,'(A\)')' WHAT IS COLUMN NUMBER OF NORTH COORDINATE? '
      READ(*,80,ERR=280) INORTH
   80 FORMAT(BN,I2)
         IF(INORTH.LT.1.OR.INORTH.GT.M) THEN
         CALL SOUND(256,100)
         WRITE(*,'('' SPECIFIED COLUMN DOES NOT EXIST.  REDO'')')
         GO TO 70
         ENDIF
   90 IFLAG=2
      WRITE(*,'(A\)')' WHAT IS COLUMN NUMBER OF EAST COORDINATE? '
      READ(*,80,ERR=280) IEAST
         IF(IEAST.LT.1.OR.IEAST.GT.M) THEN
         CALL SOUND(256,100)
         WRITE(*,'('' SPECIFIED COLUMN DOES NOT EXIST.  REDO'')')
         GO TO 90
         ENDIF
         IF(IEAST.EQ.INORTH) THEN
         CALL SOUND(256,100)
         WRITE(*,'(A)') ' SPECIFIED COLUMNS NOT UNIQUE.  REDO'
         GO TO 70
         ENDIF
      WRITE(*,100)
  100 FORMAT(5X,'DO ANY ROWS OF THE STATPAC FILE HAVE MISSING ',
     1'DATA?',/,5X,'ANSWER YES OR NO: ',\)
      READ(*,110) IANS
  110 FORMAT(A1)
         IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') THEN
         WRITE(*,120)
  120    FORMAT(5X,'HAVE ALL THE MISSING DATA BEEN GIVEN STATPAC ',
     1   'QUALIFIERS?',/,5X,'ANSWER YES OR NO: ',\)
         READ(*,110) IANS
         IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') GO TO 150
         IFLAGQ=.TRUE.
         WRITE(*,130)
  130    FORMAT(5X,'ENTER THE NUMERIC VALUE WHICH WAS USED TO FLAG ',
     1   'MISSING VALUES: ',\)
         READ(*,140) XFLAG
  140    FORMAT(BN,F10.0)
         ENDIF
C -----------------------------------------------------------------------
C***  READ STATPAC INPUT FILE AND STORE NORTH- AND EAST-COORDINATES
C -----------------------------------------------------------------------
  150 WRITE(*,'(A)') '        ...READING STATPAC INPUT FILE'
      CALL FNDCUR(IRRR,ICCC,0)
         DO 170 I=1,N
         READ(KI) IROW,NAME,LOC,(X(J),J=1,M),(IY(J),J=1,NPACK)
         CALL UNPK(IA,M,IY)
            IF(IFLAGQ) THEN
               DO 160 J=1,M
               IF(X(J).EQ.XFLAG.AND.IA(J).EQ.' ') IA(J)='B'
  160          CONTINUE
            ENDIF
         XE(I)=X(IEAST)
         YN(I)=X(INORTH)
         IHOLID(I)=TEMP
         IPOINT(I)=I
C -----------------------------------------------------------------------
C***  WRITE DIRECT ACCESS SCRATCH FILE
C -----------------------------------------------------------------------
         WRITE(KS) NAME,LOC,(X(J),IA(J),J=1,M)
         IF(MOD(I,10).EQ.0) CALL ODOMTR(IRRR,I)
  170    CONTINUE
C -----------------------------------------------------------------------
C***  SORT ARRAY OF POINTERS WITH NORTH AS MAJOR AND EAST AS MINOR KEY
C -----------------------------------------------------------------------
      WRITE(*,'(''+'',I5)') N
      CALL FSORT(YN,IPOINT,N)
      CALL B2SORT(YN,XE,IPOINT,N)
C -----------------------------------------------------------------------
C***  NOW LOOP TO SEARCH FOR MATCHED COORDINATES
C -----------------------------------------------------------------------
      WRITE(*,180)
  180 FORMAT(' NOW SEARCHING FOR DUPLICATES',//)
C -----------------------------------------------------------------------
C***  WRITE HEADER FOR POSSIBLE DUPLICATES
C -----------------------------------------------------------------------
      WRITE(*,190) UNIT10
  190 FORMAT(6X,'RECORD OF DUPLICATED COORDINATES IN FILE ',A32,//,
     11X,'PRIMARY-ID  ROW-NUMBER  MATCHES  PRIMARY-ID  ROW-NUMBER')
         DO 210 J=2,N
         K=J-1
         IF(YN(IPOINT(J)).EQ.YN(IPOINT(K)).AND.XE(IPOINT(J)).EQ.
     1      XE(IPOINT(K))) THEN
            WRITE(*,200) IHOLID(IPOINT(K)),IPOINT(K),IHOLID(IPOINT(J)),
     1      IPOINT(J)
  200       FORMAT(' ',A8,4X,I4,17X,A8,4X,I4)
            IDUP=IDUP+1
            ENDIF
  210    CONTINUE
C -----------------------------------------------------------------------
C***  IF NO DUPLICATES WERE FOUND WRITE A STATPAC OUTPUT FILE
C -----------------------------------------------------------------------
         IF(IDUP.EQ.0) THEN
         WRITE(*,220)
  220    FORMAT(' NO DUPLICATED LOCATIONS WERE FOUND',/)
C -----------------------------------------------------------------------
C***  IT IS OKAY TO WRITE THE STATPAC OUTPUT FILE
C -----------------------------------------------------------------------
         WRITE(*,'(A)') '        ...WRITING STATPAC OUTPUT FILE'
         CALL FNDCUR(IRRR,ICCC,0)
         CALL CHGATR(34,3,IRRR-2,0,0)
         CALL MOVCUR(IRRR,ICCC,0)
         WRITE(KO) MDY,N,M,(IVID(I),I=1,M)
            DO 230 I=1,N
            READ(KS,REC=IPOINT(I)) NAME,LOC,(X(J),IA(J),J=1,M)
            CALL PACK(IA,M,IY,NPACK)
            WRITE(KO) I,NAME,LOC,(X(J),J=1,M),(IY(J),J=1,NPACK)
            IF(MOD(I,10).EQ.0) CALL ODOMTR(IRRR,I)
  230       CONTINUE
         WRITE(*,'(''+'',I5)') N
C -----------------------------------------------------------------------
C***  WRITE MESSAGES TO SCREEN AND FINISH
C -----------------------------------------------------------------------
         WRITE(*,240) UNIT10,UNIT14
  240    FORMAT(' ','THE PREPROCESSED STATPAC FILE HAS BEEN WRITTEN',/,
     1   ' THE STATPAC INPUT FILE WAS ',A32,/,' THE STATPAC OUTPUT FILE 
     1   ',
     2   'IS ',A32)
         ELSE
         WRITE(*,250) IDUP
  250    FORMAT(/,' ',I4,' DUPLICATE(S) WERE FOUND.',/,' RESOLVE THESE',
     1      ' DUPLICATES AND THEN REMOVE UNWANTED ROWS FROM THE ',
     2      /,' STATPAC INPUT FILE USING EITHER PROGRAM ADDORDEL OR ',
     3      'PROGRAM ROWIDRET.')
            CLOSE(KO,STATUS='DELETE')
            ENDIF
         CLOSE(KS,STATUS='DELETE')
            DO 260 I=1,5
            CALL SOUND(1000,25)
  260       CONTINUE
         CALL TICKER(ITIME2)
         TIME=FLOAT(ITIME2-ITIME1)/6000.
         WRITE(*,270) TIME
  270    FORMAT(' ','RUN TIME WAS ',F6.2,' MINUTES')
         STOP 'NORMAL END OF PROGRAM'
  280    CALL SOUND(256,100)
         WRITE(*,'('' INTEGER VALUE WAS EXPECTED.  REDO'')')
         GO TO (70,90), IFLAG
         END
