$NOFLOATCALLS
$STORAGE:2
      PROGRAM SS2DXVAL
C -----------------------------------------------------------------------
C***  PROGRAM PERFORMS CROSS-VALIDATION OF VARIOGRAM/DRIFT MODELS
C***  REFERENCES:
C***
C***  Clark, Isobel, 1986, The art of cross-validation in geostatist-
C***      ical applications, Proceedings 19th APCOM (Application of
C***      computers and operations research in the mineral industry)
C***
C***  Delhomme, J.P., 1976, Kriging in the hydrosciences: Advances
C***      in Water Resources, vol. 1, p.251-266.
C***
C***  Galli, Alain, Gerdil-Neuillet, Francois, and Clair Dadou, 1984,
C***      Factorial kriging analysis: a substitute to spectral analy-
C***      sis of magnetic data, in Geostatistics for Natural Resources
C***      Characterization (G. Verly and others editors),Part 1, pp.
C***      543,557.  (NATO ASI Series, Series C: Mathematical and Phy-
C***      Physical Sciences, Vol. 122), D. Reidel Publishing Co.,
C***      Dordrecht, Holland.
C***
C***  Journel, A.G., and Huijbregts, Ch. J., 1978, Mining geo-
C***      statistics: Academic Press, New York, 600 p.
C***
C***  Knudsen, H.P., and Kim, Y.C., 1977, A short course on geostatist-
C***      ical ore reserve estimation: College of mines, University of
C***      Arizona, Tucson, Arizona, 202 pages and appendices.
C***
C***  Skrivan, J.A., and Karlinger, M.R., 1980, Semi-variogram estima-
C***      tion and universal kriging program, computer contribution,
C***      U.S. Geological Survey, Water Resources Division, Tacoma,
C***      Washington, 98 p.
C***
C***  PROGRAM SS2DXVAL IS A TWO-DIMENSIONAL POINT-KRIGING PROGRAM
C***  USED TO CROSS-VALIDATE VARIOGRAM/DRIFT MODELS.  IT USES THE
C***  MOVING NEIGHBORHOOD TECHNIQUE AND ALLOWS THE USER A CHOICE
C***  OF ORDINARY OR UNIVERSAL KRIGING.  FIVE VARIOGRAM MODELS,
C***  THE SPHERICAL, EXPONENTIAL, LINEAR-ROOT-PARABOLIC, GAUSSIAN,
C***  AND CUBIC ARE INCLUDED.  ANISOTROPIES, IF PRESENT, MUST BE
C***  THE GEOMETRIC TYPE.  THE USER SPECIFIES THE NEIGHBORHOOD
C***  SIZE AS A SEARCH RADIUS.  IF UNIVERSAL KRIGING IS SELECTED,
C***  TED, THE X- AND Y-EXPONENTS OF UP TO TEN MONOMIAL TERMS CAN
C***  BE ENTERED FOR THE DRIFT.
C***
C***  SS2DXVAL WAS ADAPTED BY W.D. GRUNDY FROM PROGRAM K603 (SKRIVAN
C***  AND KARLINGER, 1980) AND FROM PROGRAM UKRIG (KNUDSEN AND KIM,
C***  1977).  PROGRAM K603 IS A POINT-KRIGING PROGRAM WHICH FEATURES
C***  BOTH ORDINARY AND UNIVERSAL KRIGING IN A UNIQUE NEIGHBORHOOD.
C***  PROGRAM UKRIG IS A BLOCK-KRIGING PROGRAM WHICH USES A MOVING
C***  NEIGHBORHOOD, BUT WHICH IS RESTRICTED TO ORDINARY KRIGING.
C***  THE ANISOTROPY AND NESTED VARIOGRAM ROUTINES USED IN SS2DXVAL
C***  WERE ADAPTED FROM ALGORITHMS PUBLISHED IN JOURNEL AND HUIJBREGTS
C***  IN COMMENTS IN THE FORTRAN SOURCE CODES OF THESE ROUTINES.
C***
C***
C***  CROSS-VALIDATION CAN BE DONE IN ANY ONE OF THREE WAYS: USING
C***  HOLE-BY-HOLE SUPRESSION OF THE ENTIRE DATA SET; BY SPLITTING
C***  A RANDOMLY PERMUTED DATA SET INTO TWO PARTS, AND POINT-KRIGING
C***  ONE SUBSET FROM THE OTHER; OR BY SELECTING A BEGINNING STATPAC
C***  ROW NUMBER OF THE INPUT DATA SET WHICH IS TO BE VALIDATED.  IN
C***  THE FIRST METHOD, ALL DATA POINTS ARE ESTIMATED ONE-BY-ONE.  IN
C***  THE SECOND METHOD, ODD-NUMBERED DATA POINTS (CONTAINED IN STATPAC
C***  ROWS 1,3,5,7,9,ETC.) ARE ESTIMATED BY EVEN-NUMBERED DATA POINTS
C***  (CONTAINED IN ROWS 2,4,8,10,ETC.).  IN THE THIRD METHOD, THE ROWS
C***  FROM THE SPECIFIED ROW NUMBER TO THE LAST ROW IN THE STATPAC FILE
C***  ARE ESTIMATED USING DATA FROM THE FIRST ROW OF THE FILE TO THE
C***  SPECIFIED ROW NUMBER.  THIS LAST METHOD IS USEFUL IN PREDICTING
C***  THE VALUES OF HYPOTHETICAL DATA POINTS WHICH HAVE NOT BEEN
C***  SAMPLED.  PROGRAM TERFIL CAN BE USED TO CREATE A STATPAC FILE OF
C***  HYPOTHETICAL POINTS AND PROGRAM ADROWS CAN BE USED TO APPEND THIS
C***  FILE TO THE ORIGINAL STATPAC FILE OF OBSERVED VALUES.
C***
C***
C***                       INPUT AND OUTPUT FILES
C***
C***  PROGRAM REQUIRES AN INPUT STATPAC DATA FILE.  THE INPUT FILE
C***  MUST CONTAIN CARTESIAN X- AND Y-COORDINATES (EASTINGS AND NORTH-
C***  INGS) AND AN OBSERVED VALUE OF A REGIONALIZED VARIABLE FOR EACH
C***  DATA POINT.  QUALIFIED DATA POINTS ARE BY-PASSED BY THE PROGRAM
C***  AND ARE NOT INCLUDED IN THE COMPUTATIONS.
C***
C***  PROGRAM CREATES A STATPAC OUTPUT FILE CONTAINING THE FOLLOWING
C***  VARIABLES FOR EACH DATA POINT: 1)X-COORDINATE, 2)Y-COORDINATE,
C***  3)OBSERVED VALUE, 4)PREDICTED VALUE, 5)KRIGING ERROR, 6)KRIGING
C***  STANDARD DEVIATION, 7)STANDARDIZED ERROR (KRIGING ERROR DIVIDED
C***  BY KRIGING STANDARD DEVIATION).
C***
C***  PROGRAM ALSO CREATES A CHARACTER OUTPUT FILE CONTAINING THE
C***  SAME INFORMATION AS THE STATPAC OUTPUT FILE, BUT ALSO CONTAINING
C***  A HEADER SHOWING THE PARAMETERS USED IN THE CROSS-VALIDATION
C***  SESSION AND A TRAILING RECORD OF SUMMARY STATISTICS OF THE
C***  CROSS-VALIDATION RUN.  THE PRIMARY STATPAC ROWID (EG.,DRILL-HOLE
C***  NUMBER) OF EACH DATA POINT IS ALSO LISTED.
C***
C***
C***  CRITERIA TO JUDGE ACCEPTABILITY OF VARIOGRAM/DRIFT MODEL
C***             (IN NO PARTICULAR ORDER OF IMPORTANCE)
C***
C***  1) AVERAGE KRIGING ERROR SHOULD BE CLOSE TO ZERO.
C***  2) ROOT MEAN SQUARE ERROR (STANDARD DEVIATION OF THE KRIGING
C***  ERRORS) SHOULD BE LOWER THAN THE STANDARD DEVIATION OF THE
C***  REGIONALIZED VARIABLE.
C***  3) KRIGED REDUCED ROOT MEAN SQUARE ERROR (STANDARD DEVIATION OF
C***  THE STANDARDIZED ERRORS) SHOULD BE CLOSE TO UNITY (I.E. IN
C***  INTERVAL 1 PLUS OR MINUS 2*SQRT(2/N) (DELHOMME, 1978, P. 258))
C***  4) THE STANDARDIZED ERRORS SHOULD BE INDEPENDENT OF THE KRIGED
C***  (ESTIMATED) VALUES.
C***  5) THE STANDARDIZED ERRORS SHOULD BE INDEPENDENT OF THEIR
C***  LOCATION AS EXPRESSED BY THEIR X- AND Y-COORDINATES.
C***  6) THE KRIGED VALUE SHOULD BE POSITIVELY CORRELATED WITH THE
C***  OBSERVED VALUE OF THE REGIONALIZED VARIABLE.  THE HIGHER THE
C***  CORRELATION, THE BETTER, SO LONG AS THE PRECEDING CRITERIA ARE
C***  MET.
C***  7) IT IS IMPORTANT FOR THE FINAL THEORETICAL VARIOGRAM MODEL TO
C***  CLOSELY APPROXIMATE  THE OVERALL APPEARANCE OF THE FINAL
C***  OBSERVED VARIOGRAM (AFTER DELETION OF OUTLIER VALUES).  CLARK
C***  (1986) DESCRIBES SOME OF THE PITFALLS AND PROBLEMS IN CROSS-
C***  VALIDATION.  PAY SPECIAL HEED TO HER FINAL PARAGRAPH ON PAGE 220.
C***  ALSO SEE JOURNEL AND HUIJBREGTS, 1978, P. 167; P. 246.
C***
C***  USE PROGRAMS STATCOR AND/OR BASTAT AND THE STATPAC OUTPUT FILE
C***  FROM THIS PROGRAM TO CHECK THESE CRITERIA.  ALSO USE PROGRAM
C***  XYPLOT TO EXAMINE THE CORRELATIONS VISUALLY.  PROGRAM MAPPLT
C***  CAN BE USED TO SPOT AREAS ON THE MAP WHERE OBSERVED DATA VALUES
C***  ARE NOT WELL PREDICTED USING THE VARIOGRAM/DRIFT MODEL.
C***
C***  IT IS DESIRABLE, BUT NOT NECESSARY, THAT THE STANDARDIZED ERRORS
C***  BE NORMALLY DISTRIBUTED.  THERE IS NO RIGOROUS TEST FOR NORMALITY
C***  OF KRIGING ERRORS, BUT THE HISTOGRAM OPTION OF PROGRAM BASTAT
C***  AND THE CHI-SQUARE TEST OF PROGRAM NORCHI CAN BE USEFUL IN
C***  FORMING A SUBJECTIVE JUDGMENT CONCERNING THE NORMALITY OF THE
C***  STANDARDIZED ERRORS.
C***
C***
C***  LINKS REQUIRED: ASCIIZ, BYTMOV, CHEKER, CHRKNT, CKSAFE, CLEAR,
C***                  CLREOL, CONCAT, COVAR4, CRSOFF, CRSON, DATE,
C***                  DELAY, DOLITL, DRFTRM, DSORT, ERRCHK, FNDCUR,
C***                  FSTDSP, GETFLE, HEADER, HLSRCH, INT21H, INTR,
C***                  KRIG4P, MAXCOR, MDYHMS, MNVAR, MOVCUR, NEIGH,
C***                  ODOMTR, OUTGEO, PACK, PCHCEN, PSET4, SAVRST,
C***                  SCREAD, SOUND, TERM, TICKER, TIME, UNPK, VALDOP,
C***                  VARIN4
C***  NOTE: ABOVE OBJECT MODULES ARE CONTAINED IN STATPAC.LIB AND
C***        NEED NOT BE RECOMPILED OR REASSEMBLED.
C***        COMPILE SS2DXVAL AND ITS SUBPROGRAMS WITH FILES PTKRIG.INC
C***        AND PCREGS.INC PRESENT.
C***  NOTE: SUBROUTINE FSTDSP WRITES DIRECTLY TO VIDEO RAM
C***
C***  FILE NAME REQUIREMENTS
C***       THE STATPAC INPUT FILE MUST HAVE THE EXTENSION .UPR
C***       THE CHARACTER OUTPUT FILE IS AUTOMATICALLY GIVEN THE
C***       EXTENSION .FXV
C***       THE STATPAC OUTPUT FILE IS AUTOMATICALLY GIVEN THE
C***       EXTENSION .UXV
C***
C***
C***  MAJOR VARIABLES AND DIMENSION REQUIREMENTS
C***
C***    NEST         NUMBER OF VARIOGRAMS IN MODEL (MAX=5)
C***    C0           NUGGET EFFECT
C***    C(NEST)      C-VALUES OF VARIOGRAMS
C***    AVALU(NEST)  A-VALUES OF VARIOGRAMS
C***                 IF VARIOGRAM IS OF FORM GAM(H)=C0+SLOPE*H**A
C***                 AVALU REPRESENTS THE POWER A.
C***                 FOR CUBIC AND SPHERICAL VARIOGRAMS, AVALU
C***                 REPRESENTS THE VARIOGRAM RANGE.
C***                 FOR EXPLANATION OF THIS PARAMETER FOR GAUSSIAN
C***                 AND EXPONENTIAL VARIOGRAMS, SEE JOURNEL AND
C***                 HUIJBREGTS, 1978, PAGE 164.  THE CUBIC VARIOGRAM
C***                 MODEL IS DESCRIBED IN ALLAIN ET. AL, 1984, P 547.
C***    ANGLE(NEST)  ANGLES OF ROTATION OF ANISOTROPIC VARIOGRAMS ABOUT
C***                 EAST-WEST AXIS.  EAST IS TAKEN AS ZERO DEGREES.
C***                 ANGLES ARE ENTERED IN DECIMAL DEGREES.
C***                 NEGATIVE ANGLE INDICATES CLOCKWISE ROTATION.
C***                 POSITIVE ANGLE INDICATES COUNTER-CLOCKWISE
C***                 ROTATION. (SEE COMMENTS IN VARIN4.FOR FOR DETAILS
C***                 ON ENTERING VARIOGRAM PARAMETERS)
C***                 IF GEOMETRIC ANISOTROPY IS PRESENT, ANGLE IS THE
C***                 BEARING OF THE MAJOR AXIS OF THE ELLIPSE.
C***    AFH(NEST)    ANISOTROPY RATIOS OF VARIOGRAMS
C***                 AFH.NE.1  INDICATES GEOMETRIC ANISOTROPY
C***                           RATIO OF MAJOR AXIS OF ELLIPSE TO
C***                           MINOR AXIS OF ELLIPSE.  (SEE JOURNEL
C***                           AND HUIJBREGTS (1978, PP. 179-180)
C***                 AFH.EQ.1  INDICATES ISOTROPIC VARIOGRAM
C***    SILL         C0 PLUS SUM OF C'S
C***    ITYPE(NEST) VARIOGRAM TYPE
C***                 =1 INDICATES SPHERICAL VARIOGRAM
C***                 =2 INDICATES EXPONENTIAL VARIOGRAM
C***                 =3 INDICATES LINEAR VARIOGRAM
C***                 =4 INDICATES GAUSSIAN VARIOGRAM
C***                 =5 INDICATES CUBIC VARIOGRAM
C***    AISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    BISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    CISI(NEST)     ANISOTROPY CORRECTION FACTOR
C***    SLOPE(NEST)  SLOPES OF LINEAR VARIOGRAMS
C***                 ZERO SLOPE INDICATES VARIO-
C***                 GRAM IS NOT A LINEAR TYPE.
C***
C***    NT           NUMBER OF DRIFT TERMS (MAX=10)
C***    EX(NT)       X-EXPONENTS OF DRIFT TERMS
C***    EY(NT)       Y-EXPONENTS OF DRIFT TERMS
C***    MAXH         MAXIMUM NUMBER OF DRILL HOLES WHICH ARE ALLOWED TO BE
C***                 READ IN FROM THE STATPAC FILE.  IN THIS PROGRAM
C***                 MAXH IS SET TO 1500.
C***    IHOLID(MAXH) ALPHAMERIC DRILL HOLE IDENTIFIERS
C***    XE(MAXH)     ARRAY OF X-COORDINATES (EASTINGS) OF DRILL HOLES
C***    YN(MAXH)     ARRAY OF Y-COORDINATES (NORTHINGS) OF DRILL HOLES
C***    GD(MAXH)     ARRAY OF SAMPLE MEASUREMENTS OF DRILL HOLES
C***    ERMEAS(MAXH) ARRAY OF MEASUREMENT VARIANCES OF GD (USE OF THIS
C***                 VARIABLE IS AN OPTION OF THE USER.  IN PRACTICE,
C***                 THE ERROR VARIANCE OF THE MEASUREMENT AT EACH
C***                 SAMPLE LOCATION IS RARELY KNOWN, SO THE OPTION
C***                 WILL ORDINARILY NOT BE USED.)
C***                 FOR MORE INFORMATION ON THE ERROR MEASURE, SEE
C***                 DELHOMME (1978, P. 261)
C***
C***  /COV4/
C***    MAXK         MAXIMUM NUMBER OF HOLES IN NEIGHBORHOOD WHICH CAN
C***                 BE USED IN KRIGING A POINT.  IN THIS PROGRAM MAXK
C***                 IS SET TO 150.
C***    P(KDIM)      COVARIANCE AND DRIFT TERM MATRIX
C***                 KDIM=(MAXK+NT)*(MAXK+NT+1)/2
C***    ID(MAXK+NT)  AN ARRAY OF POINTERS FOR MANIPULATING MATRIX P
C***    W(MAXK+NT)   ARRAY OF KRIGING WEIGHTS AND LAGRANGIAN MULTIPLIERS
C***    IHOL(MAXK+1) ARRAY OF POINTERS TO HOLES TO BE USED IN KRIGING
C***                 A PARTICULAR LOCATION
C***    R(MAXK+NT)   COLUMN VECTOR OF KRIGING SYSTEM
C***
C***    IPT          STATPAC INPUT FILE
C***    IOUT         STATPAC OUTPUT FILE
C***    IPR          CHARACTER OUTPUT FILE
C***    NHOL         ACTUAL NUMBER OF DRILL HOLES OBTAINED FROM STATPAC
C***    IVIB(2,99)   ARRAY OF IDENTIFIERS OF STATPAC FILE VARIABLES
C***
C***    DIST(MAXK+1) ARRAY OF DISTANCES OF DRILL HOLES FROM POINT BEING
C***                 KRIGED.
C***    B(2)         TEMPORARY STORAGE FOR COORDINATES OF POINT BEING
C***                 KRIGED
C***    NK           USER-SPECIFIED ACTUAL MAXIMUM NUMBER OF HOLES TO
C***                 BE USED IN KRIGING A POINT
C***    RMAX         SEARCH RADIUS (NEIGHBORHOOD) ABOUT POINT
C***                 BEING KRIGED
C***
C***
C -----------------------------------------------------------------------
$INCLUDE:'PTKRIG.INC'
C -----------------------------------------------------------------------
C***
C***  COMMON STATEMENTS ARE INCLUDED IN DISKFILE 'PTKRIG.INC'
C***  PTKRIG.INC MUST BE PRESENT WHEN THIS PROGRAM AND ITS SUBROUTINES
C***  ARE COMPILED.
C -----------------------------------------------------------------------
      INTEGER*4 ITIME1,ITIME2
      CHARACTER*10 HMS
      CHARACTER*4 IDSV,EXT1,EXT2,WKDAY
      CHARACTER*8 MDY
      CHARACTER*32 UNIT10,UNIT14,UNIT21
      LOGICAL FLAG
      DIMENSION B(2),IDSV(2,7),DUMMY(7)
      DATA IDSV/'X-CO','ORD.','Y-CO','ORD.','ASAY',' VAL','KRIG',
     1' VAL','KRIG',' ERR','KRGS','TDEV','ERR/','STDV'/
      DATA EXT1/'.FXV'/, EXT2/'.UXV'/
C -----------------------------------------------------------------------
C***  INITIALIZE TIMING OF PROGRAM RUN AND OTHER VARIABLES
C -----------------------------------------------------------------------
      CALL TICKER(ITIME1)
      CALL DATE(WKDAY,MDY)
      CALL TIME(HMS)
      IPT=10
      IPR=21
      IOUT=14
      ISCRCH=13
      ITOTER=0
      IBYPAS=0
      NEGERR=0
      MAXH=1500
      IRRR=1
      MAXK=150
C -----------------------------------------------------------------------
C***  GET NAMES OF INPUT/OUTPUT FILES FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL VALDOP(ISCRCH,UNIT10,UNIT14,UNIT21,EXT1,EXT2,FLAG)
C -----------------------------------------------------------------------
C***  DETERMINE TYPE OF VALIDATION WANTED
C -----------------------------------------------------------------------
   10 WRITE(*,20)
   20 FORMAT(//,5X,'DO YOU WANT 1) HOLE-BY-HOLE SUPPRESSION, 2) EVERY-OT
     1HER-SAMPLE',/,5X,'OR 3) SELECTED-ROW VALIDATION?',/,
     220X,'ANSWER 1, 2, OR 3 : '\)
      CALL ERRCHK(1,IFLAG,DUM)
   30 FORMAT(BN,I4)
      IF(IFLAG.GE.1.AND.IFLAG.LE.3) GO TO 40
      CALL SOUND(256,100)
      GO TO 10
   40 IF(IFLAG.NE.3) GO TO 70
   50 WRITE(*,60)
   60 FORMAT(5X,'ENTER STARTING ROW NUMBER OF LOCATIONS TO BE VALIDATED
     1: ',\)
      CALL ERRCHK(1,ISROW,DUM)
      GO TO(70,120), IRRR
C -----------------------------------------------------------------------
C***  CHECK IF UNIVERSAL KRIGING IS WANTED AND GET POSSIBLE DRIFT TERMS
C***  FROM KEYBOARD.  SEE COMMENTS IN DRFTRM.FOR.
C -----------------------------------------------------------------------
   70 CALL DRFTRM(0)
C -----------------------------------------------------------------------
C***  GET VARIOGRAM PARAMETERS FROM KEYBOARD.  SEE VARIN4.FOR COMMENTS.
C -----------------------------------------------------------------------
      CALL VARIN4
C -----------------------------------------------------------------------
C***  GET NEIGHBORHOOD LIMITS (SEARCH RADIUS AND MAXIMUM NUMBER OF
C***  SAMPLE POINTS TO BE USED) FROM KEYBOARD
C -----------------------------------------------------------------------
      CALL NEIGH(RMAX,NK,NT,MAXK)
      IF(NK.GT.MAXK) NK=MAXK
C -----------------------------------------------------------------------
C***
C***  GET DESCRIPTION OF STATPAC FILE FROM THE KEYBOARD AND THEN
C***  READ THE FILE FROM DISK.
C -----------------------------------------------------------------------
      CALL GETFLE(IGD)
C -----------------------------------------------------------------------
C***  WRITE OUT HEADER FOR VARIOGRAM(S) AND ANY DRIFT TERMS.
C -----------------------------------------------------------------------
      IF(FLAG) CALL HEADER(IPR,RMAX,NK,WKDAY,MDY,HMS)
C -----------------------------------------------------------------------
C***  WRITE HEADER ON SCREEN.
C***  DELAY FOR TWO SECONDS TO ALLOW USER TIME TO VIEW SCREEN.
C -----------------------------------------------------------------------
      CALL DELAY(200)
C -----------------------------------------------------------------------
C***  CLEAR THE SCREEN.
C -----------------------------------------------------------------------
      CALL CLEAR
      CALL HEADER(0,RMAX,NK,WKDAY,MDY,HMS)
C -----------------------------------------------------------------------
C***  WRITE HEADER FOR CROSS-VALIDATION RESULTS
C -----------------------------------------------------------------------
      IF(FLAG) WRITE(IPR,80)
   80 FORMAT(//,10X,'***OUTPUT OF MODEL VARIOGRAM TEST RESULTS***',//,
     11X,'HOLE',7X,'X-COORD.',2X,'Y-COORD.',3X,'ASSAY',2X,'KRIGED',
     22X,'KRIGING',2X,'KRIGING',2X,'ERROR/',/,2X,'NO.',28X,'VALUE',3X,
     3'VALUE',4X,'ERROR',2X,'STD.DEV',2X,'STD.DEV',/)
C -----------------------------------------------------------------------
C***  BRANCH ACCORDING TO TYPE OF CROSS-VALIDATION WANTED.
C -----------------------------------------------------------------------
      GO TO (90,100,120),IFLAG
C -----------------------------------------------------------------------
C***  SET UP LOOP INDEXING PARAMETERS FOR HOLE-BY-HOLE SUPPRESSION
C -----------------------------------------------------------------------
   90 IS1=1
      IEND1=NHOL
      INC=1
      IS2=1
      IEND2=NHOL
      GO TO 150
C -----------------------------------------------------------------------
C***  SET UP LOOP INDEXING PARAMETERS FOR SPLIT SAMPLE
C -----------------------------------------------------------------------
  100 IS1=1
      IEND1=NHOL-1
      INC=2
      IS2=2
      IEND2=NHOL
      IF(NHOL/2*2-NHOL) 110,150,150
  110 IEND1=NHOL
      IEND2=NHOL-1
      GO TO 150
C -----------------------------------------------------------------------
C***  SET UP LOOP INDEXING PARAMETERS FOR SELECTED ROWS
C -----------------------------------------------------------------------
  120 IF(ISROW.LE.NHOL) GO TO 140
      CALL SOUND(256,100)
      WRITE(*,130) ISROW,NHOL
  130 FORMAT(' ','THE STARTING ROW NUMBER THAT WAS SELECTED,',I5,
     1', IS GREATER THAN',/,' THE NUMBER OF DRILL HOLES THAT WERE'
     2,' OBTAINED FROM THE DATA FILE,',I5,/,' ','REENTER A LOWER'
     3,' STARTING ROW NUMBER OR BREAK.')
      IRRR=2
      GO TO 50
  140 INC=1
      IS1=ISROW
      IEND1=NHOL
      INC1=1
      IS2=1
      IEND2=ISROW-1
C -----------------------------------------------------------------------
C***  START THE KRIGING PROCESS
C -----------------------------------------------------------------------
  150 WRITE(*,'(A)') '       ....NOW KRIGING DATA POINTS'
      CALL FNDCUR(IROW,ICOL,0)
      R2=RMAX*RMAX
      KOUNT=0
      AE=0.
      ASE=0.
      E=0.
         DO 280 J=IS1,IEND1,INC
         B(1)=XE(J)
         B(2)=YN(J)
C -----------------------------------------------------------------------
C***  FIND NEAREST HOLES FROM POINT BEING KRIGED.  MAKE SURE
C***  THE NUMBER OF THESE HOLES, NS, IS NOT GREATER THAN NK.
C -----------------------------------------------------------------------
         CALL HLSRCH(B(1),B(2),NS,NK,R2,RMAX,IS2,IEND2,INC,J)
C -----------------------------------------------------------------------
C***  BE SURE THERE IS AT LEAST ONE DEGREE OF FREEDOM
C -----------------------------------------------------------------------
         IF(NS.GT.NT) GO TO 160
         IBYPAS=IBYPAS+1
         GO TO 280
C -----------------------------------------------------------------------
C***  KRIG THE POINT IF THERE ARE ENOUGH DRILL HOLES
C***  IN THE SEARCH RADIUS.
C***  GET KRIGING WEIGHTS WHICH WILL BE STORED IN ARRAY W
C -----------------------------------------------------------------------
  160    CALL KRIG4P(B,NS,IFERR)
         IF(IFERR.EQ.0) GO TO 200
         IF(FLAG) WRITE(IPR,170) J
  170    FORMAT(' ERROR IN MATRIX SOLUTION POINT ',I5)
            DO 190 M=1,NS
            LL=IHOL(M)
            IF(FLAG) WRITE(IPR,180) IHOLID(LL),YN(LL),XE(LL)
  180       FORMAT(1X,A8,1X,F10.2,1X,F10.2)
  190       CONTINUE
         ITOTER=ITOTER+1
         GO TO 280
C -----------------------------------------------------------------------
C***  COMPUTE THE KRIGED VALUE AND THE KRIGING VARIANCE
C***  KRIGED MEAN VALUE WILL BE STORED IN GR
C -----------------------------------------------------------------------
  200    CALL MNVAR(NS,SILL,GR,VARKG)
C -----------------------------------------------------------------------
C***  CHECK FOR NEGATIVE OR ZERO KRIGING VARIANCE
C -----------------------------------------------------------------------
         EPS=0.0001
         IF(VARKG) 210,220,260
  210    IF(EPS+VARKG) 230,220,220
  220    VARKG=EPS
         GO TO 260
  230    WRITE(*,240) J,IHOLID(J),YN(J),XE(J),VARKG
         IF(FLAG) WRITE(IPR,240) J,IHOLID(J),YN(J),XE(J),VARKG
  240    FORMAT(' NEGATIVE KRIGING VARIANCE AT POINT NO ',I4,/,
     1   ' HOLE NO ',A8,5X,'NORTH = ',F9.4,5X,'EAST = ',F9.4,5X,
     2   'VARKG = ',F12.4,//)
         NEGERR=NEGERR+1
            DO 250 M=1,NS
            LL=IHOL(M)
            IF(FLAG) WRITE(IPR,180) IHOLID(LL),YN(LL),XE(LL)
  250       CONTINUE
         GO TO 280
  260    SIGMA=SQRT(VARKG)
         DIFF=GD(J)-GR
         S=DIFF/SIGMA
         AE=AE+DIFF
         ASE=ASE+DIFF*DIFF
         E=E+S*S
         IF(FLAG) WRITE(IPR,270)IHOLID(J),B,GD(J),GR,DIFF,SIGMA,S
  270    FORMAT(1X,A8,1X,2F10.3,2F8.2,3(1X,F8.2))
         WRITE(ISCRCH)IHOLID(J),B,GD(J),GR,DIFF,SIGMA,S
         KOUNT=KOUNT+1
         IF(MOD(KOUNT,10).EQ.0) CALL ODOMTR(IROW,KOUNT)
  280    CONTINUE
      WRITE(*,'(''+'',I5)') KOUNT
C -----------------------------------------------------------------------
C***  CROSS-VALIDATION HAS BEEN COMPLETED.  COMPUTE SUMMARY STATISTICS
C -----------------------------------------------------------------------
      IF(KOUNT.EQ.0) GO TO 350
      XN=FLOAT(KOUNT)
      AE=AE/XN
      ASE=SQRT(ASE/XN)
      E=SQRT(E/XN)
      IF(FLAG) WRITE(IPR,290) AE,ASE,E
      WRITE(*,290) AE,ASE,E
  290 FORMAT(/,' KRIGED AVERAGE ERROR = ',F10.4,/,
     1' KRIGED ROOT MEAN SQUARE ERROR = ',F10.4,/,
     2' KRIGED REDUCED ROOT MEAN SQUARE ERROR = ',F10.4,/)
C -----------------------------------------------------------------------
C***  WRITE RESULTS ON STATPAC OUTPUT FILE
C -----------------------------------------------------------------------
      CALL OUTGEO(ISCRCH,MDY,IDSV,IGD,KOUNT,7,DUMMY,UNIT10,UNIT14,
     1UNIT21)
C -----------------------------------------------------------------------
C***  INFORM USER OF END OF PROGRAM AND PRINT OUT ANY MESSAGES
C***  CONCERNING PROBLEMS ENCOUNTERED IN THE CROSS-VALIDATION
C -----------------------------------------------------------------------
         DO 300 I=1,5
         CALL SOUND(1000,25)
  300    CONTINUE
      IF(NEGERR.NE.0) WRITE(*,310) NEGERR
  310 FORMAT('+','THERE WERE ',I4,' DATA POINTS WITH NEGATIVE KRIG',
     1'ING VARIANCES.',/,' THIS CAN BE THE RESULT OF USING AN UN',
     2'AUTHORIZED VARIOGRAM MODEL,',/,' OR HAVING MORE THAN 6',
     3' SIGNIFICANT DIGITS IN THE NORTH AND EAST COORDINATES.')
      IF(ITOTER.NE.0) WRITE(*,320) ITOTER
  320 FORMAT(' ','THERE WERE',I5,' DATA POINTS WHICH COULD NOT BE',
     1' PREDICTED BECAUSE OF',/,' ERRORS IN THE MATRIX SOLUTION.'
     2,'   EXAMINE THE CHARACTER OUTPUT FILE AND',/,' LOOK FOR'
     3,' DUPLICATED SAMPLE LOCATIONS.  REMOVE THESE DUPLICATES',/,
     4' AND START OVER WITH PROGRAM SS2DGAMH.')
      IF(IBYPAS.NE.0) WRITE(*,330) IBYPAS
  330 FORMAT(' ',I4,' DATA POINTS COULD NOT BE VALIDATED BECAUSE ',
     1'OF TOO FEW DATA ',/,' POINTS IN THE SEARCH RADIUS.',
     2'  YOU MAY WISH TO RERUN THIS PROGRAM',/,' USING A ',
     3'LARGER SEARCH RADIUS.')
      CALL TICKER(ITIME2)
      RTIME=(ITIME2-ITIME1)/6000.
      WRITE(*,340) RTIME
  340 FORMAT(' ','PROGRAM ACTIVE TIME WAS ',F6.2,' MINUTES')
      STOP 'NORMAL END OF PROGRAM'
  350 STOP 'NO POINTS COULD BE KRIGED'
      END
