
Program Tabran;
               {Generates a table of pseudo-random numbers
                with starting and ending values specified
                             by the user.
                Adapted by W. D. Grundy from FORTRAN Program
                TABRAN written by George Van Trump, 9/2/80}

Const
  Max = 10000;
  ON = TRUE;
  OFF = FALSE;
Type
  RegPack = record    {Note: this record construction serves the same}
    case byte of      {purpose as a FORTRAN equivalence statement}
      1 : (AX, BX, CX, DX, BP, SI, DI, DS, ES, Flags : integer);
      2 : (AL, AH, BL, BH, CL, CH, DL, DH : byte);
  end;
  ArrayElement = Integer;
  ArrayType = Array[1..Max] of ArrayElement;
Var
  New, Number    : ArrayType;
  Istart, Istop, Ntot, I, J, K, L, M, N : Integer;
  Icol, Irow : Byte;
  Ans : Char;
  OK, Again : Boolean;

procedure ChangeCursor(TurnOn : boolean); { from Turbo Tutor demo disk}
{ Uses interrupt $10 to turn the cursor on or off }
const
  OldScanStart : byte = 0;        { The saved values of the cursor scan lines }
  OldScanEnd : byte = 0;
var Reg : RegPack;           { Registers used for BIOS and DOS function calls }
begin
  with Reg do
  begin
    case TurnOn of
      ON : begin
             AH := 1;
             CH := OldScanStart;
             CL := OldScanEnd;
             Intr($10, Reg);                     { Turn cursor on }
           end;
     OFF : begin
             AH := 3;
             BH := 0;
             Intr($10, Reg);
             OldScanStart := CH;     { Save old cursor scan lines }
             OldScanEnd := CL;
             AH := 1;
             CH := 32;
             CL := 0;
             Intr($10, Reg);                     { Turn cursor off }
           end;
    end; { case }
  end; { with }
end; { ChangeCursor }

Procedure WriteTable(Istart, Istop, N, M : Integer; New, Number : ArrayType);
Var
  I, J, K, L : integer;
Begin
  Writeln('TABLE OF THE NUMBERS ',Istart,' THROUGH ',Istop,' RANDOMIZED:');
  Writeln; For I:= 1 to 7 Do Write('   No  Seq'); Writeln;
  For L := 0 to N - 1 Do Begin
    For K := 1 to 7 Do Begin
      J := L * 7 + K;
      Write(' ',Number[J]:4, New[J]:5);
    End; Writeln;
  End;
  L := N;
  For K := 1 to M Do Begin
    J := L * 7 + K;
    Write(' ',Number[J]:4, New[J]:5)
  End; Writeln; Writeln
End;  { of proc WriteTable}

Procedure Odometer(Icol, Irow : Byte; Kount : Integer);
Begin
  GotoXY(Icol, Irow); Write(Kount)
End;  { of proc Odometer}

Procedure ShellTagSort(Var New, Number : ArrayType; Ntot : Integer);
   { See Borland's Turbo Tutor Manual (1986) pp 267-268}
Label ExitLoop;
Var Indx, Jndx, Val, ValPtr, Inc : Integer;
Begin
  Inc := 1;
  Repeat
    Inc := 3 * Inc + 1;
    Until Inc > Ntot;
    Repeat
      Inc := Inc Div 3;
      For Indx := Inc + 1 to Ntot Do Begin
        Val := New[Indx];
        ValPtr := Number[Indx];
        Jndx := Indx;
        While New[Jndx - Inc] > Val Do Begin
        New[Jndx] := New[Jndx - Inc];
        Number[Jndx] := Number[Jndx -Inc];
        Jndx := Jndx - Inc;
        If Jndx <= Inc Then GOTO ExitLoop
      End;
ExitLoop:
      New[Jndx] := Val;
      Number[Jndx] := ValPtr
    End
  Until Inc = 1
End; { of proc ShellTagSort}

Begin             { beginning of program Tabran}
  Again := TRUE;
  While Again Do Begin

    Repeat
      Writeln; Write('ENTER STARTING NO. OF SEQUENCE (1-',Max-1,'): ');
      {$I-} Readln(Istart); {$I+}
    Until (IOResult = 0) and (Istart >=1) and (Istart <= (Max-1));

    Repeat
      Write('ENTER ENDING NO. (MIN = ',Istart+1,', MAX = ',Max,'): ');
      {$I-} Readln(Istop); {$I+}
    Until (IOResult = 0) and (Istop >= Istart+1) and (Istop <= Max);

    Randomize; Ntot := Istop - Istart + 1;
    For I := 1 to Ntot Do Number[I] := I + Istart - 1;
    Irow := WhereY; Icol := 1;
    Writeln; Write('       ....Computing random numbers');
    ChangeCursor(OFF);
    For I := 1 to Ntot Do
      Begin
        J := Ntot + 1 - I;
        K :=  Random(32767) Mod J + 1;
        New[I] := Number[K];
        L := K + 1;
        If L <= J then For M := L to J Do Number[M -1] := Number[M];
        If (I Mod 100 = 0) Then Odometer(Icol, Irow, I);
      End;
    Odometer(Icol, Irow, Ntot); GotoXY(Icol, Irow); ChangeCursor(ON);

    N := Ntot Div 7;
    M := Ntot Mod 7;
    For I := 1 to Ntot Do Number[I] := I;
    WriteTable(Istart, Istop, N, M, New, Number);

    Write('WANT INVERSE OF ABOVE TABLE PRINTED ? ');
    Readln(Ans);
    If (Ans in ['Y','y']) Then
      Begin
        ShellTagSort(New, Number, Ntot);
        Writeln; WriteTable(Istart, Istop, N, M, New, Number);
      End;

    Write('DO YOU WANT TO PROCESS MORE DATA ? ');
    Readln(Ans);
    Again := Ans in ['Y','y']
  End;  { end of While Again Do loop}
End.  { end of program Tabran}

