$NOFLOATCALLS
$STORAGE:2
      SUBROUTINE VARIN4
C***
C***  THIS SUBROUTINE WAS WRITTEN BY W.D. GRUNDY  JANUARY 1981
C***  MODIFIED JULY 1985 TO INCORPORATE FORTRAN 77 FEATURES
C***
C***  THIS SUBROUTINE INPUTS THE VARIOGRAM PARAMETERS
C***  AND COMPUTES GEOMETRIC ANISOTROPY CORRECTION FACTORS FOR
C***  PROGRAMS SS2DXVAL, SS2DBLOK, SS2DGRID, AND SS2DDRES.
C***  SEE JOURNEL AND HUIJBREGTS (1978,PP. 175-183)
C***
C***                   HOW TO ENTER ANISOTROPY
C***
C***  FOR GEOMETRIC ANISOTROPY:
C***
C***  1) WITH PENCIL AND GRAPH PAPER, CONSTRUCT ELLIPTICAL VECTOR
C***     DIAGRAM OF VARIOGRAM RANGES IN DIFFERENT DIRECTIONS.  (SEE
C***     JOURNEL AND HUIJBREGTS, 1978, P. 180)
C***  2) DETERMINE PARAMETERS OF VARIOGRAM IN DIRECTION PARALLEL
C***     TO THE MAJOR AXIS OF THE ELLIPSE (NUGGET, C-VALUE, A-VALUE)
C***  3) MEASURE BEARING IN DEGREES OF THE MAJOR AXIS OF THE ELLIPSE
C***     (EAST IS TAKEN AS 0 DEGREES; A COUNTER-CLOCKWISE DIRECTION
C***     IS POSITIVE; A CLOCKWISE DIRECTION IS NEGATIVE.)
C***  4) CALCULATE THE RATIO OF THE LENGTH OF THE MAJOR AXIS TO THE
C***     LENGTH OF THE MINOR AXIS.  THIS IS THE ANISOTROPY RATIO
C***     AND SHOULD BE GREATER THAN ONE.
C***  5) WHEN REQUESTED BY PROGRAMS SS2DXVAL, SS2DBLOK, SS2DGRID, OR
C***     SS2DDRES, ENTER THE VARIOGRAM PARAMETERS FROM STEP 2 ABOVE.
C***     INDICATE TO THE PROGRAM THAT THE ANISOTROPY IS GEOMETRIC,
C***     AND ENTER THE BEARING OF THE MAJOR AXIS OF THE ELLIPSE OB-
C***     TAINED IN STEP 3 ABOVE.  NEXT ENTER THE ANISOTROPY RATIO
C***     OBTAINED IN STEP 4 ABOVE.
C***
C***
C**********************************************************************
C***
C***  NOTE: THIS PROGRAM REQUESTS A C-VALUE FOR A LINEAR VARIOGRAM,
C***  ALTHOUGH A LINEAR VARIOGRAM CANNOT HAVE A C-VALUE AS IT HAS
C***  NO SILL.  THE C-VALUE IN THIS CASE IS A PSEUDO-PARAMETER WHICH
C***  IS INTRODUCED TO ALLOW USE OF AN EFFICIENT SUBPROGRAM FOR
C***  SOLVING FOR KRIGING WEIGHTS.  (SEE FOOTNOTE ON PAGE 306 OF
C***  JOURNEL AND HUIJBREGTS).  WHEN USING A LINEAR VARIOGRAM, THE
C***  SEARCH RADIUS MUST BE DEFINED SO THAT NO TWO POINTS INVOLVED
C***  IN COVARIANCE CALCULATIONS ARE FARTHER APART THAN THE DISTANCE
C***  DEFINED FROM THE ORIGIN OF THE VARIOGRAM TO THE POINT WHERE
C***  THE LINEAR SEGMENT INTERSECTS THE PSEUDO-SILL.
C***
C***  NOTE: THIS SUBROUTINE MUST USE THE FORTRAN 77 DO LOOP SEMANTICS
C***
$INCLUDE:'PTKRIG.INC'
C***  INITIALIZE VARIOGRAM PARAMETERS
      DO 10 I=1,5
      C(I)=0.
      ANGLE(I)=0.
      AFH(I)=1.
      AVALU(I)=0.
      ITYPE(I)=0
      SLOPE(I)=0.
   10 CONTINUE
      C0=0.
      WRITE(*,'(//,5X,''ENTER PARAMETERS OF THE VARIOGRAM(S)'',//)')
   20 WRITE(*,'(5X,''ENTER THE NUGGET VALUE : '',\)')
      CALL ERRCHK(2,IVAL,C0)
   30 FORMAT(BN,F10.0)
      IF(C0.GE.0) GO TO 40
      CALL SOUND(256,100)
      WRITE(*,'('' NUGGET VALUE MUST NOT BE NEGATIVE'')')
      WRITE(*,60)
      GO TO 20
   40 WRITE(*,'(5X,''HOW MANY OTHER VARIOGRAMS ARE IN THE MODEL ?'',
     1'' (MAX=5) : '',\)')
      CALL ERRCHK(1,NEST,DUM)
   50 FORMAT(BN,I2)
      IF(NEST.LT.0.OR.NEST.GT.5) THEN
      CALL SOUND(256,100)
      WRITE(*,'('' ANSWER MUST BE 0,1,2,3,4,OR 5'')')
      GO TO 40
      ENDIF
      IF(C0.EQ.0..AND.NEST.EQ.0) THEN
      CALL SOUND(256,100)
      WRITE(*,'('' THE NUGGET VALUE AND THE NUMBER OF OTHER'',
     1'' VARIOGRAMS CANNOT BOTH'',/,'' BE EQUAL TO ZERO.  '',
     2''REDO OR BREAK.'')')
      WRITE(*,60)
      GO TO 20
      ENDIF
      WRITE(*,60)
   60 FORMAT(//)
C***  IF OTHER VARIOGRAMS ARE WANTED, EXECUTE THE FOLLOWING DO LOOP
      DO 180 I=1,NEST
   70 WRITE(*,'(5X,''IS VARIOGRAM NO. '',I1,'' (1) SPHERICAL, '',
     1''(2) EXPONENTIAL,'',/,5X,''(3) LINEAR, (4) GAUSSIAN, OR '',
     2''(5) CUBIC?'',/,20X,''ANSWER 1, 2, 3, 4, OR 5 : '',\)') I
      CALL ERRCHK(1,ITYPE(I),DUM)
      IF(ITYPE(I).GE.1.AND.ITYPE(I).LE.5) GO TO 80
      CALL SOUND(256,100)
      GO TO  70
   80 WRITE(*,'(5X,''ENTER C-VALUE OF VARIOGRAM NO '',I1,'' : '',
     1\)') I
      CALL ERRCHK(2,IVAL,C(I))
      IF(C(I).GT.0.) GO TO 90
      CALL SOUND(256,100)
      WRITE(*,'('' C-VALUE MUST BE GREATER THAN ZERO'')')
      GO TO 80
   90 WRITE(*,'(5X,''ENTER A-VALUE OF VARIOGRAM NO '',I1,'' : '',
     1\)') I
      CALL ERRCHK(2,IVAL,AVALU(I))
      IF(AVALU(I).GT.0.) GO TO 100
      CALL SOUND(256,100)
      WRITE(*,'('' A-VALUE MUST BE GREATER THAN ZERO'')')
      GO TO 90
  100 IF(ITYPE(I).NE.3) GO TO 120
      IF(AVALU(I).LT.2.) GO TO 110
      CALL SOUND(256,100)
      WRITE(*,'(5X,''FOR LINEAR VARIOGRAM A-VALUE MUST BE GREATER '',
     1''THAN ZERO AND LESS THAN TWO'')')
      GO TO 90
  110 WRITE(*,'(5X,''ENTER SLOPE OF LINEAR VARIOGRAM NO. '',I1,'' : '',
     1\)') I
      CALL ERRCHK(2,IVAL,SLOPE(I))
      IF(SLOPE(I).GT.0.) GO TO 120
      CALL SOUND(256,100)
      WRITE(*,'('' SLOPE OF LINEAR VARIOGRAM MUST BE GREATER THAN '',
     1''ZERO'')')
      GO TO 110
  120 WRITE(*,'(5X,''IS VARIOGRAM NO '',I1,'' (1) ISOTROPIC OR (2) '',
     1''ANISOTROPIC ?'',/,20X,''ANSWER 1 OR 2 : '',\)') I
      CALL ERRCHK(1,IANS,DUM)
      IF(IANS.EQ.1) GO TO 170
      IF(IANS.NE.2) THEN
      CALL SOUND(256,100)
      GO TO 120
      ENDIF
  150 WRITE(*,'(5X,''ENTER ANISOTROPY ANGLE OF VARIOGRAM NO. '',I1,
     1'' (DEC. DEG) : '',\)') I
      CALL ERRCHK(2,IVAL,ANGLE(I))
  160 WRITE(*,'(5X,''ENTER ANISOTROPY FACTOR OF VARIOGRAM NO. '',I1,
     1''  : '',\)') I
      CALL ERRCHK(2,IVAL,AFH(I))
      IF(AFH(I).GT.0.) GO TO 170
      CALL SOUND(256,100)
      WRITE(*,'('' ANISOTROPY FACTOR MUST BE GREATER THAN ZERO'')')
      GO TO 160
  170 WRITE(*,60)
  180 CONTINUE
C***  COMPUTE SILL VALUE
      SILL=C0
      DO 190 I=1,NEST
      SILL=SILL+C(I)
  190 CONTINUE
C***  COMPUTE ANISOTROPY CORRECTION FACTORS
      DO 200 I=1,NEST
C***  TEMPORARILY CHANGE SIGN OF ANGLE OF ANISOTROPY
      ANGLE(I)=-ANGLE(I)
      SINANG=SIN(ANGLE(I)*1.7453292D-02)
      COSANG=COS(ANGLE(I)*1.7453292D-02)
      AISI(I)=COSANG**2+AFH(I)*SINANG**2
      BISI(I)=SINANG**2+AFH(I)*COSANG**2
      CISI(I)=(1.-AFH(I))*SINANG*COSANG
      ANGLE(I)=-ANGLE(I)
  200 CONTINUE
      RETURN
      END
